@@ -16,6 +16,7 @@
 */
 #include <array>
 #include "sdf/Camera.hh"
+#include "sdf/parser.hh"
 #include "Utils.hh"
 
 using namespace sdf;
@@ -54,6 +55,12 @@ class sdf::Camera::Implementation
   /// \brief Name of the camera.
   public: std::string name = "";
 
+  /// \brief True if the camera is triggered by a topic.
+  public: bool triggered{false};
+
+  /// \brief Camera trigger topic.
+  public: std::string triggerTopic = "";
+
   /// \brief Horizontal fied of view.
   public: ignition::math::Angle hfov{1.047};
 
@@ -66,6 +73,9 @@ class sdf::Camera::Implementation
   /// \brief Image format.
   public: PixelFormatType pixelFormat{PixelFormatType::RGB_INT8};
 
+  /// \brief Anti-aliasing value.
+  public: uint32_t antiAliasingValue{4};
+
   /// \brief Near clip distance.
   public: double nearClip{0.1};
 
@@ -78,6 +88,12 @@ class sdf::Camera::Implementation
   /// \brief Far clip distance for depth camera.
   public: double depthFarClip{10.0};
 
+  /// \brief Segmentation type for segmentation camera.
+  public: std::string segmentationType{"semantic"};
+
+  /// \brief Boundingbox type for boundingbox camera.
+  public: std::string boundingBoxType{"2d"};
+
   /// \brief True indicates the depth camera was set.
   public: bool hasDepthCamera{false};
 
@@ -87,6 +103,12 @@ class sdf::Camera::Implementation
   /// \brief True indicates the depth camera near clip distance was set.
   public: bool hasDepthNearClip{false};
 
+  /// \brief True indicates the segmentation type was set.
+  public: bool hasSegmentationType{false};
+
+  /// \brief True indicates the boundingobx type was set.
+  public: bool hasBoundingBoxType{false};
+
   /// \brief True indicates frames should be saved.
   public: bool save{false};
 
@@ -162,8 +184,11 @@ class sdf::Camera::Implementation
   /// \brief lens instrinsics s.
   public: double lensIntrinsicsS{1.0};
 
+  /// \brief True if this camera has custom intrinsics values
+  public: bool hasIntrinsics = false;
+
   /// \brief Visibility mask of a camera. Defaults to 0xFFFFFFFF
-  public: uint32_t visibilityMask{4294967295u};
+  public: uint32_t visibilityMask{UINT32_MAX};
 };
 
 /////////////////////////////////////////////////
@@ -200,6 +225,12 @@ Errors Camera::Load(ElementPtr _sdf)
   // Read the camera's name
   loadName(_sdf, this->dataPtr->name);
 
+  this->dataPtr->triggered = _sdf->Get<bool>("triggered",
+      this->dataPtr->triggered).first;
+
+  this->dataPtr->triggerTopic = _sdf->Get<std::string>("trigger_topic",
+      this->dataPtr->triggerTopic).first;
+
   this->dataPtr->hfov = _sdf->Get<ignition::math::Angle>("horizontal_fov",
       this->dataPtr->hfov).first;
 
@@ -238,6 +269,10 @@ Errors Camera::Load(ElementPtr _sdf)
       errors.push_back({ErrorCode::ELEMENT_INVALID,
         "Camera sensor <image><format> has invalid value of " + format});
     }
+
+    this->dataPtr->antiAliasingValue =
+        elem->Get<uint32_t>("anti_aliasing",
+            this->dataPtr->antiAliasingValue).first;
   }
   else
   {
@@ -277,6 +312,16 @@ Errors Camera::Load(ElementPtr _sdf)
         "Camera sensor is missing a <clip> element."});
   }
 
+  if (_sdf->HasElement("segmentation_type"))
+  {
+    this->SetSegmentationType(_sdf->Get<std::string>("segmentation_type"));
+  }
+
+  if (_sdf->HasElement("box_type"))
+  {
+    this->SetBoundingBoxType(_sdf->Get<std::string>("box_type"));
+  }
+
   if (_sdf->HasElement("save"))
   {
     sdf::ElementPtr elem = _sdf->GetElement("save");
@@ -345,6 +390,7 @@ Errors Camera::Load(ElementPtr _sdf)
           this->dataPtr->lensIntrinsicsCy).first;
       this->dataPtr->lensIntrinsicsS = intrinsics->Get<double>("s",
           this->dataPtr->lensIntrinsicsS).first;
+      this->dataPtr->hasIntrinsics = true;
     }
   }
 
@@ -375,6 +421,32 @@ void Camera::SetName(const std::string &_name)
   this->dataPtr->name = _name;
 }
 
+/////////////////////////////////////////////////
+bool Camera::Triggered() const
+{
+  return this->dataPtr->triggered;
+}
+
+
+/////////////////////////////////////////////////
+void Camera::SetTriggered(bool _triggered)
+{
+  this->dataPtr->triggered = _triggered;
+}
+
+/////////////////////////////////////////////////
+std::string Camera::TriggerTopic() const
+{
+  return this->dataPtr->triggerTopic;
+}
+
+
+/////////////////////////////////////////////////
+void Camera::SetTriggerTopic(const std::string &_triggerTopic)
+{
+  this->dataPtr->triggerTopic = _triggerTopic;
+}
+
 /////////////////////////////////////////////////
 ignition::math::Angle Camera::HorizontalFov() const
 {
@@ -435,6 +507,18 @@ void Camera::SetPixelFormatStr(const std::string &_fmt)
   this->dataPtr->pixelFormat = ConvertPixelFormat(_fmt);
 }
 
+//////////////////////////////////////////////////
+uint32_t Camera::AntiAliasingValue() const
+{
+  return this->dataPtr->antiAliasingValue;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetAntiAliasingValue(uint32_t _antiAliasingValue)
+{
+  this->dataPtr->antiAliasingValue = _antiAliasingValue;
+}
+
 //////////////////////////////////////////////////
 double Camera::DepthNearClip() const
 {
@@ -485,6 +569,56 @@ void Camera::SetFarClip(double _far)
   this->dataPtr->farClip = _far;
 }
 
+//////////////////////////////////////////////////
+const std::string &Camera::SegmentationType() const
+{
+  return this->dataPtr->segmentationType;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetSegmentationType(const std::string &_type)
+{
+  this->dataPtr->hasSegmentationType = true;
+  this->dataPtr->segmentationType = _type;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetHasSegmentationType(bool _type)
+{
+  this->dataPtr->hasSegmentationType = _type;
+}
+
+//////////////////////////////////////////////////
+bool Camera::HasSegmentationType() const
+{
+  return this->dataPtr->hasSegmentationType;
+}
+
+//////////////////////////////////////////////////
+const std::string &Camera::BoundingBoxType() const
+{
+  return this->dataPtr->boundingBoxType;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetBoundingBoxType(const std::string &_type)
+{
+  this->dataPtr->hasBoundingBoxType = true;
+  this->dataPtr->boundingBoxType = _type;
+}
+
+//////////////////////////////////////////////////
+void Camera::SetHasBoundingBoxType(bool _type)
+{
+  this->dataPtr->hasBoundingBoxType = _type;
+}
+
+//////////////////////////////////////////////////
+bool Camera::HasBoundingBoxType() const
+{
+  return this->dataPtr->hasBoundingBoxType;
+}
+
 //////////////////////////////////////////////////
 void Camera::SetHasDepthCamera(bool _camera)
 {
@@ -793,6 +927,7 @@ double Camera::LensIntrinsicsFx() const
 void Camera::SetLensIntrinsicsFx(double _fx)
 {
   this->dataPtr->lensIntrinsicsFx = _fx;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -805,6 +940,7 @@ double Camera::LensIntrinsicsFy() const
 void Camera::SetLensIntrinsicsFy(double _fy)
 {
   this->dataPtr->lensIntrinsicsFy = _fy;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -817,6 +953,7 @@ double Camera::LensIntrinsicsCx() const
 void Camera::SetLensIntrinsicsCx(double _cx)
 {
   this->dataPtr->lensIntrinsicsCx = _cx;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -829,6 +966,7 @@ double Camera::LensIntrinsicsCy() const
 void Camera::SetLensIntrinsicsCy(double _cy)
 {
   this->dataPtr->lensIntrinsicsCy = _cy;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -841,6 +979,7 @@ double Camera::LensIntrinsicsSkew() const
 void Camera::SetLensIntrinsicsSkew(double _s)
 {
   this->dataPtr->lensIntrinsicsS = _s;
+  this->dataPtr->hasIntrinsics = true;
 }
 
 /////////////////////////////////////////////////
@@ -896,3 +1035,119 @@ void Camera::SetVisibilityMask(uint32_t _mask)
 {
   this->dataPtr->visibilityMask = _mask;
 }
+
+/////////////////////////////////////////////////
+bool Camera::HasLensIntrinsics() const
+{
+  return this->dataPtr->hasIntrinsics;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Camera::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("camera.sdf", elem);
+
+  elem->GetAttribute("name")->Set<std::string>(this->Name());
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
+  elem->GetElement("horizontal_fov")->Set<double>(
+      this->HorizontalFov().Radian());
+  sdf::ElementPtr imageElem = elem->GetElement("image");
+  imageElem->GetElement("width")->Set<double>(this->ImageWidth());
+  imageElem->GetElement("height")->Set<double>(this->ImageHeight());
+  imageElem->GetElement("format")->Set<std::string>(this->PixelFormatStr());
+  imageElem->GetElement("anti_aliasing")->Set<uint32_t>(
+      this->AntiAliasingValue());
+
+  sdf::ElementPtr clipElem = elem->GetElement("clip");
+  clipElem->GetElement("near")->Set<double>(this->NearClip());
+  clipElem->GetElement("far")->Set<double>(this->FarClip());
+
+  if (this->dataPtr->hasDepthCamera)
+  {
+    sdf::ElementPtr depthElem = elem->GetElement("depth_camera");
+    sdf::ElementPtr depthClipElem = depthElem->GetElement("clip");
+    depthClipElem->GetElement("near")->Set<double>(this->DepthNearClip());
+    depthClipElem->GetElement("far")->Set<double>(this->DepthFarClip());
+  }
+
+  sdf::ElementPtr saveElem = elem->GetElement("save");
+  saveElem->GetAttribute("enabled")->Set<bool>(this->SaveFrames());
+  if (this->SaveFrames())
+    saveElem->GetElement("path")->Set<std::string>(this->SaveFramesPath());
+  elem->GetElement("visibility_mask")->Set<uint32_t>(
+      this->VisibilityMask());
+
+  sdf::ElementPtr noiseElem = elem->GetElement("noise");
+  std::string noiseType;
+  switch (this->dataPtr->imageNoise.Type())
+  {
+    case sdf::NoiseType::NONE:
+      noiseType = "none";
+      break;
+    case sdf::NoiseType::GAUSSIAN:
+      noiseType = "gaussian";
+      break;
+    case sdf::NoiseType::GAUSSIAN_QUANTIZED:
+      noiseType = "gaussian_quantized";
+      break;
+    default:
+      noiseType = "none";
+  }
+
+  // camera does not use noise.sdf description
+  noiseElem->GetElement("type")->Set<std::string>(noiseType);
+  noiseElem->GetElement("mean")->Set<double>(this->dataPtr->imageNoise.Mean());
+  noiseElem->GetElement("stddev")->Set<double>(
+      this->dataPtr->imageNoise.StdDev());
+
+  sdf::ElementPtr distortionElem = elem->GetElement("distortion");
+  distortionElem->GetElement("k1")->Set<double>(this->DistortionK1());
+  distortionElem->GetElement("k2")->Set<double>(this->DistortionK2());
+  distortionElem->GetElement("k3")->Set<double>(this->DistortionK3());
+  distortionElem->GetElement("p1")->Set<double>(this->DistortionP1());
+  distortionElem->GetElement("p2")->Set<double>(this->DistortionP2());
+  distortionElem->GetElement("center")->Set<ignition::math::Vector2d>(
+      this->DistortionCenter());
+
+  sdf::ElementPtr lensElem = elem->GetElement("lens");
+  lensElem->GetElement("type")->Set<std::string>(this->LensType());
+  lensElem->GetElement("scale_to_hfov")->Set<bool>(this->LensScaleToHfov());
+  lensElem->GetElement("cutoff_angle")->Set<double>(
+     this->LensCutoffAngle().Radian());
+  lensElem->GetElement("env_texture_size")->Set<double>(
+      this->LensEnvironmentTextureSize());
+  if (this->LensType() == "custom")
+  {
+    sdf::ElementPtr customLensElem = lensElem->GetElement("custom_function");
+    customLensElem->GetElement("c1")->Set<double>(this->LensC1());
+    customLensElem->GetElement("c2")->Set<double>(this->LensC2());
+    customLensElem->GetElement("c3")->Set<double>(this->LensC3());
+    customLensElem->GetElement("f")->Set<double>(this->LensFocalLength());
+    customLensElem->GetElement("fun")->Set<std::string>(
+        this->LensFunction());
+  }
+  if (this->HasLensIntrinsics())
+  {
+    sdf::ElementPtr intrinsicsElem = lensElem->GetElement("intrinsics");
+    intrinsicsElem->GetElement("fx")->Set<double>(this->LensIntrinsicsFx());
+    intrinsicsElem->GetElement("fy")->Set<double>(this->LensIntrinsicsFy());
+    intrinsicsElem->GetElement("cx")->Set<double>(this->LensIntrinsicsCx());
+    intrinsicsElem->GetElement("cy")->Set<double>(this->LensIntrinsicsCy());
+    intrinsicsElem->GetElement("s")->Set<double>(this->LensIntrinsicsSkew());
+  }
+
+  if (this->HasSegmentationType())
+  {
+    elem->GetElement("segmentation_type")->Set<std::string>(
+        this->SegmentationType());
+  }
+
+  return elem;
+}
