@@ -26,9 +26,13 @@ TEST(DOMCamera, Construction)
   cam.SetName("my_camera");
   EXPECT_EQ("my_camera", cam.Name());
 
-  EXPECT_EQ("", cam.CameraInfoTopic());
-  cam.SetCameraInfoTopic("/camera/camera_info");
-  EXPECT_EQ("/camera/camera_info", cam.CameraInfoTopic());
+  EXPECT_FALSE(cam.Triggered());
+  cam.SetTriggered(true);
+  EXPECT_TRUE(cam.Triggered());
+
+  EXPECT_TRUE(cam.TriggerTopic().empty());
+  cam.SetTriggerTopic("my_camera/trigger");
+  EXPECT_EQ("my_camera/trigger", cam.TriggerTopic());
 
   EXPECT_DOUBLE_EQ(1.047, cam.HorizontalFov().Radian());
   cam.SetHorizontalFov(1.45);
@@ -70,6 +74,22 @@ TEST(DOMCamera, Construction)
   cam.SetFarClip(200.2);
   EXPECT_DOUBLE_EQ(200.2, cam.FarClip());
 
+  EXPECT_EQ("semantic", cam.SegmentationType());
+  EXPECT_FALSE(cam.HasSegmentationType());
+  cam.SetSegmentationType("panoptic");
+  EXPECT_TRUE(cam.HasSegmentationType());
+  EXPECT_EQ("panoptic", cam.SegmentationType());
+  cam.SetHasSegmentationType(false);
+  EXPECT_FALSE(cam.HasSegmentationType());
+
+  EXPECT_EQ("2d", cam.BoundingBoxType());
+  EXPECT_FALSE(cam.HasBoundingBoxType());
+  cam.SetBoundingBoxType("3d");
+  EXPECT_TRUE(cam.HasBoundingBoxType());
+  EXPECT_EQ("3d", cam.BoundingBoxType());
+  cam.SetHasBoundingBoxType(false);
+  EXPECT_FALSE(cam.HasBoundingBoxType());
+
   EXPECT_FALSE(cam.SaveFrames());
   cam.SetSaveFrames(true);
   EXPECT_TRUE(cam.SaveFrames());
@@ -98,13 +118,13 @@ TEST(DOMCamera, Construction)
   cam.SetDistortionP2(0.2);
   EXPECT_DOUBLE_EQ(0.2, cam.DistortionP2());
 
-  EXPECT_EQ(gz::math::Vector2d(0.5, 0.5), cam.DistortionCenter());
-  cam.SetDistortionCenter(gz::math::Vector2d(0.1, 0.2));
-  EXPECT_EQ(gz::math::Vector2d(0.1, 0.2), cam.DistortionCenter());
+  EXPECT_EQ(ignition::math::Vector2d(0.5, 0.5), cam.DistortionCenter());
+  cam.SetDistortionCenter(ignition::math::Vector2d(0.1, 0.2));
+  EXPECT_EQ(ignition::math::Vector2d(0.1, 0.2), cam.DistortionCenter());
 
-  EXPECT_EQ(gz::math::Pose3d::Zero, cam.RawPose());
-  cam.SetRawPose(gz::math::Pose3d(1, 2, 3, 0, 0, 0));
-  EXPECT_EQ(gz::math::Pose3d(1, 2, 3, 0, 0, 0), cam.RawPose());
+  EXPECT_EQ(ignition::math::Pose3d::Zero, cam.RawPose());
+  cam.SetRawPose(ignition::math::Pose3d(1, 2, 3, 0, 0, 0));
+  EXPECT_EQ(ignition::math::Pose3d(1, 2, 3, 0, 0, 0), cam.RawPose());
 
   EXPECT_TRUE(cam.PoseRelativeTo().empty());
   cam.SetPoseRelativeTo("/frame");
@@ -162,36 +182,13 @@ TEST(DOMCamera, Construction)
   cam.SetLensIntrinsicsCy(123);
   EXPECT_DOUBLE_EQ(123, cam.LensIntrinsicsCy());
 
-  EXPECT_DOUBLE_EQ(277, cam.LensProjectionFx());
-  cam.SetLensProjectionFx(132);
-  EXPECT_DOUBLE_EQ(132, cam.LensProjectionFx());
-
-  EXPECT_DOUBLE_EQ(277, cam.LensProjectionFy());
-  cam.SetLensProjectionFy(456);
-  EXPECT_DOUBLE_EQ(456, cam.LensProjectionFy());
-
-  EXPECT_DOUBLE_EQ(160, cam.LensProjectionCx());
-  cam.SetLensProjectionCx(254);
-  EXPECT_DOUBLE_EQ(254, cam.LensProjectionCx());
-
-  EXPECT_DOUBLE_EQ(120, cam.LensProjectionCy());
-  cam.SetLensProjectionCy(123);
-  EXPECT_DOUBLE_EQ(123, cam.LensProjectionCy());
-
-  EXPECT_DOUBLE_EQ(0, cam.LensProjectionTx());
-  cam.SetLensProjectionTx(1);
-  EXPECT_DOUBLE_EQ(1, cam.LensProjectionTx());
-
-  EXPECT_DOUBLE_EQ(0, cam.LensProjectionTy());
-  cam.SetLensProjectionTy(2);
-  EXPECT_DOUBLE_EQ(2, cam.LensProjectionTy());
-
   EXPECT_DOUBLE_EQ(1.0, cam.LensIntrinsicsSkew());
   cam.SetLensIntrinsicsSkew(2.3);
   EXPECT_DOUBLE_EQ(2.3, cam.LensIntrinsicsSkew());
 
-  EXPECT_EQ(4294967295u, cam.VisibilityMask());
-  EXPECT_TRUE(cam.HasLensProjection());
+  EXPECT_TRUE(cam.HasLensIntrinsics());
+
+  EXPECT_EQ(UINT32_MAX, cam.VisibilityMask());
   cam.SetVisibilityMask(123u);
   EXPECT_EQ(123u, cam.VisibilityMask());
 
@@ -226,3 +223,48 @@ TEST(DOMCamera, Construction)
   // The Camera::Load function is tested more thoroughly in the
   // link_dom.cc integration test.
 }
+
+/////////////////////////////////////////////////
+TEST(DOMCamera, ToElement)
+{
+  sdf::Camera cam;
+  cam.SetName("my_camera");
+  EXPECT_EQ("my_camera", cam.Name());
+  cam.SetHorizontalFov(1.45);
+  cam.SetImageWidth(123);
+  cam.SetImageHeight(125);
+  cam.SetPixelFormat(sdf::PixelFormatType::L_INT8);
+  cam.SetNearClip(0.2);
+  cam.SetFarClip(200.2);
+  cam.SetVisibilityMask(123u);
+  cam.SetPoseRelativeTo("/frame");
+  cam.SetSaveFrames(true);
+  cam.SetSaveFramesPath("/tmp");
+
+  sdf::ElementPtr camElem = cam.ToElement();
+  EXPECT_NE(nullptr, camElem);
+  EXPECT_EQ(nullptr, cam.Element());
+
+  // verify values after loading the element back
+  sdf::Camera cam2;
+  cam2.Load(camElem);
+
+  EXPECT_DOUBLE_EQ(1.45, cam2.HorizontalFov().Radian());
+  EXPECT_EQ(123u, cam2.ImageWidth());
+  EXPECT_EQ(125u, cam2.ImageHeight());
+  EXPECT_EQ(sdf::PixelFormatType::L_INT8 , cam2.PixelFormat());
+  EXPECT_DOUBLE_EQ(0.2, cam2.NearClip());
+  EXPECT_DOUBLE_EQ(200.2, cam2.FarClip());
+  EXPECT_EQ(123u, cam2.VisibilityMask());
+  EXPECT_EQ("/frame", cam2.PoseRelativeTo());
+  EXPECT_TRUE(cam2.SaveFrames());
+  EXPECT_EQ("/tmp", cam2.SaveFramesPath());
+
+  // make changes to DOM and verify ToElement produces updated values
+  cam2.SetNearClip(0.33);
+  sdf::ElementPtr cam2Elem = cam2.ToElement();
+  EXPECT_NE(nullptr, cam2Elem);
+  sdf::Camera cam3;
+  cam3.Load(cam2Elem);
+  EXPECT_DOUBLE_EQ(0.33, cam3.NearClip());
+}
