@@ -91,11 +91,9 @@ TEST(Element, SetExplicitlySetInFile)
 
   sdf::ElementPtr parent = std::make_shared<sdf::Element>();
   sdf::ElementPtr elem = std::make_shared<sdf::Element>();
-  elem->SetParent(parent);
-  parent->InsertElement(elem);
+  parent->InsertElement(elem, true);
   sdf::ElementPtr elem2 = std::make_shared<sdf::Element>();
-  elem2->SetParent(parent);
-  parent->InsertElement(elem2);
+  parent->InsertElement(elem2, true);
 
   EXPECT_TRUE(elem->GetExplicitlySetInFile());
 
@@ -111,7 +109,7 @@ TEST(Element, SetExplicitlySetInFile)
   // set to the same value when using this function
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
   child->SetParent(elem);
-  elem->InsertElement(child);
+  elem->InsertElement(child, false);
 
   sdf::ElementPtr sibling = std::make_shared<sdf::Element>();
   sibling->SetParent(elem);
@@ -191,40 +189,42 @@ TEST(Element, ReferenceSDF)
 /////////////////////////////////////////////////
 TEST(Element, AddValue)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  elem.SetName("test");
-  elem.AddValue("string", "foo", false, "foo description");
+  elem->SetName("test");
+  elem->AddValue("string", "foo", false, "foo description");
 
-  sdf::ParamPtr param = elem.GetValue();
+  sdf::ParamPtr param = elem->GetValue();
   ASSERT_TRUE(param->IsType<std::string>());
   ASSERT_EQ(param->GetKey(), "test");
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), elem);
 }
 
 /////////////////////////////////////////////////
 TEST(Element, AddAttribute)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
 
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  ASSERT_EQ(elem.GetAttributeCount(), 1UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  ASSERT_EQ(elem->GetAttributeCount(), 1UL);
 
-  elem.AddAttribute("attr", "float", "0.0", false, "float description");
-  ASSERT_EQ(elem.GetAttributeCount(), 2UL);
+  elem->AddAttribute("attr", "float", "0.0", false, "float description");
+  ASSERT_EQ(elem->GetAttributeCount(), 2UL);
 
-  sdf::ParamPtr param = elem.GetAttribute("test");
+  sdf::ParamPtr param = elem->GetAttribute("test");
   ASSERT_TRUE(param->IsType<std::string>());
   ASSERT_EQ(param->GetKey(), "test");
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
 
-  param = elem.GetAttribute("attr");
+  param = elem->GetAttribute("attr");
   ASSERT_TRUE(param->IsType<float>());
   ASSERT_EQ(param->GetKey(), "attr");
   ASSERT_EQ(param->GetTypeName(), "float");
@@ -235,48 +235,48 @@ TEST(Element, AddAttribute)
 /////////////////////////////////////////////////
 TEST(Element, GetAttributeSet)
 {
-  sdf::Element elem;
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  ASSERT_EQ(elem.GetAttributeCount(), 1UL);
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  ASSERT_EQ(elem->GetAttributeCount(), 1UL);
 
-  EXPECT_FALSE(elem.GetAttributeSet("test"));
-  elem.GetAttribute("test")->Set("asdf");
-  EXPECT_TRUE(elem.GetAttributeSet("test"));
+  EXPECT_FALSE(elem->GetAttributeSet("test"));
+  elem->GetAttribute("test")->Set("asdf");
+  EXPECT_TRUE(elem->GetAttributeSet("test"));
 }
 
 /////////////////////////////////////////////////
 TEST(Element, RemoveAttribute)
 {
-  sdf::Element elem;
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
 
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  elem.AddAttribute("attr", "float", "0.0", false, "float description");
-  ASSERT_EQ(elem.GetAttributeCount(), 2UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  elem->AddAttribute("attr", "float", "0.0", false, "float description");
+  ASSERT_EQ(elem->GetAttributeCount(), 2UL);
 
-  elem.RemoveAttribute("test");
-  EXPECT_EQ(elem.GetAttributeCount(), 1UL);
-  EXPECT_EQ(elem.GetAttribute("test"), nullptr);
-  EXPECT_NE(elem.GetAttribute("attr"), nullptr);
+  elem->RemoveAttribute("test");
+  EXPECT_EQ(elem->GetAttributeCount(), 1UL);
+  EXPECT_EQ(elem->GetAttribute("test"), nullptr);
+  EXPECT_NE(elem->GetAttribute("attr"), nullptr);
 }
 
 /////////////////////////////////////////////////
 TEST(Element, RemoveAllAttributes)
 {
-  sdf::Element elem;
-  ASSERT_EQ(elem.GetAttributeCount(), 0UL);
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
+  ASSERT_EQ(elem->GetAttributeCount(), 0UL);
 
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  elem.AddAttribute("test2", "string", "foo", false, "foo description");
-  elem.AddAttribute("attr", "float", "0.0", false, "float description");
-  ASSERT_EQ(elem.GetAttributeCount(), 3UL);
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  elem->AddAttribute("test2", "string", "foo", false, "foo description");
+  elem->AddAttribute("attr", "float", "0.0", false, "float description");
+  ASSERT_EQ(elem->GetAttributeCount(), 3UL);
 
-  elem.RemoveAllAttributes();
-  EXPECT_EQ(elem.GetAttributeCount(), 0UL);
-  EXPECT_EQ(elem.GetAttribute("test"), nullptr);
-  EXPECT_EQ(elem.GetAttribute("test2"), nullptr);
-  EXPECT_EQ(elem.GetAttribute("attr"), nullptr);
+  elem->RemoveAllAttributes();
+  EXPECT_EQ(elem->GetAttributeCount(), 0UL);
+  EXPECT_EQ(elem->GetAttribute("test"), nullptr);
+  EXPECT_EQ(elem->GetAttribute("test2"), nullptr);
+  EXPECT_EQ(elem->GetAttribute("attr"), nullptr);
 }
 
 /////////////////////////////////////////////////
@@ -284,14 +284,6 @@ TEST(Element, Include)
 {
   sdf::Element elem;
 
-  SDF_SUPPRESS_DEPRECATED_BEGIN
-  ASSERT_EQ(elem.GetInclude(), "");
-
-  elem.SetInclude("foo.txt");
-
-  ASSERT_EQ(elem.GetInclude(), "foo.txt");
-  SDF_SUPPRESS_DEPRECATED_END
-
   auto includeElemToStore = std::make_shared<sdf::Element>();
   includeElemToStore->SetName("include");
   auto uriDesc = std::make_shared<sdf::Element>();
@@ -405,6 +397,9 @@ TEST(Element, Clone)
   sdf::ElementPtr child = std::make_shared<sdf::Element>();
   sdf::ElementPtr desc = std::make_shared<sdf::Element>();
 
+  parent->SetName("parent");
+  child->SetName("child");
+
   parent->InsertElement(child);
   ASSERT_NE(parent->GetFirstElement(), nullptr);
 
@@ -438,6 +433,15 @@ TEST(Element, Clone)
   ASSERT_NE(newelem->GetIncludeElement(), nullptr);
   EXPECT_EQ("include", newelem->GetIncludeElement()->GetName());
   ASSERT_TRUE(newelem->GetExplicitlySetInFile());
+
+  ASSERT_NE(nullptr, parent->GetValue()->GetParentElement());
+  EXPECT_EQ(parent, parent->GetValue()->GetParentElement());
+
+  ASSERT_NE(nullptr, newelem->GetValue()->GetParentElement());
+  EXPECT_EQ(newelem, newelem->GetValue()->GetParentElement());
+
+  auto clonedAttribs = newelem->GetAttributes();
+  EXPECT_EQ(newelem, clonedAttribs[0]->GetParentElement());
 }
 
 /////////////////////////////////////////////////
@@ -601,23 +605,6 @@ TEST(Element, ToStringClonedElement)
   EXPECT_EQ(parent->ToString("myprefix"), parentClone->ToString("myprefix"));
 }
 
-/////////////////////////////////////////////////
-TEST(Element, ToStringInclude)
-{
-  sdf::Element elem;
-
-  SDF_SUPPRESS_DEPRECATED_BEGIN
-  ASSERT_EQ(elem.GetInclude(), "");
-
-  elem.SetInclude("foo.txt");
-
-  ASSERT_EQ(elem.GetInclude(), "foo.txt");
-  SDF_SUPPRESS_DEPRECATED_END
-
-  std::string stringval = elem.ToString("myprefix");
-  ASSERT_EQ(stringval, "myprefix<include filename='foo.txt'/>\n");
-}
-
 /////////////////////////////////////////////////
 TEST(Element, ToStringDefaultElements)
 {
@@ -708,16 +695,16 @@ TEST(Element, DocLeftPane)
 /////////////////////////////////////////////////
 TEST(Element, DocRightPane)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  elem.SetDescription("Element description");
-  elem.AddAttribute("test", "string", "foo", false, "foo description");
-  elem.AddValue("string", "val", false, "val description");
-  elem.AddElementDescription(std::make_shared<sdf::Element>());
+  elem->SetDescription("Element description");
+  elem->AddAttribute("test", "string", "foo", false, "foo description");
+  elem->AddValue("string", "val", false, "val description");
+  elem->AddElementDescription(std::make_shared<sdf::Element>());
 
   std::string html;
   int index = 1;
-  elem.PrintDocRightPane(html, 0, index);
+  elem->PrintDocRightPane(html, 0, index);
   ASSERT_EQ(html,
             "<a name=\"1\">&lt&gt</a><div style='padding-left:0px;'>\n"
             "<div style='background-color: #ffffff'>\n"
@@ -766,11 +753,11 @@ TEST(Element, SetEmpty)
 /////////////////////////////////////////////////
 TEST(Element, Set)
 {
-  sdf::Element elem;
+  sdf::ElementPtr elem = std::make_shared<sdf::Element>();
 
-  elem.AddValue("string", "val", false, "val description");
+  elem->AddValue("string", "val", false, "val description");
 
-  ASSERT_TRUE(elem.Set<std::string>("hello"));
+  ASSERT_TRUE(elem->Set<std::string>("hello"));
 }
 
 /////////////////////////////////////////////////
@@ -806,6 +793,8 @@ TEST(Element, Copy)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "val");
   ASSERT_EQ(param->GetDescription(), "val description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_EQ(dest->GetAttributeCount(), 1UL);
   ASSERT_TRUE(dest->GetExplicitlySetInFile());
@@ -815,6 +804,8 @@ TEST(Element, Copy)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_NE(dest->GetFirstElement(), nullptr);
   ASSERT_NE(dest->GetIncludeElement(), nullptr);
@@ -840,6 +831,8 @@ TEST(Element, CopyDestValue)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "val");
   ASSERT_EQ(param->GetDescription(), "val description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_EQ(dest->GetAttributeCount(), 1UL);
   param = dest->GetAttribute("test");
@@ -848,6 +841,8 @@ TEST(Element, CopyDestValue)
   ASSERT_EQ(param->GetTypeName(), "string");
   ASSERT_EQ(param->GetDefaultAsString(), "foo");
   ASSERT_EQ(param->GetDescription(), "foo description");
+  ASSERT_NE(param->GetParentElement(), nullptr);
+  EXPECT_EQ(param->GetParentElement(), dest);
 
   ASSERT_NE(dest->GetFirstElement(), nullptr);
 }
