@@ -15,6 +15,7 @@
  *
  */
 #include "sdf/Lidar.hh"
+#include "sdf/parser.hh"
 
 using namespace sdf;
 using namespace ignition;
@@ -60,6 +61,9 @@ class sdf::Lidar::Implementation
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf{nullptr};
+
+  /// \brief Visibility mask of a lidar. Defaults to 0xFFFFFFFF
+  public: uint32_t visibilityMask{UINT32_MAX};
 };
 
 //////////////////////////////////////////////////
@@ -172,6 +176,12 @@ Errors Lidar::Load(ElementPtr _sdf)
   if (_sdf->HasElement("noise"))
     this->dataPtr->lidarNoise.Load(_sdf->GetElement("noise"));
 
+  if (_sdf->HasElement("visibility_mask"))
+  {
+    this->dataPtr->visibilityMask = _sdf->Get<uint32_t>("visibility_mask",
+        this->dataPtr->visibilityMask).first;
+  }
+
   return errors;
 }
 
@@ -325,6 +335,18 @@ void Lidar::SetLidarNoise(const Noise &_noise)
   this->dataPtr->lidarNoise = _noise;
 }
 
+/////////////////////////////////////////////////
+uint32_t Lidar::VisibilityMask() const
+{
+  return this->dataPtr->visibilityMask;
+}
+
+/////////////////////////////////////////////////
+void Lidar::SetVisibilityMask(uint32_t _mask)
+{
+  this->dataPtr->visibilityMask = _mask;
+}
+
 //////////////////////////////////////////////////
 bool Lidar::operator==(const Lidar &_lidar) const
 {
@@ -357,6 +379,8 @@ bool Lidar::operator==(const Lidar &_lidar) const
   }
   if (this->dataPtr->lidarNoise != _lidar.LidarNoise())
     return false;
+  if (this->dataPtr->visibilityMask != _lidar.VisibilityMask())
+    return false;
 
   return true;
 }
@@ -366,3 +390,63 @@ bool Lidar::operator!=(const Lidar &_lidar) const
 {
   return !(*this == _lidar);
 }
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Lidar::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("lidar.sdf", elem);
+
+  sdf::ElementPtr scanElem = elem->GetElement("scan");
+  sdf::ElementPtr horElem = scanElem->GetElement("horizontal");
+  horElem->GetElement("samples")->Set<double>(this->HorizontalScanSamples());
+  horElem->GetElement("resolution")->Set<double>(
+      this->HorizontalScanResolution());
+  horElem->GetElement("min_angle")->Set<double>(
+      this->HorizontalScanMinAngle().Radian());
+  horElem->GetElement("max_angle")->Set<double>(
+      this->HorizontalScanMaxAngle().Radian());
+
+  sdf::ElementPtr vertElem = scanElem->GetElement("vertical");
+  vertElem->GetElement("samples")->Set<double>(this->VerticalScanSamples());
+  vertElem->GetElement("resolution")->Set<double>(
+      this->VerticalScanResolution());
+  vertElem->GetElement("min_angle")->Set<double>(
+      this->VerticalScanMinAngle().Radian());
+  vertElem->GetElement("max_angle")->Set<double>(
+      this->VerticalScanMaxAngle().Radian());
+
+  sdf::ElementPtr rangeElem = elem->GetElement("range");
+  rangeElem->GetElement("min")->Set<double>(this->RangeMin());
+  rangeElem->GetElement("max")->Set<double>(this->RangeMax());
+  rangeElem->GetElement("resolution")->Set<double>(
+      this->RangeResolution());
+
+  sdf::ElementPtr noiseElem = elem->GetElement("noise");
+  std::string noiseType;
+  switch (this->dataPtr->lidarNoise.Type())
+  {
+    case sdf::NoiseType::NONE:
+      noiseType = "none";
+      break;
+    case sdf::NoiseType::GAUSSIAN:
+      noiseType = "gaussian";
+      break;
+    case sdf::NoiseType::GAUSSIAN_QUANTIZED:
+      noiseType = "gaussian_quantized";
+      break;
+    default:
+      noiseType = "none";
+  }
+
+  // lidar does not use noise.sdf description
+  noiseElem->GetElement("type")->Set<std::string>(noiseType);
+  noiseElem->GetElement("mean")->Set<double>(this->dataPtr->lidarNoise.Mean());
+  noiseElem->GetElement("stddev")->Set<double>(
+      this->dataPtr->lidarNoise.StdDev());
+
+  elem->GetElement("visibility_mask")->Set<uint32_t>(
+      this->VisibilityMask());
+
+  return elem;
+}
