@@ -17,9 +17,9 @@
 #include <memory>
 #include <string>
 #include <vector>
-#include <gz/math/Inertial.hh>
-#include <gz/math/Pose3.hh>
-#include <gz/math/Vector3.hh>
+#include <ignition/math/Inertial.hh>
+#include <ignition/math/Pose3.hh>
+#include <ignition/math/Vector3.hh>
 
 #include "sdf/Collision.hh"
 #include "sdf/Error.hh"
@@ -43,7 +43,7 @@ class sdf::Link::Implementation
   public: std::string name = "";
 
   /// \brief Pose of the link
-  public: gz::math::Pose3d pose = gz::math::Pose3d::Zero;
+  public: ignition::math::Pose3d pose = ignition::math::Pose3d::Zero;
 
   /// \brief Frame of the pose.
   public: std::string poseRelativeTo = "";
@@ -64,9 +64,9 @@ class sdf::Link::Implementation
   public: std::vector<ParticleEmitter> emitters;
 
   /// \brief The inertial information for this link.
-  public: gz::math::Inertiald inertial {{1.0,
-            gz::math::Vector3d::One, gz::math::Vector3d::Zero},
-            gz::math::Pose3d::Zero};
+  public: ignition::math::Inertiald inertial {{1.0,
+            ignition::math::Vector3d::One, ignition::math::Vector3d::Zero},
+            ignition::math::Pose3d::Zero};
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -74,16 +74,13 @@ class sdf::Link::Implementation
   /// \brief True if this link should be subject to wind, false otherwise.
   public: bool enableWind = false;
 
-  /// \brief True if this link should be subject to gravity, false otherwise.
-  public: bool enableGravity = true;
-
   /// \brief Scoped Pose Relative-To graph at the parent model scope.
   public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
 };
 
 /////////////////////////////////////////////////
 Link::Link()
-  : dataPtr(gz::utils::MakeImpl<Implementation>())
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
@@ -148,9 +145,9 @@ Errors Link::Load(ElementPtr _sdf)
   errors.insert(errors.end(), emitterLoadErrors.begin(),
       emitterLoadErrors.end());
 
-  gz::math::Vector3d xxyyzz = gz::math::Vector3d::One;
-  gz::math::Vector3d xyxzyz = gz::math::Vector3d::Zero;
-  gz::math::Pose3d inertiaPose;
+  ignition::math::Vector3d xxyyzz = ignition::math::Vector3d::One;
+  ignition::math::Vector3d xyxzyz = ignition::math::Vector3d::Zero;
+  ignition::math::Pose3d inertiaPose;
   std::string inertiaFrame = "";
   double mass = 1.0;
 
@@ -178,7 +175,7 @@ Errors Link::Load(ElementPtr _sdf)
     }
   }
   if (!this->dataPtr->inertial.SetMassMatrix(
-      gz::math::MassMatrix3d(mass, xxyyzz, xyxzyz)))
+      ignition::math::MassMatrix3d(mass, xxyyzz, xyxzyz)))
   {
     errors.push_back({ErrorCode::LINK_INERTIA_INVALID,
                      "A link named " +
@@ -192,9 +189,6 @@ Errors Link::Load(ElementPtr _sdf)
   this->dataPtr->enableWind = _sdf->Get<bool>("enable_wind",
       this->dataPtr->enableWind).first;
 
-  this->dataPtr->enableGravity = _sdf->Get<bool>("gravity",
-      this->dataPtr->enableGravity).first;
-
   return errors;
 }
 
@@ -415,35 +409,20 @@ ParticleEmitter *Link::ParticleEmitterByName(const std::string &_name)
 }
 
 /////////////////////////////////////////////////
-const gz::math::Inertiald &Link::Inertial() const
+const ignition::math::Inertiald &Link::Inertial() const
 {
   return this->dataPtr->inertial;
 }
 
 /////////////////////////////////////////////////
-bool Link::SetInertial(const gz::math::Inertiald &_inertial)
+bool Link::SetInertial(const ignition::math::Inertiald &_inertial)
 {
   this->dataPtr->inertial = _inertial;
   return _inertial.MassMatrix().IsValid();
 }
 
 /////////////////////////////////////////////////
-Errors Link::ResolveInertial(
-  gz::math::Inertiald &_inertial,
-  const std::string &_resolveTo) const
-{
-  gz::math::Pose3d linkPose;
-  auto errors = this->SemanticPose().Resolve(linkPose, _resolveTo);
-  if (errors.empty())
-  {
-    _inertial = this->dataPtr->inertial;
-    _inertial.SetPose(linkPose * _inertial.Pose());
-  }
-  return errors;
-}
-
-/////////////////////////////////////////////////
-const gz::math::Pose3d &Link::RawPose() const
+const ignition::math::Pose3d &Link::RawPose() const
 {
   return this->dataPtr->pose;
 }
@@ -455,7 +434,7 @@ const std::string &Link::PoseRelativeTo() const
 }
 
 /////////////////////////////////////////////////
-void Link::SetRawPose(const gz::math::Pose3d &_pose)
+void Link::SetRawPose(const ignition::math::Pose3d &_pose)
 {
   this->dataPtr->pose = _pose;
 }
@@ -585,23 +564,11 @@ bool Link::EnableWind() const
 }
 
 /////////////////////////////////////////////////
-bool Link::EnableGravity() const
-{
-  return this->dataPtr->enableGravity;
-}
-
-/////////////////////////////////////////////////
-void Link::SetEnableWind(bool _enableWind)
+void Link::SetEnableWind(const bool _enableWind)
 {
   this->dataPtr->enableWind = _enableWind;
 }
 
-/////////////////////////////////////////////////
-void Link::SetEnableGravity(bool _enableGravity)
-{
-  this->dataPtr->enableGravity = _enableGravity;
-}
-
 //////////////////////////////////////////////////
 bool Link::AddCollision(const Collision &_collision)
 {
@@ -692,13 +659,13 @@ sdf::ElementPtr Link::ToElement() const
     poseElem->GetAttribute("relative_to")->Set<std::string>(
         this->dataPtr->poseRelativeTo);
   }
-  poseElem->Set<gz::math::Pose3d>(this->RawPose());
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
 
   // inertial
   sdf::ElementPtr inertialElem = elem->GetElement("inertial");
   inertialElem->GetElement("pose")->Set(
       this->dataPtr->inertial.Pose());
-  const gz::math::MassMatrix3d &massMatrix =
+  const ignition::math::MassMatrix3d &massMatrix =
     this->dataPtr->inertial.MassMatrix();
   inertialElem->GetElement("mass")->Set<double>(massMatrix.Mass());
   sdf::ElementPtr inertiaElem = inertialElem->GetElement("inertia");
