@@ -15,17 +15,19 @@
  *
 */
 #include <string>
+#include <optional>
 #include <vector>
-#include <gz/math/Vector3.hh>
+#include <ignition/math/Vector3.hh>
 
-#include "sdf/Types.hh"
 #include "sdf/Material.hh"
+#include "sdf/parser.hh"
 #include "sdf/Pbr.hh"
+#include "sdf/Types.hh"
 #include "Utils.hh"
 
 using namespace sdf;
 
-class sdf::MaterialPrivate
+class sdf::Material::Implementation
 {
   /// \brief Script URI
   public: std::string scriptUri = "";
@@ -46,22 +48,25 @@ class sdf::MaterialPrivate
   public: bool doubleSided = false;
 
   /// \brief Ambient color
-  public: gz::math::Color ambient {0, 0, 0, 1};
+  public: ignition::math::Color ambient {0, 0, 0, 1};
 
   /// \brief Diffuse color
-  public: gz::math::Color diffuse {0, 0, 0, 1};
+  public: ignition::math::Color diffuse {0, 0, 0, 1};
 
   /// \brief Specular color
-  public: gz::math::Color specular {0, 0, 0, 1};
+  public: ignition::math::Color specular {0, 0, 0, 1};
 
   /// \brief Specular exponent
   public: double shininess {0};
 
   /// \brief Emissive color
-  public: gz::math::Color emissive {0, 0, 0, 1};
+  public: ignition::math::Color emissive {0, 0, 0, 1};
+
+  /// \brief Render order
+  public: float renderOrder = 0;
 
   /// \brief Physically Based Rendering (PBR) properties
-  public: std::unique_ptr<Pbr> pbr;
+  public: std::optional<Pbr> pbr;
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -72,57 +77,10 @@ class sdf::MaterialPrivate
 
 /////////////////////////////////////////////////
 Material::Material()
-  : dataPtr(new MaterialPrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Material::~Material()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Material::Material(const Material &_material)
-  : dataPtr(new MaterialPrivate)
-{
-  this->dataPtr->scriptUri = _material.dataPtr->scriptUri;
-  this->dataPtr->scriptName = _material.dataPtr->scriptName;
-  this->dataPtr->shader = _material.dataPtr->shader;
-  this->dataPtr->normalMap = _material.dataPtr->normalMap;
-  this->dataPtr->lighting = _material.dataPtr->lighting;
-  this->dataPtr->doubleSided = _material.dataPtr->doubleSided;
-  this->dataPtr->ambient = _material.dataPtr->ambient;
-  this->dataPtr->diffuse = _material.dataPtr->diffuse;
-  this->dataPtr->specular = _material.dataPtr->specular;
-  this->dataPtr->shininess = _material.dataPtr->shininess;
-  this->dataPtr->emissive = _material.dataPtr->emissive;
-  this->dataPtr->sdf = _material.dataPtr->sdf;
-  this->dataPtr->filePath = _material.dataPtr->filePath;
-  if (_material.dataPtr->pbr)
-    this->dataPtr->pbr = std::make_unique<Pbr>(*_material.dataPtr->pbr);
-}
-
-/////////////////////////////////////////////////
-Material::Material(Material &&_material) noexcept
-  : dataPtr(std::exchange(_material.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Material &Material::operator=(const Material &_material)
-{
-  return *this = Material(_material);
-}
-
-/////////////////////////////////////////////////
-Material &Material::operator=(Material &&_material)
-{
-  std::swap(this->dataPtr, _material.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Material::Load(sdf::ElementPtr _sdf)
 {
@@ -211,19 +169,22 @@ Errors Material::Load(sdf::ElementPtr _sdf)
     }
   }
 
-  this->dataPtr->ambient = _sdf->Get<gz::math::Color>("ambient",
+  this->dataPtr->renderOrder = _sdf->Get<float>("render_order",
+      this->dataPtr->renderOrder).first;
+
+  this->dataPtr->ambient = _sdf->Get<ignition::math::Color>("ambient",
       this->dataPtr->ambient).first;
 
-  this->dataPtr->diffuse = _sdf->Get<gz::math::Color>("diffuse",
+  this->dataPtr->diffuse = _sdf->Get<ignition::math::Color>("diffuse",
       this->dataPtr->diffuse).first;
 
-  this->dataPtr->specular = _sdf->Get<gz::math::Color>("specular",
+  this->dataPtr->specular = _sdf->Get<ignition::math::Color>("specular",
       this->dataPtr->specular).first;
 
   this->dataPtr->shininess = _sdf->Get<double>("shininess",
       this->dataPtr->shininess).first;
 
-  this->dataPtr->emissive = _sdf->Get<gz::math::Color>("emissive",
+  this->dataPtr->emissive = _sdf->Get<ignition::math::Color>("emissive",
       this->dataPtr->emissive).first;
 
   this->dataPtr->lighting = _sdf->Get<bool>("lighting",
@@ -235,7 +196,7 @@ Errors Material::Load(sdf::ElementPtr _sdf)
   // load pbr param
   if (_sdf->HasElement("pbr"))
   {
-    this->dataPtr->pbr.reset(new sdf::Pbr());
+    this->dataPtr->pbr.emplace();
     Errors pbrErrors = this->dataPtr->pbr->Load(_sdf->GetElement("pbr"));
     errors.insert(errors.end(), pbrErrors.begin(), pbrErrors.end());
   }
@@ -244,37 +205,37 @@ Errors Material::Load(sdf::ElementPtr _sdf)
 }
 
 //////////////////////////////////////////////////
-gz::math::Color Material::Ambient() const
+ignition::math::Color Material::Ambient() const
 {
   return this->dataPtr->ambient;
 }
 
 //////////////////////////////////////////////////
-void Material::SetAmbient(const gz::math::Color &_color) const
+void Material::SetAmbient(const ignition::math::Color &_color)
 {
   this->dataPtr->ambient = _color;
 }
 
 //////////////////////////////////////////////////
-gz::math::Color Material::Diffuse() const
+ignition::math::Color Material::Diffuse() const
 {
   return this->dataPtr->diffuse;
 }
 
 //////////////////////////////////////////////////
-void Material::SetDiffuse(const gz::math::Color &_color) const
+void Material::SetDiffuse(const ignition::math::Color &_color)
 {
   this->dataPtr->diffuse = _color;
 }
 
 //////////////////////////////////////////////////
-gz::math::Color Material::Specular() const
+ignition::math::Color Material::Specular() const
 {
   return this->dataPtr->specular;
 }
 
 //////////////////////////////////////////////////
-void Material::SetSpecular(const gz::math::Color &_color) const
+void Material::SetSpecular(const ignition::math::Color &_color)
 {
   this->dataPtr->specular = _color;
 }
@@ -292,17 +253,29 @@ void Material::SetShininess(const double _shininess)
 }
 
 //////////////////////////////////////////////////
-gz::math::Color Material::Emissive() const
+ignition::math::Color Material::Emissive() const
 {
   return this->dataPtr->emissive;
 }
 
 //////////////////////////////////////////////////
-void Material::SetEmissive(const gz::math::Color &_color) const
+void Material::SetEmissive(const ignition::math::Color &_color)
 {
   this->dataPtr->emissive = _color;
 }
 
+//////////////////////////////////////////////////
+float Material::RenderOrder() const
+{
+  return this->dataPtr->renderOrder;
+}
+
+//////////////////////////////////////////////////
+void Material::SetRenderOrder(const float _renderOrder)
+{
+  this->dataPtr->renderOrder = _renderOrder;
+}
+
 //////////////////////////////////////////////////
 bool Material::Lighting() const
 {
@@ -384,13 +357,13 @@ void Material::SetNormalMap(const std::string &_map)
 //////////////////////////////////////////////////
 void Material::SetPbrMaterial(const Pbr &_pbr)
 {
-  this->dataPtr->pbr.reset(new Pbr(_pbr));
+  this->dataPtr->pbr = _pbr;
 }
 
 //////////////////////////////////////////////////
-Pbr *Material::PbrMaterial() const
+const Pbr *Material::PbrMaterial() const
 {
-  return this->dataPtr->pbr.get();
+  return optionalToPointer(this->dataPtr->pbr);
 }
 
 //////////////////////////////////////////////////
@@ -404,3 +377,107 @@ void Material::SetFilePath(const std::string &_filePath)
 {
   this->dataPtr->filePath = _filePath;
 }
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Material::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("material.sdf", elem);
+
+  elem->GetElement("ambient")->Set(this->Ambient());
+  elem->GetElement("diffuse")->Set(this->Diffuse());
+  elem->GetElement("specular")->Set(this->Specular());
+  elem->GetElement("emissive")->Set(this->Emissive());
+  elem->GetElement("render_order")->Set(this->RenderOrder());
+  elem->GetElement("lighting")->Set(this->Lighting());
+  elem->GetElement("double_sided")->Set(this->DoubleSided());
+
+  // Script, if set
+  if (!this->ScriptName().empty() && !this->ScriptUri().empty())
+  {
+    sdf::ElementPtr scriptElem = elem->GetElement("script");
+    scriptElem->GetElement("uri")->Set(this->ScriptUri());
+    scriptElem->GetElement("name")->Set(this->ScriptName());
+  }
+
+  // Shader properties
+  sdf::ElementPtr shaderElem = elem->GetElement("shader");
+  switch (this->dataPtr->shader)
+  {
+    default:
+    case ShaderType::PIXEL:
+      shaderElem->GetAttribute("type")->Set("pixel");
+      break;
+    case ShaderType::VERTEX:
+      shaderElem->GetAttribute("type")->Set("vertex");
+      break;
+    case ShaderType::NORMAL_MAP_OBJECTSPACE:
+      shaderElem->GetAttribute("type")->Set("normal_map_object_space");
+      break;
+    case ShaderType::NORMAL_MAP_TANGENTSPACE:
+      shaderElem->GetAttribute("type")->Set("normal_map_tangent_space");
+      break;
+  }
+  if (!this->NormalMap().empty())
+    shaderElem->GetElement("normal_map")->Set(this->NormalMap());
+
+  // PBR material
+  if (this->dataPtr->pbr)
+  {
+    const PbrWorkflow *workflow = this->dataPtr->pbr->Workflow(
+        PbrWorkflowType::METAL);
+    sdf::ElementPtr pbrElem = elem->GetElement("pbr");
+    if (workflow && workflow->Type() == PbrWorkflowType::METAL)
+    {
+      sdf::ElementPtr metalElem = pbrElem->GetElement("metal");
+      metalElem->GetElement("albedo_map")->Set(workflow->AlbedoMap());
+      metalElem->GetElement("roughness_map")->Set(workflow->RoughnessMap());
+      metalElem->GetElement("roughness")->Set(workflow->Roughness());
+      metalElem->GetElement("metalness_map")->Set(workflow->MetalnessMap());
+      metalElem->GetElement("metalness")->Set(workflow->Metalness());
+      metalElem->GetElement("ambient_occlusion_map")->Set(
+          workflow->AmbientOcclusionMap());
+      sdf::ElementPtr normalElem = metalElem->GetElement("normal_map");
+      if (workflow->NormalMapType() == NormalMapSpace::TANGENT)
+        normalElem->GetAttribute("type")->Set("tangent");
+      else
+        normalElem->GetAttribute("type")->Set("object");
+      normalElem->Set(workflow->NormalMap());
+
+      metalElem->GetElement("emissive_map")->Set(workflow->EmissiveMap());
+
+      sdf::ElementPtr lightElem = metalElem->GetElement("light_map");
+      lightElem->GetAttribute("uv_set")->Set(workflow->LightMapTexCoordSet());
+      lightElem->Set(workflow->LightMap());
+    }
+
+    workflow = this->dataPtr->pbr->Workflow(PbrWorkflowType::SPECULAR);
+    if (workflow && workflow->Type() == PbrWorkflowType::SPECULAR)
+    {
+      sdf::ElementPtr specularElem = pbrElem->GetElement("specular");
+      specularElem->GetElement("albedo_map")->Set(workflow->AlbedoMap());
+      specularElem->GetElement("specular_map")->Set(workflow->SpecularMap());
+      specularElem->GetElement("environment_map")->Set(
+          workflow->EnvironmentMap());
+      specularElem->GetElement("ambient_occlusion_map")->Set(
+          workflow->AmbientOcclusionMap());
+      specularElem->GetElement("emissive_map")->Set(workflow->EmissiveMap());
+      specularElem->GetElement("glossiness_map")->Set(
+          workflow->GlossinessMap());
+      specularElem->GetElement("glossiness")->Set(workflow->Glossiness());
+
+      sdf::ElementPtr normalElem = specularElem->GetElement("normal_map");
+      if (workflow->NormalMapType() == NormalMapSpace::TANGENT)
+        normalElem->GetAttribute("type")->Set("tangent");
+      else
+        normalElem->GetAttribute("type")->Set("object");
+      normalElem->Set(workflow->NormalMap());
+
+      sdf::ElementPtr lightElem = specularElem->GetElement("light_map");
+      lightElem->GetAttribute("uv_set")->Set(workflow->LightMapTexCoordSet());
+      lightElem->Set(workflow->LightMap());
+    }
+  }
+
+  return elem;
+}
