@@ -23,6 +23,7 @@
 #include "sdf/Error.hh"
 #include "sdf/Frame.hh"
 #include "sdf/InterfaceElements.hh"
+#include "sdf/InterfaceLink.hh"
 #include "sdf/InterfaceModel.hh"
 #include "sdf/InterfaceModelPoseGraph.hh"
 #include "sdf/Joint.hh"
@@ -80,8 +81,15 @@ class sdf::Model::Implementation
   public: std::vector<Model> models;
 
   /// \brief The interface models specified in this model.
-  public: std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelPtr>>
-              interfaceModels;
+  public: std::vector<std::pair<std::optional<sdf::NestedInclude>,
+          sdf::InterfaceModelConstPtr>> interfaceModels;
+
+  /// \brief The interface models added into in this model via merge include.
+  public: std::vector<std::pair<std::optional<sdf::NestedInclude>,
+          sdf::InterfaceModelConstPtr>> mergedInterfaceModels;
+
+  /// \brief The interface links specified in this model.
+  public: std::vector<InterfaceLink> interfaceLinks;
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -94,6 +102,30 @@ class sdf::Model::Implementation
 
   /// \brief Scope name of parent Pose Relative-To Graph (world or __model__).
   public: std::string poseGraphScopeVertexName;
+
+  /// \brief Optional URI string that specifies where this model was or
+  /// can be loaded from.
+  public: std::string uri = "";
+
+  /// \brief All of the model plugins.
+  public: std::vector<Plugin> plugins;
+
+  /// \brief The model plugins that were specified only in an <include> tag.
+  /// This data structure is used by the ToElement() function to accurately
+  /// reproduce the <include> tag.
+  ///
+  /// For example, a world could be:
+  ///  <world name="default">
+  ///    <include>
+  ///       <uri>test_model_with_plugin</uri>
+  ///       <plugin name="plugin_name" filename="plugin_filename"/>
+  ///    </include>
+  /// </world>
+  ///
+  /// and the included model could also have a plugin. We want the
+  /// ToElement() function to output only the plugin specified in the
+  /// <include> tag when the ToElementUseIncludeTag policy is true..
+  public: std::vector<Plugin> includePlugins;
 };
 
 /////////////////////////////////////////////////
@@ -196,15 +228,53 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
     frameNames.insert(model.Name());
   }
 
+  // Load InterfaceModels into a temporary container so we can have special
+  // handling for merged InterfaceModels.
+  std::vector<std::pair<sdf::NestedInclude, sdf::InterfaceModelPtr>>
+      tmpInterfaceModels;
   // Load included models via the interface API
   Errors interfaceModelLoadErrors = loadIncludedInterfaceModels(
-      _sdf, _config, this->dataPtr->interfaceModels);
+      _sdf, _config, tmpInterfaceModels);
   errors.insert(errors.end(), interfaceModelLoadErrors.begin(),
       interfaceModelLoadErrors.end());
 
-  for (const auto &ifaceModelPair : this->dataPtr->interfaceModels)
+  for (const auto &[ifaceInclude, ifaceModel] : tmpInterfaceModels)
   {
-    frameNames.insert(ifaceModelPair.second->Name());
+    if (!ifaceInclude.IsMerge().value_or(false))
+    {
+      frameNames.insert(ifaceModel->Name());
+      this->dataPtr->interfaceModels.emplace_back(ifaceInclude, ifaceModel);
+    }
+    else
+    {
+      const std::string proxyModelFrameName =
+          computeMergedModelProxyFrameName(ifaceModel->Name());
+
+      this->dataPtr->mergedInterfaceModels.emplace_back(ifaceInclude,
+                                                        ifaceModel);
+
+      // Copy interface links so that they can be used for resolving canonical
+      // links.
+      for (const auto &ifaceLink : ifaceModel->Links())
+      {
+        this->dataPtr->interfaceLinks.push_back(ifaceLink);
+      }
+
+      for (const auto &ifaceNestedModel : ifaceModel->NestedModels())
+      {
+        sdf::NestedInclude nestedInclude;
+        // The pose of nested interface models is always given relative to the
+        // parent model, but since this is a merged model, we set the relativeTo
+        // to proxyModelFrameName. This nested interface models don't actually
+        // have an //include that was used to nest them, so we create a
+        // NestedInclude object and fill in the the necessary bits.
+        nestedInclude.SetIncludePoseRelativeTo(proxyModelFrameName);
+        nestedInclude.SetIncludeRawPose(
+            ifaceNestedModel->ModelFramePoseInParentFrame());
+        this->dataPtr->interfaceModels.emplace_back(nestedInclude,
+                                                    ifaceNestedModel);
+      }
+    }
   }
 
   // Load all the links.
@@ -245,10 +315,11 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
   }
 
   // If the model is not static and has no nested models:
-  // Require at least one link so the implicit model frame can be attached to
-  // something.
+  // Require at least one (interface) link so the implicit model frame can be
+  // attached to something.
   if (!this->Static() && this->dataPtr->links.empty() &&
-      this->dataPtr->models.empty() && this->dataPtr->interfaceModels.empty())
+      this->dataPtr->interfaceLinks.empty() && this->dataPtr->models.empty() &&
+      this->dataPtr->interfaceModels.empty())
   {
     errors.push_back({ErrorCode::MODEL_WITHOUT_LINK,
                      "A model must have at least one link."});
@@ -328,6 +399,23 @@ Errors Model::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
     frameNames.insert(frameName);
   }
 
+  // Load the model plugins
+  Errors pluginErrors = loadRepeated<Plugin>(_sdf, "plugin",
+    this->dataPtr->plugins);
+  errors.insert(errors.end(), pluginErrors.begin(), pluginErrors.end());
+
+  // Check whether the model was loaded from an <include> tag. If so, set
+  // the URI and capture the plugins.
+  if (_sdf->GetIncludeElement() && _sdf->GetIncludeElement()->HasElement("uri"))
+  {
+    sdf::ElementPtr includeElem = _sdf->GetIncludeElement();
+    this->SetUri(includeElem->Get<std::string>("uri"));
+
+    Errors includePluginErrors = loadRepeated<Plugin>(includeElem, "plugin",
+        this->dataPtr->includePlugins);
+    errors.insert(errors.end(), includePluginErrors.begin(),
+        includePluginErrors.end());
+  }
 
   return errors;
 }
@@ -417,6 +505,13 @@ const Link *Model::LinkByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Link *Model::LinkByIndex(uint64_t _index)
+{
+  return const_cast<Link*>(
+      static_cast<const Model*>(this)->LinkByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Model::LinkNameExists(const std::string &_name) const
 {
@@ -437,6 +532,13 @@ const Joint *Model::JointByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Joint *Model::JointByIndex(uint64_t _index)
+{
+  return const_cast<Joint*>(
+      static_cast<const Model*>(this)->JointByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Model::JointNameExists(const std::string &_name) const
 {
@@ -470,6 +572,14 @@ const Joint *Model::JointByName(const std::string &_name) const
     }
   }
   return nullptr;
+
+}
+
+/////////////////////////////////////////////////
+Joint *Model::JointByName(const std::string &_name)
+{
+  return const_cast<Joint*>(
+      static_cast<const Model*>(this)->JointByName(_name));
 }
 
 /////////////////////////////////////////////////
@@ -486,6 +596,13 @@ const Frame *Model::FrameByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Frame *Model::FrameByIndex(uint64_t _index)
+{
+  return const_cast<Frame*>(
+      static_cast<const Model*>(this)->FrameByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Model::FrameNameExists(const std::string &_name) const
 {
@@ -521,6 +638,13 @@ const Frame *Model::FrameByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Frame *Model::FrameByName(const std::string &_name)
+{
+  return const_cast<Frame*>(
+      static_cast<const Model*>(this)->FrameByName(_name));
+}
+
 /////////////////////////////////////////////////
 uint64_t Model::ModelCount() const
 {
@@ -535,6 +659,13 @@ const Model *Model::ModelByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Model *Model::ModelByIndex(uint64_t _index)
+{
+  return const_cast<Model*>(
+      static_cast<const Model*>(this)->ModelByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool Model::ModelNameExists(const std::string &_name) const
 {
@@ -562,6 +693,14 @@ const Model *Model::ModelByName(const std::string &_name) const
     return nextModel->ModelByName(_name.substr(index + 2));
   }
   return nextModel;
+
+}
+
+/////////////////////////////////////////////////
+Model *Model::ModelByName(const std::string &_name)
+{
+  return const_cast<Model*>(
+      static_cast<const Model*>(this)->ModelByName(_name));
 }
 
 /////////////////////////////////////////////////
@@ -580,6 +719,11 @@ std::pair<const Link*, std::string> Model::CanonicalLinkAndRelativeName() const
       auto firstLink = this->LinkByIndex(0);
       return std::make_pair(firstLink, firstLink->Name());
     }
+    if (this->dataPtr->interfaceLinks.size() > 0)
+    {
+      const auto &firstLink = this->dataPtr->interfaceLinks.front();
+      return std::make_pair(nullptr, firstLink.Name());
+    }
     else if (this->ModelCount() > 0)
     {
       // Recursively choose the canonical link of the first nested model
@@ -683,7 +827,17 @@ void Model::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
   }
   for (auto &ifaceModelPair : this->dataPtr->interfaceModels)
   {
-    ifaceModelPair.second->InvokeRespostureFunction(childPoseGraph);
+    // Don't invoke reposture for interface models that were merged because we
+    // will reposture them below with a different scope in the poe graph.
+    if (ifaceModelPair.first.has_value())
+    {
+      ifaceModelPair.second->InvokeRepostureFunction(childPoseGraph, {});
+    }
+  }
+  for (auto &ifaceModelPair : this->dataPtr->mergedInterfaceModels)
+  {
+    ifaceModelPair.second->InvokeRepostureFunction(this->dataPtr->poseGraph,
+                                                   this->Name());
   }
   for (auto &link : this->dataPtr->links)
   {
@@ -759,6 +913,14 @@ const Link *Model::LinkByName(const std::string &_name) const
     }
   }
   return nullptr;
+
+}
+
+/////////////////////////////////////////////////
+Link *Model::LinkByName(const std::string &_name)
+{
+  return const_cast<Link*>(
+      static_cast<const Model*>(this)->LinkByName(_name));
 }
 
 /////////////////////////////////////////////////
@@ -787,10 +949,169 @@ const NestedInclude *Model::InterfaceModelNestedIncludeByIndex(
     const uint64_t _index) const
 {
   if (_index < this->dataPtr->interfaceModels.size())
-    return &this->dataPtr->interfaceModels[_index].first;
+    return optionalToPointer(this->dataPtr->interfaceModels[_index].first);
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+const std::vector<
+    std::pair<std::optional<sdf::NestedInclude>, sdf::InterfaceModelConstPtr>>
+    &Model::MergedInterfaceModels() const
+{
+  return this->dataPtr->mergedInterfaceModels;
+}
+
+/////////////////////////////////////////////////
+std::string Model::Uri() const
+{
+  return this->dataPtr->uri;
+}
+
+/////////////////////////////////////////////////
+void Model::SetUri(const std::string &_uri)
+{
+  this->dataPtr->uri = _uri;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Model::ToElement(const OutputConfig &_config) const
+{
+  if (_config.ToElementUseIncludeTag() && !this->dataPtr->uri.empty())
+  {
+    sdf::ElementPtr worldElem(new sdf::Element);
+    sdf::initFile("world.sdf", worldElem);
+
+    sdf::ElementPtr includeElem = worldElem->AddElement("include");
+    includeElem->GetElement("uri")->Set(this->Uri());
+    includeElem->GetElement("name")->Set(this->Name());
+    includeElem->GetElement("pose")->Set(this->RawPose());
+    if (!this->dataPtr->poseRelativeTo.empty())
+    {
+      includeElem->GetElement("pose")->GetAttribute(
+          "relative_to")->Set<std::string>(this->dataPtr->poseRelativeTo);
+    }
+    includeElem->GetElement("static")->Set(this->Static());
+    if (!this->dataPtr->placementFrameName.empty())
+    {
+      includeElem->GetElement("placement_frame")->Set(
+          this->PlacementFrameName());
+    }
+
+    // Output the plugins
+    for (const Plugin &plugin : this->dataPtr->includePlugins)
+      includeElem->InsertElement(plugin.ToElement(), true);
+
+    return includeElem;
+  }
+
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("model.sdf", elem);
+  elem->GetAttribute("name")->Set(this->Name());
+
+  if (!this->dataPtr->canonicalLink.empty())
+  {
+    elem->GetAttribute("canonical_link")->Set(this->dataPtr->canonicalLink);
+  }
+
+  if (!this->dataPtr->placementFrameName.empty())
+  {
+    elem->GetAttribute("placement_frame")->Set(
+        this->dataPtr->placementFrameName);
+  }
+
+  elem->GetElement("static")->Set(this->Static());
+  elem->GetElement("self_collide")->Set(this->SelfCollide());
+  elem->GetElement("allow_auto_disable")->Set(this->AllowAutoDisable());
+  elem->GetElement("enable_wind")->Set(this->EnableWind());
+
+  // Set pose
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
+
+  // Links
+  for (const sdf::Link &link : this->dataPtr->links)
+    elem->InsertElement(link.ToElement(), true);
+
+  // Joints
+  for (const sdf::Joint &joint : this->dataPtr->joints)
+    elem->InsertElement(joint.ToElement(), true);
+
+  // Model
+  for (const sdf::Model &model : this->dataPtr->models)
+    elem->InsertElement(model.ToElement(_config), true);
+
+  // Add in the plugins
+  for (const Plugin &plugin : this->dataPtr->plugins)
+    elem->InsertElement(plugin.ToElement(), true);
+
+  return elem;
+}
+
+//////////////////////////////////////////////////
+bool Model::AddLink(const Link &_link)
+{
+  if (this->LinkNameExists(_link.Name()))
+    return false;
+  this->dataPtr->links.push_back(_link);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Model::AddJoint(const Joint &_joint)
+{
+  if (this->JointNameExists(_joint.Name()))
+    return false;
+  this->dataPtr->joints.push_back(_joint);
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Model::AddModel(const Model &_model)
+{
+  if (this->ModelNameExists(_model.Name()))
+    return false;
+  this->dataPtr->models.push_back(_model);
+  return true;
+}
+
+//////////////////////////////////////////////////
+void Model::ClearLinks()
+{
+  this->dataPtr->links.clear();
+}
+
+//////////////////////////////////////////////////
+void Model::ClearJoints()
+{
+  this->dataPtr->joints.clear();
+}
+
+//////////////////////////////////////////////////
+void Model::ClearModels()
+{
+  this->dataPtr->models.clear();
+}
+
+//////////////////////////////////////////////////
+bool Model::AddFrame(const Frame &_frame)
+{
+  if (this->FrameNameExists(_frame.Name()))
+    return false;
+  this->dataPtr->frames.push_back(_frame);
+  return true;
+}
+
+//////////////////////////////////////////////////
+void Model::ClearFrames()
+{
+  this->dataPtr->frames.clear();
+}
+
 /////////////////////////////////////////////////
 bool Model::NameExistsInFrameAttachedToGraph(const std::string &_name) const
 {
@@ -800,3 +1121,27 @@ bool Model::NameExistsInFrameAttachedToGraph(const std::string &_name) const
   return this->dataPtr->frameAttachedToGraph.VertexIdByName(sdf::JoinName(
              this->Name(), _name)) != ignition::math::graph::kNullId;
 }
+
+/////////////////////////////////////////////////
+const sdf::Plugins &Model::Plugins() const
+{
+  return this->dataPtr->plugins;
+}
+
+/////////////////////////////////////////////////
+sdf::Plugins &Model::Plugins()
+{
+  return this->dataPtr->plugins;
+}
+
+/////////////////////////////////////////////////
+void Model::ClearPlugins()
+{
+  this->dataPtr->plugins.clear();
+}
+
+/////////////////////////////////////////////////
+void Model::AddPlugin(const Plugin &_plugin)
+{
+  this->dataPtr->plugins.push_back(_plugin);
+}
