@@ -16,18 +16,19 @@
 */
 #include <vector>
 
+#include "sdf/parser.hh"
 #include "sdf/Polyline.hh"
 
 using namespace sdf;
 
 // Private data class
-class sdf::PolylinePrivate
+class sdf::Polyline::Implementation
 {
   /// \brief Height in meters
   public: double height{1.0};
 
   /// \brief 2D points.
-  public: std::vector<gz::math::Vector2d> points;
+  public: std::vector<ignition::math::Vector2d> points;
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
@@ -35,45 +36,10 @@ class sdf::PolylinePrivate
 
 /////////////////////////////////////////////////
 Polyline::Polyline()
-  : dataPtr(new PolylinePrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Polyline::~Polyline()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-//////////////////////////////////////////////////
-Polyline::Polyline(const Polyline &_polyline)
-  : dataPtr(new PolylinePrivate)
-{
-  this->dataPtr->height = _polyline.dataPtr->height;
-  this->dataPtr->points = _polyline.dataPtr->points;
-  this->dataPtr->sdf = _polyline.dataPtr->sdf;
-}
-
-//////////////////////////////////////////////////
-Polyline::Polyline(Polyline &&_polyline) noexcept
-  : dataPtr(std::exchange(_polyline.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Polyline &Polyline::operator=(const Polyline &_polyline)
-{
-  return *this = Polyline(_polyline);
-}
-
-/////////////////////////////////////////////////
-Polyline &Polyline::operator=(Polyline &&_polyline)
-{
-  std::swap(this->dataPtr, _polyline.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Polyline::Load(ElementPtr _sdf)
 {
@@ -121,7 +87,7 @@ Errors Polyline::Load(ElementPtr _sdf)
        pointElem != nullptr;
        pointElem = pointElem->GetNextElement("point"))
   {
-    this->dataPtr->points.push_back(pointElem->Get<gz::math::Vector2d>());
+    this->dataPtr->points.push_back(pointElem->Get<ignition::math::Vector2d>());
   }
 
   return errors;
@@ -146,7 +112,7 @@ uint64_t Polyline::PointCount() const
 }
 
 /////////////////////////////////////////////////
-const gz::math::Vector2d *Polyline::PointByIndex(uint64_t _index) const
+const ignition::math::Vector2d *Polyline::PointByIndex(uint64_t _index) const
 {
   if (_index < this->dataPtr->points.size())
     return &this->dataPtr->points[_index];
@@ -154,7 +120,7 @@ const gz::math::Vector2d *Polyline::PointByIndex(uint64_t _index) const
 }
 
 /////////////////////////////////////////////////
-gz::math::Vector2d *Polyline::PointByIndex(uint64_t _index)
+ignition::math::Vector2d *Polyline::PointByIndex(uint64_t _index)
 {
   if (_index < this->dataPtr->points.size())
     return &this->dataPtr->points[_index];
@@ -162,7 +128,7 @@ gz::math::Vector2d *Polyline::PointByIndex(uint64_t _index)
 }
 
 /////////////////////////////////////////////////
-bool Polyline::AddPoint(const gz::math::Vector2d &_point)
+bool Polyline::AddPoint(const ignition::math::Vector2d &_point)
 {
   if (this->dataPtr->points.size() == this->dataPtr->points.max_size())
   {
@@ -179,7 +145,7 @@ void Polyline::ClearPoints()
 }
 
 /////////////////////////////////////////////////
-const std::vector<gz::math::Vector2d> &Polyline::Points() const
+const std::vector<ignition::math::Vector2d> &Polyline::Points() const
 {
   return this->dataPtr->points;
 }
@@ -189,3 +155,21 @@ sdf::ElementPtr Polyline::Element() const
 {
   return this->dataPtr->sdf;
 }
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Polyline::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("polyline_shape.sdf", elem);
+
+  auto heightElem = elem->GetElement("height");
+  heightElem->Set<double>(this->Height());
+
+  for (auto &point : this->dataPtr->points)
+  {
+    auto pointElem = elem->AddElement("point");
+    pointElem->Set<ignition::math::Vector2d>(point);
+  }
+
+  return elem;
+}
