@@ -14,24 +14,15 @@
  * limitations under the License.
  *
 */
+#include "sdf/parser.hh"
 #include "sdf/Scene.hh"
 #include "Utils.hh"
 
 using namespace sdf;
 
 /// \brief Scene private data.
-class sdf::ScenePrivate
+class sdf::Scene::Implementation
 {
-  /// \brief Default constructor
-  public: ScenePrivate() = default;
-
-  /// \brief Copy constructor
-  /// \param[in] _scenePrivate private data to copy
-  public: explicit ScenePrivate(const ScenePrivate &_scenePrivate);
-
-  // Delete copy assignment so it is not accidentally used
-  public: ScenePrivate &operator=(const ScenePrivate &) = delete;
-
   /// \brief True if grid should be enabled
   public: bool grid = true;
 
@@ -42,74 +33,26 @@ class sdf::ScenePrivate
   public: bool originVisual = true;
 
   /// \brief Ambient light color of the scene.
-  public: gz::math::Color ambient =
-      gz::math::Color(0.4f, 0.4f, 0.4f);
+  public: ignition::math::Color ambient =
+      ignition::math::Color(0.4f, 0.4f, 0.4f);
 
   /// \brief Background color of the scene.
-  public: gz::math::Color background =
-      gz::math::Color(0.7f, 0.7f, .7f);
+  public: ignition::math::Color background =
+      ignition::math::Color(0.7f, 0.7f, .7f);
 
   /// \brief Pointer to the sky properties.
-  public: std::unique_ptr<Sky> sky;
+  public: std::optional<sdf::Sky> sky;
 
   /// \brief The SDF element pointer used during load.
   public: sdf::ElementPtr sdf;
 };
 
-/////////////////////////////////////////////////
-ScenePrivate::ScenePrivate(const ScenePrivate &_scenePrivate)
-    : grid(_scenePrivate.grid),
-      shadows(_scenePrivate.shadows),
-      originVisual(_scenePrivate.originVisual),
-      ambient(_scenePrivate.ambient),
-      background(_scenePrivate.background),
-      sdf(_scenePrivate.sdf)
-{
-  if (_scenePrivate.sky)
-  {
-    this->sky =
-        std::make_unique<Sky>(*(_scenePrivate.sky));
-  }
-}
-
 /////////////////////////////////////////////////
 Scene::Scene()
-  : dataPtr(new ScenePrivate)
+  : dataPtr(ignition::utils::MakeImpl<Implementation>())
 {
 }
 
-/////////////////////////////////////////////////
-Scene::~Scene()
-{
-  delete this->dataPtr;
-  this->dataPtr = nullptr;
-}
-
-/////////////////////////////////////////////////
-Scene::Scene(const Scene &_scene)
-  : dataPtr(new ScenePrivate(*_scene.dataPtr))
-{
-}
-
-/////////////////////////////////////////////////
-Scene::Scene(Scene &&_scene) noexcept
-  : dataPtr(std::exchange(_scene.dataPtr, nullptr))
-{
-}
-
-/////////////////////////////////////////////////
-Scene &Scene::operator=(const Scene &_scene)
-{
-  return *this = Scene(_scene);
-}
-
-/////////////////////////////////////////////////
-Scene &Scene::operator=(Scene &&_scene)
-{
-  std::swap(this->dataPtr, _scene.dataPtr);
-  return *this;
-}
-
 /////////////////////////////////////////////////
 Errors Scene::Load(ElementPtr _sdf)
 {
@@ -128,11 +71,11 @@ Errors Scene::Load(ElementPtr _sdf)
   }
 
   // Get the ambient property
-  this->dataPtr->ambient = _sdf->Get<gz::math::Color>("ambient",
+  this->dataPtr->ambient = _sdf->Get<ignition::math::Color>("ambient",
       this->dataPtr->ambient).first;
 
   // Get the background color property
-  this->dataPtr->background = _sdf->Get<gz::math::Color>("background",
+  this->dataPtr->background = _sdf->Get<ignition::math::Color>("background",
       this->dataPtr->background).first;
 
   // Get the grid property
@@ -150,7 +93,7 @@ Errors Scene::Load(ElementPtr _sdf)
   // load sky
   if (_sdf->HasElement("sky"))
   {
-    this->dataPtr->sky = std::make_unique<sdf::Sky>();
+    this->dataPtr->sky.emplace();
     Errors err = this->dataPtr->sky->Load(_sdf->GetElement("sky"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
@@ -159,25 +102,25 @@ Errors Scene::Load(ElementPtr _sdf)
 }
 
 /////////////////////////////////////////////////
-gz::math::Color Scene::Ambient() const
+ignition::math::Color Scene::Ambient() const
 {
   return this->dataPtr->ambient;
 }
 
 /////////////////////////////////////////////////
-void Scene::SetAmbient(const gz::math::Color &_ambient)
+void Scene::SetAmbient(const ignition::math::Color &_ambient)
 {
   this->dataPtr->ambient = _ambient;
 }
 
 /////////////////////////////////////////////////
-gz::math::Color Scene::Background() const
+ignition::math::Color Scene::Background() const
 {
   return this->dataPtr->background;
 }
 
 /////////////////////////////////////////////////
-void Scene::SetBackground(const gz::math::Color &_background)
+void Scene::SetBackground(const ignition::math::Color &_background)
 {
   this->dataPtr->background = _background;
 }
@@ -220,13 +163,13 @@ void Scene::SetOriginVisual(const bool _enabled)
 /////////////////////////////////////////////////
 void Scene::SetSky(const sdf::Sky &_sky)
 {
-  this->dataPtr->sky = std::make_unique<sdf::Sky>(_sky);
+  this->dataPtr->sky = _sky;
 }
 
 /////////////////////////////////////////////////
 const sdf::Sky *Scene::Sky() const
 {
-  return this->dataPtr->sky.get();
+  return optionalToPointer(this->dataPtr->sky);
 }
 
 /////////////////////////////////////////////////
@@ -234,3 +177,21 @@ sdf::ElementPtr Scene::Element() const
 {
   return this->dataPtr->sdf;
 }
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Scene::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("scene.sdf", elem);
+
+  elem->GetElement("ambient")->Set(this->Ambient());
+  elem->GetElement("background")->Set(this->Background());
+  elem->GetElement("grid")->Set(this->Grid());
+  elem->GetElement("origin_visual")->Set(this->OriginVisual());
+  elem->GetElement("shadows")->Set(this->Shadows());
+
+  if (this->dataPtr->sky)
+    elem->InsertElement(this->dataPtr->sky->ToElement(), true);
+
+  return elem;
+}
