@@ -28,6 +28,7 @@
 #include "sdf/Imu.hh"
 #include "sdf/Magnetometer.hh"
 #include "sdf/Lidar.hh"
+#include "sdf/parser.hh"
 #include "sdf/Sensor.hh"
 #include "sdf/Types.hh"
 #include "FrameSemantics.hh"
@@ -61,7 +62,11 @@ const std::vector<std::string> sensorTypeStrs =
   "air_pressure",
   "rgbd_camera",
   "thermal_camera",
-  "navsat"
+  "navsat",
+  "segmentation_camera",
+  "boundingbox_camera",
+  "custom",
+  "wide_angle_camera"
 };
 
 class sdf::Sensor::Implementation
@@ -124,6 +129,9 @@ class sdf::Sensor::Implementation
   /// \brief The frequency at which the sensor data is generated.
   /// If left unspecified (0.0), the sensor will generate data every cycle.
   public: double updateRate = 0.0;
+
+  /// \brief Sensor plugins.
+  public: std::vector<Plugin> plugins;
 };
 
 /////////////////////////////////////////////////
@@ -166,6 +174,9 @@ bool Sensor::operator==(const Sensor &_sensor) const
     case SensorType::DEPTH_CAMERA:
     case SensorType::RGBD_CAMERA:
     case SensorType::THERMAL_CAMERA:
+    case SensorType::WIDE_ANGLE_CAMERA:
+    case SensorType::SEGMENTATION_CAMERA:
+    case SensorType::BOUNDINGBOX_CAMERA:
       return *(this->dataPtr->camera) == *(_sensor.dataPtr->camera);
     case SensorType::LIDAR:
       return *(this->dataPtr->lidar) == *(_sensor.dataPtr->lidar);
@@ -258,6 +269,10 @@ Errors Sensor::Load(ElementPtr _sdf)
   {
     this->dataPtr->type = SensorType::CONTACT;
   }
+  else if (type == "custom")
+  {
+    this->dataPtr->type = SensorType::CUSTOM;
+  }
   else if (type == "depth" || type == "depth_camera")
   {
     this->dataPtr->type = SensorType::DEPTH_CAMERA;
@@ -279,6 +294,27 @@ Errors Sensor::Load(ElementPtr _sdf)
     Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
     errors.insert(errors.end(), err.begin(), err.end());
   }
+  else if (type == "segmentation" || type == "segmentation_camera")
+  {
+    this->dataPtr->type = SensorType::SEGMENTATION_CAMERA;
+    this->dataPtr->camera.emplace();
+    Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
+  else if (type == "boundingbox" || type == "boundingbox_camera")
+  {
+    this->dataPtr->type = SensorType::BOUNDINGBOX_CAMERA;
+    this->dataPtr->camera.emplace();
+    Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
+  else if (type == "wideanglecamera" || type == "wide_angle_camera")
+  {
+    this->dataPtr->type = SensorType::WIDE_ANGLE_CAMERA;
+    this->dataPtr->camera.emplace();
+    Errors err = this->dataPtr->camera->Load(_sdf->GetElement("camera"));
+    errors.insert(errors.end(), err.begin(), err.end());
+  }
   else if (type == "force_torque")
   {
     this->dataPtr->type = SensorType::FORCE_TORQUE;
@@ -365,6 +401,11 @@ Errors Sensor::Load(ElementPtr _sdf)
   // Load the pose. Ignore the return value since the sensor pose is optional.
   loadPose(_sdf, this->dataPtr->pose, this->dataPtr->poseRelativeTo);
 
+  // Load the sensor plugins
+  Errors pluginErrors = loadRepeated<Plugin>(_sdf, "plugin",
+    this->dataPtr->plugins);
+  errors.insert(errors.end(), pluginErrors.begin(), pluginErrors.end());
+
   return errors;
 }
 
@@ -489,6 +530,12 @@ const Magnetometer *Sensor::MagnetometerSensor() const
   return optionalToPointer(this->dataPtr->magnetometer);
 }
 
+/////////////////////////////////////////////////
+Magnetometer *Sensor::MagnetometerSensor()
+{
+  return optionalToPointer(this->dataPtr->magnetometer);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetMagnetometerSensor(const Magnetometer &_mag)
 {
@@ -501,6 +548,12 @@ const Altimeter *Sensor::AltimeterSensor() const
   return optionalToPointer(this->dataPtr->altimeter);
 }
 
+/////////////////////////////////////////////////
+Altimeter *Sensor::AltimeterSensor()
+{
+  return optionalToPointer(this->dataPtr->altimeter);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetAltimeterSensor(const Altimeter &_alt)
 {
@@ -513,6 +566,12 @@ const AirPressure *Sensor::AirPressureSensor() const
   return optionalToPointer(this->dataPtr->airPressure);
 }
 
+/////////////////////////////////////////////////
+AirPressure *Sensor::AirPressureSensor()
+{
+  return optionalToPointer(this->dataPtr->airPressure);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetAirPressureSensor(const AirPressure &_air)
 {
@@ -525,6 +584,12 @@ const Lidar *Sensor::LidarSensor() const
   return optionalToPointer(this->dataPtr->lidar);
 }
 
+/////////////////////////////////////////////////
+Lidar *Sensor::LidarSensor()
+{
+  return optionalToPointer(this->dataPtr->lidar);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetLidarSensor(const Lidar &_lidar)
 {
@@ -564,6 +629,12 @@ const Camera *Sensor::CameraSensor() const
   return optionalToPointer(this->dataPtr->camera);
 }
 
+/////////////////////////////////////////////////
+Camera *Sensor::CameraSensor()
+{
+  return optionalToPointer(this->dataPtr->camera);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetForceTorqueSensor(const ForceTorque &_ft)
 {
@@ -576,6 +647,12 @@ const ForceTorque *Sensor::ForceTorqueSensor() const
   return optionalToPointer(this->dataPtr->forceTorque);
 }
 
+/////////////////////////////////////////////////
+ForceTorque *Sensor::ForceTorqueSensor()
+{
+  return optionalToPointer(this->dataPtr->forceTorque);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetNavSatSensor(const NavSat &_gps)
 {
@@ -588,6 +665,12 @@ const NavSat *Sensor::NavSatSensor() const
   return optionalToPointer(this->dataPtr->navSat);
 }
 
+/////////////////////////////////////////////////
+NavSat *Sensor::NavSatSensor()
+{
+  return optionalToPointer(this->dataPtr->navSat);
+}
+
 /////////////////////////////////////////////////
 void Sensor::SetImuSensor(const Imu &_imu)
 {
@@ -599,3 +682,115 @@ const Imu *Sensor::ImuSensor() const
 {
   return optionalToPointer(this->dataPtr->imu);
 }
+
+/////////////////////////////////////////////////
+Imu *Sensor::ImuSensor()
+{
+  return optionalToPointer(this->dataPtr->imu);
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr Sensor::ToElement() const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("sensor.sdf", elem);
+
+  elem->GetAttribute("type")->Set<std::string>(this->TypeStr());
+  elem->GetAttribute("name")->Set<std::string>(this->Name());
+  sdf::ElementPtr poseElem = elem->GetElement("pose");
+  if (!this->dataPtr->poseRelativeTo.empty())
+  {
+    poseElem->GetAttribute("relative_to")->Set<std::string>(
+        this->dataPtr->poseRelativeTo);
+  }
+  poseElem->Set<ignition::math::Pose3d>(this->RawPose());
+
+  elem->GetElement("topic")->Set<std::string>(this->Topic());
+  elem->GetElement("update_rate")->Set<double>(this->UpdateRate());
+  elem->GetElement("enable_metrics")->Set<double>(this->EnableMetrics());
+
+  // air pressure
+  if (this->Type() == sdf::SensorType::AIR_PRESSURE &&
+      this->dataPtr->airPressure)
+  {
+    sdf::ElementPtr airPressureElem = elem->GetElement("air_pressure");
+    airPressureElem->Copy(this->dataPtr->airPressure->ToElement());
+  }
+  // altimeter
+  else if (this->Type() == sdf::SensorType::ALTIMETER &&
+      this->dataPtr->altimeter)
+  {
+    sdf::ElementPtr altimeterElem = elem->GetElement("altimeter");
+    altimeterElem->Copy(this->dataPtr->altimeter->ToElement());
+  }
+  // camera, depth, thermal, segmentation
+  else if (this->CameraSensor())
+  {
+    sdf::ElementPtr cameraElem = elem->GetElement("camera");
+    cameraElem->Copy(this->dataPtr->camera->ToElement());
+  }
+  // force torque
+  else if (this->Type() == sdf::SensorType::FORCE_TORQUE  &&
+      this->dataPtr->forceTorque)
+  {
+    sdf::ElementPtr forceTorqueElem = elem->GetElement("force_torque");
+    forceTorqueElem->Copy(this->dataPtr->forceTorque->ToElement());
+  }
+  // imu
+  else if (this->Type() == sdf::SensorType::IMU && this->dataPtr->imu)
+  {
+    sdf::ElementPtr imuElem = elem->GetElement("imu");
+    imuElem->Copy(this->dataPtr->imu->ToElement());
+  }
+  // lidar, gpu_lidar
+  else if ((this->Type() == sdf::SensorType::GPU_LIDAR ||
+            this->Type() == sdf::SensorType::LIDAR) &&
+           this->dataPtr->lidar)
+  {
+    sdf::ElementPtr rayElem = (elem->HasElement("ray")) ?
+        elem->GetElement("ray") : elem->GetElement("lidar");
+    rayElem->Copy(this->dataPtr->lidar->ToElement());
+  }
+  // magnetometer
+  else if (this->Type() == sdf::SensorType::MAGNETOMETER &&
+      this->dataPtr->magnetometer)
+  {
+    sdf::ElementPtr magnetometerElem = elem->GetElement("magnetometer");
+    magnetometerElem->Copy(this->dataPtr->magnetometer->ToElement());
+  }
+  else
+  {
+    std::cout << "Conversion of sensor type: [" << this->TypeStr() << "] from "
+      << "SDF DOM to Element is not supported yet." << std::endl;
+  }
+
+  // Add in the plugins
+  for (const Plugin &plugin : this->dataPtr->plugins)
+    elem->InsertElement(plugin.ToElement(), true);
+
+  return elem;
+}
+
+/////////////////////////////////////////////////
+const sdf::Plugins &Sensor::Plugins() const
+{
+  return this->dataPtr->plugins;
+}
+
+/////////////////////////////////////////////////
+sdf::Plugins &Sensor::Plugins()
+{
+  return this->dataPtr->plugins;
+}
+
+/////////////////////////////////////////////////
+void Sensor::ClearPlugins()
+{
+  this->dataPtr->plugins.clear();
+}
+
+/////////////////////////////////////////////////
+void Sensor::AddPlugin(const Plugin &_plugin)
+{
+  this->dataPtr->plugins.push_back(_plugin);
+}
