@@ -218,39 +218,46 @@ sdf::Errors loadIncludedInterfaceModels(sdf::ElementPtr _sdf,
        includeElem = includeElem->GetNextElement("include"))
   {
     sdf::NestedInclude include;
-    include.uri = includeElem->Get<std::string>("uri");
+    include.SetUri(includeElem->Get<std::string>("uri"));
     auto absoluteParentName = computeAbsoluteName(_sdf, allErrors);
 
     if (absoluteParentName.has_value())
     {
-      include.absoluteParentName = *absoluteParentName;
+      include.SetAbsoluteParentName(*absoluteParentName);
     }
 
     if (includeElem->HasElement("name"))
     {
-      include.localModelName = includeElem->Get<std::string>("name");
+      include.SetLocalModelName(includeElem->Get<std::string>("name"));
     }
     if (includeElem->HasElement("static"))
     {
-      include.isStatic = includeElem->Get<bool>("static");
+      include.SetIsStatic(includeElem->Get<bool>("static"));
     }
-    include.resolvedFileName = sdf::findFile(include.uri, true, true, _config);
+    include.SetResolvedFileName(
+        sdf::findFile(include.Uri(), true, true, _config));
 
-    include.includeElement = includeElem;
+    include.SetIncludeElement(includeElem);
     if (includeElem->HasElement("pose"))
     {
       auto poseElem = includeElem->GetElement("pose");
-      include.includeRawPose = poseElem->Get<ignition::math::Pose3d>();
+      include.SetIncludeRawPose(poseElem->Get<ignition::math::Pose3d>());
       if (poseElem->HasAttribute("relative_to"))
       {
-        include.includePoseRelativeTo =
-            poseElem->Get<std::string>("relative_to");
+        include.SetIncludePoseRelativeTo(
+            poseElem->Get<std::string>("relative_to"));
       }
     }
 
     if (includeElem->HasElement("placement_frame"))
     {
-      include.placementFrame = includeElem->Get<std::string>("placement_frame");
+      include.SetPlacementFrame(
+          includeElem->Get<std::string>("placement_frame"));
+    }
+
+    if (includeElem->HasAttribute("merge"))
+    {
+      include.SetIsMerge(includeElem->Get<bool>("merge"));
     }
 
     // Iterate through custom model parsers in reverse per the SDFormat proposal
@@ -273,7 +280,16 @@ sdf::Errors loadIncludedInterfaceModels(sdf::ElementPtr _sdf,
         if (model->Name() == "")
         {
           allErrors.emplace_back(sdf::ErrorCode::ATTRIBUTE_INVALID,
-              "Missing name of custom model with URI [" + include.uri + "]");
+              "Missing name of custom model with URI [" + include.Uri() + "]");
+        }
+        else if (include.IsMerge().value_or(false) &&
+                 !model->ParserSupportsMergeInclude())
+        {
+          allErrors.emplace_back(sdf::ErrorCode::MERGE_INCLUDE_UNSUPPORTED,
+                                 "Custom parser does not support "
+                                 "merge-include, but merge-include was "
+                                 "requested for model with uri [" +
+                                     include.Uri() + "]");
         }
         else
         {
@@ -288,5 +304,63 @@ sdf::Errors loadIncludedInterfaceModels(sdf::ElementPtr _sdf,
 
   return allErrors;
 }
+
+/////////////////////////////////////////////////
+void copyChildren(ElementPtr _sdf, tinyxml2::XMLElement *_xml,
+    const bool _onlyUnknown)
+{
+  // Iterate over all the child elements
+  tinyxml2::XMLElement *elemXml = nullptr;
+  for (elemXml = _xml->FirstChildElement(); elemXml;
+       elemXml = elemXml->NextSiblingElement())
+  {
+    std::string elemName = elemXml->Name();
+
+    if (_sdf->HasElementDescription(elemName))
+    {
+      if (!_onlyUnknown)
+      {
+        sdf::ElementPtr element = _sdf->AddElement(elemName);
+
+        // FIXME: copy attributes
+        for (const auto *attribute = elemXml->FirstAttribute();
+             attribute; attribute = attribute->Next())
+        {
+          element->GetAttribute(attribute->Name())->SetFromString(
+            attribute->Value());
+        }
+
+        // copy value
+        const char *value = elemXml->GetText();
+        if (value)
+        {
+          element->GetValue()->SetFromString(value);
+        }
+        copyChildren(element, elemXml, _onlyUnknown);
+      }
+    }
+    else
+    {
+      sdf::ElementPtr element(new sdf::Element);
+      element->SetParent(_sdf);
+      element->SetName(elemName);
+      for (const tinyxml2::XMLAttribute *attribute = elemXml->FirstAttribute();
+           attribute; attribute = attribute->Next())
+      {
+        element->AddAttribute(attribute->Name(), "string", "", 1, "");
+        element->GetAttribute(attribute->Name())->SetFromString(
+            attribute->Value());
+      }
+
+      if (elemXml->GetText() != nullptr)
+      {
+        element->AddValue("string", elemXml->GetText(), true);
+      }
+
+      copyChildren(element, elemXml, _onlyUnknown);
+      _sdf->InsertElement(element);
+    }
+  }
+}
 }
 }
