@@ -39,19 +39,19 @@ TEST(DOMVisual, Construction)
   visual.SetTransparency(0.34f);
   EXPECT_FLOAT_EQ(0.34f, visual.Transparency());
 
-  EXPECT_EQ(gz::math::Pose3d::Zero, visual.RawPose());
+  EXPECT_EQ(ignition::math::Pose3d::Zero, visual.RawPose());
   EXPECT_TRUE(visual.PoseRelativeTo().empty());
   {
     auto semanticPose = visual.SemanticPose();
     EXPECT_EQ(visual.RawPose(), semanticPose.RawPose());
     EXPECT_TRUE(semanticPose.RelativeTo().empty());
-    gz::math::Pose3d pose;
+    ignition::math::Pose3d pose;
     // expect errors when trying to resolve pose
     EXPECT_FALSE(semanticPose.Resolve(pose).empty());
   }
 
   visual.SetRawPose({0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2});
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual.RawPose());
 
   visual.SetPoseRelativeTo("link");
@@ -60,7 +60,7 @@ TEST(DOMVisual, Construction)
     auto semanticPose = visual.SemanticPose();
     EXPECT_EQ(visual.RawPose(), semanticPose.RawPose());
     EXPECT_EQ("link", semanticPose.RelativeTo());
-    gz::math::Pose3d pose;
+    ignition::math::Pose3d pose;
     // expect errors when trying to resolve pose
     EXPECT_FALSE(semanticPose.Resolve(pose).empty());
   }
@@ -76,7 +76,7 @@ TEST(DOMVisual, Construction)
   EXPECT_EQ(nullptr, visual.Material());
 
   // visibility flags
-  EXPECT_EQ(4294967295u, visual.VisibilityFlags());
+  EXPECT_EQ(UINT32_MAX, visual.VisibilityFlags());
   visual.SetVisibilityFlags(1u);
   EXPECT_EQ(1u, visual.VisibilityFlags());
 }
@@ -102,7 +102,7 @@ TEST(DOMVisual, CopyConstructor)
   EXPECT_EQ("test_visual", visual.Name());
   EXPECT_FALSE(visual.CastShadows());
   EXPECT_FLOAT_EQ(0.345f, visual.Transparency());
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual.RawPose());
   EXPECT_EQ("link", visual.PoseRelativeTo());
   ASSERT_TRUE(nullptr != visual.Material());
@@ -112,7 +112,7 @@ TEST(DOMVisual, CopyConstructor)
   EXPECT_EQ("test_visual", visual2.Name());
   EXPECT_FALSE(visual2.CastShadows());
   EXPECT_FLOAT_EQ(0.345f, visual2.Transparency());
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual2.RawPose());
   EXPECT_EQ("link", visual2.PoseRelativeTo());
   ASSERT_TRUE(nullptr != visual2.Material());
@@ -142,7 +142,7 @@ TEST(DOMVisual, CopyAssignmentOperator)
   EXPECT_EQ("test_visual", visual.Name());
   EXPECT_FALSE(visual.CastShadows());
   EXPECT_FLOAT_EQ(0.345f, visual.Transparency());
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual.RawPose());
   EXPECT_EQ("link", visual.PoseRelativeTo());
   ASSERT_TRUE(nullptr != visual.Material());
@@ -152,7 +152,7 @@ TEST(DOMVisual, CopyAssignmentOperator)
   EXPECT_EQ("test_visual", visual2.Name());
   EXPECT_FALSE(visual2.CastShadows());
   EXPECT_FLOAT_EQ(0.345f, visual2.Transparency());
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual2.RawPose());
   EXPECT_EQ("link", visual2.PoseRelativeTo());
   ASSERT_TRUE(nullptr != visual2.Material());
@@ -181,7 +181,7 @@ TEST(DOMVisual, MoveConstructor)
   EXPECT_EQ("test_visual", visual2.Name());
   EXPECT_FALSE(visual2.CastShadows());
   EXPECT_FLOAT_EQ(0.345f, visual2.Transparency());
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual2.RawPose());
   EXPECT_EQ("link", visual2.PoseRelativeTo());
   ASSERT_TRUE(nullptr != visual2.Material());
@@ -211,7 +211,7 @@ TEST(DOMVisual, MoveAssignmentOperator)
   EXPECT_EQ("test_visual", visual2.Name());
   EXPECT_FALSE(visual2.CastShadows());
   EXPECT_FLOAT_EQ(0.345f, visual2.Transparency());
-  EXPECT_EQ(gz::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
+  EXPECT_EQ(ignition::math::Pose3d(0, -20, 30, IGN_PI_2, -IGN_PI, IGN_PI_2),
             visual2.RawPose());
   EXPECT_EQ("link", visual2.PoseRelativeTo());
   ASSERT_TRUE(nullptr != visual2.Material());
@@ -262,16 +262,16 @@ TEST(DOMVisual, SetMaterial)
   EXPECT_TRUE(visual.Name().empty());
 
   sdf::Material material;
-  material.SetAmbient(gz::math::Color(0, 0.5, 0));
-  material.SetDiffuse(gz::math::Color(1, 0, 0));
-  material.SetSpecular(gz::math::Color(0.f, 0.1f, 0.9f));
+  material.SetAmbient(ignition::math::Color(0, 0.5, 0));
+  material.SetDiffuse(ignition::math::Color(1, 0, 0));
+  material.SetSpecular(ignition::math::Color(0.f, 0.1f, 0.9f));
 
   visual.SetMaterial(material);
 
   ASSERT_NE(nullptr, visual.Material());
-  EXPECT_EQ(gz::math::Color(0, 0.5, 0), visual.Material()->Ambient());
-  EXPECT_EQ(gz::math::Color(1, 0, 0), visual.Material()->Diffuse());
-  EXPECT_EQ(gz::math::Color(0.f, 0.1f, 0.9f),
+  EXPECT_EQ(ignition::math::Color(0, 0.5, 0), visual.Material()->Ambient());
+  EXPECT_EQ(ignition::math::Color(1, 0, 0), visual.Material()->Diffuse());
+  EXPECT_EQ(ignition::math::Color(0.f, 0.1f, 0.9f),
             visual.Material()->Specular());
 }
 
@@ -287,3 +287,71 @@ TEST(DOMVisual, SetLaserRetro)
   EXPECT_TRUE(visual.HasLaserRetro());
   EXPECT_DOUBLE_EQ(150, visual.LaserRetro());
 }
+
+/////////////////////////////////////////////////
+TEST(DOMVisual, ToElement)
+{
+  sdf::Visual visual;
+  visual.SetName("my-visual");
+  visual.SetCastShadows(true);
+  visual.SetTransparency(0.2f);
+  visual.SetRawPose(ignition::math::Pose3d(1, 2, 3, 0.1, 0.2, 0.3));
+  visual.SetVisibilityFlags(1234u);
+  visual.SetHasLaserRetro(true);
+  visual.SetLaserRetro(1.2);
+
+  sdf::Geometry geom;
+  visual.SetGeom(geom);
+
+  sdf::Material mat;
+  visual.SetMaterial(mat);
+
+  sdf::Plugin plugin;
+  plugin.SetName("name1");
+  plugin.SetFilename("filename1");
+  visual.AddPlugin(plugin);
+
+  sdf::ElementPtr elem = visual.ToElement();
+  ASSERT_NE(nullptr, elem);
+
+  sdf::Visual visual2;
+  visual2.Load(elem);
+
+  EXPECT_EQ(visual.Name(), visual2.Name());
+  EXPECT_EQ(visual.CastShadows(), visual2.CastShadows());
+  EXPECT_DOUBLE_EQ(visual.Transparency(), visual2.Transparency());
+  EXPECT_EQ(visual.RawPose(), visual2.RawPose());
+  EXPECT_EQ(visual.VisibilityFlags(), visual2.VisibilityFlags());
+  EXPECT_EQ(visual.HasLaserRetro(), visual2.HasLaserRetro());
+  EXPECT_DOUBLE_EQ(visual.LaserRetro(), visual2.LaserRetro());
+  EXPECT_NE(nullptr, visual2.Geom());
+  EXPECT_NE(nullptr, visual2.Material());
+
+  ASSERT_EQ(1u, visual2.Plugins().size());
+  EXPECT_EQ("name1", visual2.Plugins()[0].Name());
+  EXPECT_EQ("filename1", visual2.Plugins()[0].Filename());
+}
+
+/////////////////////////////////////////////////
+TEST(DOMVisual, Plugins)
+{
+  sdf::Visual visual;
+  EXPECT_TRUE(visual.Plugins().empty());
+
+  sdf::Plugin plugin;
+  plugin.SetName("name1");
+  plugin.SetFilename("filename1");
+
+  visual.AddPlugin(plugin);
+  ASSERT_EQ(1u, visual.Plugins().size());
+
+  plugin.SetName("name2");
+  visual.AddPlugin(plugin);
+  ASSERT_EQ(2u, visual.Plugins().size());
+
+  EXPECT_EQ("name1", visual.Plugins()[0].Name());
+  EXPECT_EQ("name2", visual.Plugins()[1].Name());
+
+  visual.ClearPlugins();
+  EXPECT_TRUE(visual.Plugins().empty());
+}
