@@ -29,6 +29,7 @@
 #include "sdf/Model.hh"
 #include "sdf/ParserConfig.hh"
 #include "sdf/Physics.hh"
+#include "sdf/Plugin.hh"
 #include "sdf/Types.hh"
 #include "sdf/World.hh"
 #include "FrameSemantics.hh"
@@ -40,6 +41,11 @@ using namespace sdf;
 
 class sdf::World::Implementation
 {
+  /// \brief Populate sphericalCoordinates
+  /// \param[in] _elem `<spherical_coordinates>` element
+  /// \return Errors, if any.
+  public: Errors LoadSphericalCoordinates(sdf::ElementPtr _elem);
+
   /// \brief Optional atmosphere model.
   public: std::optional<sdf::Atmosphere> atmosphere;
 
@@ -69,6 +75,10 @@ class sdf::World::Implementation
   public: ignition::math::Vector3d magneticField =
            ignition::math::Vector3d(5.5645e-6, 22.8758e-6, -42.3884e-6);
 
+  /// \brief Spherical coordinates
+  public: std::optional<ignition::math::SphericalCoordinates>
+      sphericalCoordinates;
+
   /// \brief The models specified in this world.
   public: std::vector<Model> models;
 
@@ -96,8 +106,10 @@ class sdf::World::Implementation
   /// \brief Scoped Pose Relative-To graph that points to a graph owned by this
   /// world.
   public: sdf::ScopedGraph<sdf::PoseRelativeToGraph> poseRelativeToGraph;
-};
 
+  /// \brief World plugins.
+  public: sdf::Plugins plugins;
+};
 
 /////////////////////////////////////////////////
 World::World()
@@ -180,6 +192,26 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
     _sdf->Get<ignition::math::Vector3d>("magnetic_field",
         this->dataPtr->magneticField).first;
 
+  // Read the spherical coordinates
+  if (_sdf->HasElement("spherical_coordinates"))
+  {
+    Errors sphericalCoordsErrors = this->dataPtr->LoadSphericalCoordinates(
+        _sdf->GetElement("spherical_coordinates"));
+    errors.insert(errors.end(), sphericalCoordsErrors.begin(),
+        sphericalCoordsErrors.end());
+  }
+
+  for (const auto &[name, size] :
+       _sdf->CountNamedElements("", Element::NameUniquenessExceptions()))
+  {
+    if (size > 1)
+    {
+      sdfwarn << "Non-unique name[" << name << "] detected " << size
+              << " times in XML children of world with name[" << this->Name()
+              << "].\n";
+    }
+  }
+
   for (const auto &[name, size] :
        _sdf->CountNamedElements("", Element::NameUniquenessExceptions()))
   {
@@ -282,6 +314,11 @@ Errors World::Load(sdf::ElementPtr _sdf, const ParserConfig &_config)
     errors.insert(errors.end(), sceneLoadErrors.begin(), sceneLoadErrors.end());
   }
 
+  // Load the world plugins
+  Errors pluginErrors = loadRepeated<Plugin>(_sdf, "plugin",
+    this->dataPtr->plugins);
+  errors.insert(errors.end(), pluginErrors.begin(), pluginErrors.end());
+
   return errors;
 }
 
@@ -370,6 +407,13 @@ const Model *World::ModelByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Model *World::ModelByIndex(uint64_t _index)
+{
+  return const_cast<Model*>(
+      static_cast<const World*>(this)->ModelByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool World::ModelNameExists(const std::string &_name) const
 {
@@ -399,6 +443,13 @@ const Model *World::ModelByName(const std::string &_name) const
   return nextModel;
 }
 
+/////////////////////////////////////////////////
+Model *World::ModelByName(const std::string &_name)
+{
+  return const_cast<Model*>(
+      static_cast<const World*>(this)->ModelByName(_name));
+}
+
 /////////////////////////////////////////////////
 const sdf::Atmosphere *World::Atmosphere() const
 {
@@ -411,6 +462,20 @@ void World::SetAtmosphere(const sdf::Atmosphere &_atmosphere)
   this->dataPtr->atmosphere = _atmosphere;
 }
 
+/////////////////////////////////////////////////
+const ignition::math::SphericalCoordinates *
+    World::SphericalCoordinates() const
+{
+  return optionalToPointer(this->dataPtr->sphericalCoordinates);
+}
+
+/////////////////////////////////////////////////
+void World::SetSphericalCoordinates(const ignition::math::SphericalCoordinates
+    &_sphericalCoordinates)
+{
+  this->dataPtr->sphericalCoordinates = _sphericalCoordinates;
+}
+
 /////////////////////////////////////////////////
 const sdf::Gui *World::Gui() const
 {
@@ -455,6 +520,13 @@ const Frame *World::FrameByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Frame *World::FrameByIndex(uint64_t _index)
+{
+  return const_cast<Frame*>(
+      static_cast<const World*>(this)->FrameByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool World::FrameNameExists(const std::string &_name) const
 {
@@ -490,6 +562,13 @@ const Frame *World::FrameByName(const std::string &_name) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Frame *World::FrameByName(const std::string &_name)
+{
+  return const_cast<Frame*>(
+      static_cast<const World*>(this)->FrameByName(_name));
+}
+
 /////////////////////////////////////////////////
 uint64_t World::LightCount() const
 {
@@ -504,6 +583,13 @@ const Light *World::LightByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Light *World::LightByIndex(uint64_t _index)
+{
+  return const_cast<Light*>(
+      static_cast<const World*>(this)->LightByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool World::LightNameExists(const std::string &_name) const
 {
@@ -531,6 +617,13 @@ const Actor *World::ActorByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+/////////////////////////////////////////////////
+Actor *World::ActorByIndex(uint64_t _index)
+{
+  return const_cast<Actor*>(
+      static_cast<const World*>(this)->ActorByIndex(_index));
+}
+
 /////////////////////////////////////////////////
 bool World::ActorNameExists(const std::string &_name) const
 {
@@ -558,6 +651,13 @@ const Physics *World::PhysicsByIndex(const uint64_t _index) const
   return nullptr;
 }
 
+//////////////////////////////////////////////////
+Physics *World::PhysicsByIndex(uint64_t _index)
+{
+  return const_cast<Physics*>(
+      static_cast<const World*>(this)->PhysicsByIndex(_index));
+}
+
 //////////////////////////////////////////////////
 const Physics *World::PhysicsDefault() const
 {
@@ -624,8 +724,8 @@ void World::SetPoseRelativeToGraph(sdf::ScopedGraph<PoseRelativeToGraph> _graph)
   }
   for (auto &ifaceModelPair : this->dataPtr->interfaceModels)
   {
-    ifaceModelPair.second->InvokeRespostureFunction(
-        this->dataPtr->poseRelativeToGraph);
+    ifaceModelPair.second->InvokeRepostureFunction(
+        this->dataPtr->poseRelativeToGraph, {});
   }
   for (auto &frame : this->dataPtr->frames)
   {
@@ -653,3 +753,277 @@ void World::SetFrameAttachedToGraph(
     model.SetFrameAttachedToGraph(this->dataPtr->frameAttachedToGraph);
   }
 }
+
+/////////////////////////////////////////////////
+Errors World::Implementation::LoadSphericalCoordinates(
+    sdf::ElementPtr _elem)
+{
+  Errors errors;
+
+  if (_elem->GetName() != "spherical_coordinates")
+  {
+    errors.push_back({ErrorCode::ELEMENT_INCORRECT_TYPE,
+        "Attempting to load <spherical_coordinates>, but the provided SDF "
+        "element is a <" + _elem->GetName() + ">."});
+    return errors;
+  }
+
+  // Get elements
+  auto surfaceModel =
+      ignition::math::SphericalCoordinates::SurfaceType::EARTH_WGS84;
+  if (!_elem->HasElement("surface_model"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <surface_model>"});
+  }
+  else
+  {
+    auto surfaceModelStr = _elem->Get<std::string>("surface_model");
+    if (surfaceModelStr != "EARTH_WGS84")
+    {
+      errors.push_back({ErrorCode::ELEMENT_INVALID,
+          "The supplied <surface_model> [" + surfaceModelStr +
+          "] is not supported."});
+    }
+    surfaceModel = ignition::math::SphericalCoordinates::Convert(
+        surfaceModelStr);
+  }
+
+  std::string worldFrameOrientation{"ENU"};
+  if (_elem->HasElement("world_frame_orientation"))
+  {
+    worldFrameOrientation = _elem->Get<std::string>(
+        "world_frame_orientation");
+    if (worldFrameOrientation != "ENU")
+    {
+      errors.push_back({ErrorCode::ELEMENT_INVALID,
+          "The supplied <world_frame_orientation> [" + worldFrameOrientation +
+          "] is not supported."});
+    }
+  }
+
+  ignition::math::Angle latitude{0.0};
+  if (!_elem->HasElement("latitude_deg"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <latitude_deg>"});
+  }
+  else
+  {
+    latitude.SetDegree(_elem->Get<double>("latitude_deg"));
+  }
+
+  ignition::math::Angle longitude{0.0};
+  if (!_elem->HasElement("longitude_deg"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <longitude_deg>"});
+  }
+  else
+  {
+    longitude.SetDegree(_elem->Get<double>("longitude_deg"));
+  }
+
+  double elevation{0.0};
+  if (!_elem->HasElement("elevation"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <elevation>"});
+  }
+  else
+  {
+    elevation = _elem->Get<double>("elevation");
+  }
+
+  ignition::math::Angle heading{0.0};
+  if (!_elem->HasElement("heading_deg"))
+  {
+    errors.push_back({ErrorCode::ELEMENT_MISSING,
+        "Missing required element <heading_deg>"});
+  }
+  else
+  {
+    heading.SetDegree(_elem->Get<double>("heading_deg"));
+  }
+
+  // Create coordinates
+  this->sphericalCoordinates.emplace();
+  this->sphericalCoordinates =
+      ignition::math::SphericalCoordinates(surfaceModel, latitude, longitude,
+      elevation, heading);
+
+  return errors;
+}
+
+/////////////////////////////////////////////////
+sdf::ElementPtr World::ToElement(const OutputConfig &_config) const
+{
+  sdf::ElementPtr elem(new sdf::Element);
+  sdf::initFile("world.sdf", elem);
+
+  elem->GetAttribute("name")->Set(this->Name());
+  elem->GetElement("gravity")->Set(this->Gravity());
+  elem->GetElement("magnetic_field")->Set(this->MagneticField());
+
+  sdf::ElementPtr windElem = elem->GetElement("wind");
+  windElem->GetElement("linear_velocity")->Set(this->WindLinearVelocity());
+
+  // Physics
+  for (const sdf::Physics &physics : this->dataPtr->physics)
+    elem->InsertElement(physics.ToElement(), true);
+
+  // Models
+  for (const sdf::Model &model : this->dataPtr->models)
+    elem->InsertElement(model.ToElement(_config), true);
+
+  // Actors
+  for (const sdf::Actor &actor : this->dataPtr->actors)
+    elem->InsertElement(actor.ToElement(), true);
+
+  // Lights
+  for (const sdf::Light &light : this->dataPtr->lights)
+    elem->InsertElement(light.ToElement(), true);
+
+  // Spherical coordinates.
+  if (this->dataPtr->sphericalCoordinates)
+  {
+    sdf::ElementPtr sphericalElem = elem->GetElement("spherical_coordinates");
+    sphericalElem->GetElement("surface_model")->Set(
+        ignition::math::SphericalCoordinates::Convert(
+          this->dataPtr->sphericalCoordinates->Surface()));
+    sphericalElem->GetElement("world_frame_orientation")->Set("ENU");
+    sphericalElem->GetElement("latitude_deg")->Set(
+        this->dataPtr->sphericalCoordinates->LatitudeReference().Degree());
+    sphericalElem->GetElement("longitude_deg")->Set(
+        this->dataPtr->sphericalCoordinates->LongitudeReference().Degree());
+    sphericalElem->GetElement("elevation")->Set(
+        this->dataPtr->sphericalCoordinates->ElevationReference());
+    sphericalElem->GetElement("heading_deg")->Set(
+        this->dataPtr->sphericalCoordinates->HeadingOffset().Degree());
+  }
+
+  // Atmosphere
+  if (this->dataPtr->atmosphere)
+    elem->InsertElement(this->dataPtr->atmosphere->ToElement(), true);
+
+  // Gui
+  if (this->dataPtr->gui)
+    elem->InsertElement(this->dataPtr->gui->ToElement(), true);
+
+  // Scene
+  if (this->dataPtr->scene)
+    elem->InsertElement(this->dataPtr->scene->ToElement(), true);
+
+  // Audio
+  if (this->dataPtr->audioDevice != "default")
+    elem->GetElement("audio")->GetElement("device")->Set(this->AudioDevice());
+
+  // Add in the plugins
+  for (const Plugin &plugin : this->dataPtr->plugins)
+    elem->InsertElement(plugin.ToElement(), true);
+
+  return elem;
+}
+
+/////////////////////////////////////////////////
+void World::ClearModels()
+{
+  this->dataPtr->models.clear();
+}
+
+/////////////////////////////////////////////////
+void World::ClearActors()
+{
+  this->dataPtr->actors.clear();
+}
+
+/////////////////////////////////////////////////
+void World::ClearLights()
+{
+  this->dataPtr->lights.clear();
+}
+
+/////////////////////////////////////////////////
+void World::ClearPhysics()
+{
+  this->dataPtr->physics.clear();
+}
+
+/////////////////////////////////////////////////
+void World::ClearFrames()
+{
+  this->dataPtr->frames.clear();
+}
+
+/////////////////////////////////////////////////
+bool World::AddModel(const Model &_model)
+{
+  if (this->ModelNameExists(_model.Name()))
+    return false;
+  this->dataPtr->models.push_back(_model);
+  return true;
+}
+
+/////////////////////////////////////////////////
+bool World::AddActor(const Actor &_actor)
+{
+  if (this->ActorNameExists(_actor.Name()))
+    return false;
+  this->dataPtr->actors.push_back(_actor);
+
+  return true;
+}
+
+/////////////////////////////////////////////////
+bool World::AddLight(const Light &_light)
+{
+  if (this->LightNameExists(_light.Name()))
+    return false;
+  this->dataPtr->lights.push_back(_light);
+
+  return true;
+}
+
+/////////////////////////////////////////////////
+bool World::AddPhysics(const Physics &_physics)
+{
+  if (this->PhysicsNameExists(_physics.Name()))
+    return false;
+  this->dataPtr->physics.push_back(_physics);
+
+  return true;
+}
+
+/////////////////////////////////////////////////
+bool World::AddFrame(const Frame &_frame)
+{
+  if (this->FrameNameExists(_frame.Name()))
+    return false;
+  this->dataPtr->frames.push_back(_frame);
+
+  return true;
+}
+
+/////////////////////////////////////////////////
+const sdf::Plugins &World::Plugins() const
+{
+  return this->dataPtr->plugins;
+}
+
+/////////////////////////////////////////////////
+sdf::Plugins &World::Plugins()
+{
+  return this->dataPtr->plugins;
+}
+
+/////////////////////////////////////////////////
+void World::ClearPlugins()
+{
+  this->dataPtr->plugins.clear();
+}
+
+/////////////////////////////////////////////////
+void World::AddPlugin(const Plugin &_plugin)
+{
+  this->dataPtr->plugins.push_back(_plugin);
+}
