@@ -15,7 +15,9 @@
  *
 */
 
+#include <cstring>
 #include <iostream>
+#include <memory>
 #include <string>
 #include <string.h>
 #include <vector>
@@ -26,10 +28,13 @@
 #include "sdf/Model.hh"
 #include "sdf/Root.hh"
 #include "sdf/parser.hh"
+#include "sdf/PrintConfig.hh"
 #include "sdf/system_util.hh"
 
-#include "gz/math/Inertial.hh"
+#include "ignition/math/Inertial.hh"
 
+#include "FrameSemantics.hh"
+#include "ScopedGraph.hh"
 #include "ign.hh"
 
 //////////////////////////////////////////////////
@@ -43,7 +48,7 @@ extern "C" SDFORMAT_VISIBLE int cmdCheck(const char *_path)
   {
     for (auto &error : errors)
     {
-      std::cerr << "Error: " << error.Message() << std::endl;
+      std::cerr << error << std::endl;
     }
     return -1;
   }
@@ -133,7 +138,8 @@ extern "C" SDFORMAT_VISIBLE int cmdDescribe(const char *_version)
 }
 
 //////////////////////////////////////////////////
-extern "C" SDFORMAT_VISIBLE int cmdPrint(const char *_path)
+extern "C" SDFORMAT_VISIBLE int cmdPrint(const char *_path,
+    int inDegrees, int snapToDegrees, float snapTolerance)
 {
   if (!sdf::filesystem::exists(_path))
   {
@@ -155,7 +161,114 @@ extern "C" SDFORMAT_VISIBLE int cmdPrint(const char *_path)
     return -1;
   }
 
-  sdf->PrintValues();
+  sdf::PrintConfig config;
+  if (inDegrees!= 0)
+  {
+    config.SetRotationInDegrees(true);
+  }
+  if (snapToDegrees > 0)
+  {
+    config.SetRotationSnapToDegrees(static_cast<unsigned int>(snapToDegrees),
+                                    static_cast<double>(snapTolerance));
+  }
+
+  sdf->PrintValues(config);
+  return 0;
+}
+
+//////////////////////////////////////////////////
+extern "C" SDFORMAT_VISIBLE int cmdPrintPreserveIncludes(const char *_path)
+{
+  if (!sdf::filesystem::exists(_path))
+  {
+    std::cerr << "Error: File [" << _path << "] does not exist.\n";
+    return -1;
+  }
+
+  sdf::SDFPtr sdf(new sdf::SDF());
+
+  if (!sdf::init(sdf))
+  {
+    std::cerr << "Error: SDF schema initialization failed.\n";
+    return -1;
+  }
+
+  if (!sdf::readFile(_path, sdf))
+  {
+    std::cerr << "Error: SDF parsing the xml failed.\n";
+    return -1;
+  }
+
+  sdf::PrintConfig config;
+  config.SetPreserveIncludes(true);
+  sdf->PrintValues(config);
+
+  return 0;
+}
+
+//////////////////////////////////////////////////
+// cppcheck-suppress unusedFunction
+extern "C" SDFORMAT_VISIBLE int cmdGraph(
+    const char *_graphType, const char *_path)
+{
+  if (!sdf::filesystem::exists(_path))
+  {
+    std::cerr << "Error: File [" << _path << "] does not exist.\n";
+    return -1;
+  }
+
+  sdf::Root root;
+  sdf::Errors errors = root.Load(_path);
+  if (!errors.empty())
+  {
+    std::cerr << errors << std::endl;
+  }
+
+  if (std::strcmp(_graphType, "pose") == 0)
+  {
+    auto ownedGraph = std::make_shared<sdf::PoseRelativeToGraph>();
+    sdf::ScopedGraph<sdf::PoseRelativeToGraph> graph(ownedGraph);
+    if (root.WorldCount() > 0)
+    {
+      errors = sdf::buildPoseRelativeToGraph(graph, root.WorldByIndex(0));
+    }
+    else if (root.Model() != nullptr)
+    {
+      errors =
+        sdf::buildPoseRelativeToGraph(graph, root.Model());
+    }
+
+    if (!errors.empty())
+    {
+      std::cerr << errors << std::endl;
+    }
+    std::cout << graph.Graph() << std::endl;
+  }
+  else if (std::strcmp(_graphType, "frame") == 0)
+  {
+    auto ownedGraph = std::make_shared<sdf::FrameAttachedToGraph>();
+    sdf::ScopedGraph<sdf::FrameAttachedToGraph> graph(ownedGraph);
+    if (root.WorldCount() > 0)
+    {
+      errors = sdf::buildFrameAttachedToGraph(graph, root.WorldByIndex(0));
+    }
+    else if (root.Model() != nullptr)
+    {
+      errors =
+        sdf::buildFrameAttachedToGraph(graph, root.Model());
+    }
+
+    if (!errors.empty())
+    {
+      std::cerr << errors << std::endl;
+    }
+    std::cout << graph.Graph() << std::endl;
+  }
+  else
+  {
+    std::cerr << R"(Only "pose" and "frame" graph types are supported)"
+              << std::endl;
+  }
 
   return 0;
 }
@@ -174,10 +287,7 @@ extern "C" SDFORMAT_VISIBLE int cmdInertialStats(
   sdf::Errors errors = root.Load(_path);
   if (!errors.empty())
   {
-    for (auto &error : errors)
-    {
-      std::cerr << "Error: " << error.Message() << std::endl;
-    }
+    std::cerr << errors << std::endl;
   }
 
   if (root.WorldCount() > 0)
@@ -187,7 +297,7 @@ extern "C" SDFORMAT_VISIBLE int cmdInertialStats(
     return -1;
   }
 
-  const sdf::Model *model = root.ModelByIndex(0);
+  const sdf::Model *model = root.Model();
   if (!model)
   {
     std::cerr << "Error: Could not find the model." << std::endl;
@@ -200,13 +310,17 @@ extern "C" SDFORMAT_VISIBLE int cmdInertialStats(
             " models will not be included." << std::endl;
   }
 
-  gz::math::Inertiald totalInertial;
+  ignition::math::Inertiald totalInertial;
 
   for (uint64_t i = 0; i < model->LinkCount(); i++)
   {
-    gz::math::Inertiald currentLinkInertial;
-    model->LinkByIndex(i)->ResolveInertial(currentLinkInertial, "__model__");
+    ignition::math::Pose3d linkPoseRelativeToModel;
+    errors = model->LinkByIndex(i)->SemanticPose().
+      Resolve(linkPoseRelativeToModel, "__model__");
 
+    auto currentLinkInertial = model->LinkByIndex(i)->Inertial();
+    currentLinkInertial.SetPose(linkPoseRelativeToModel *
+      currentLinkInertial.Pose());
     totalInertial += currentLinkInertial;
   }
 
