@@ -20,7 +20,7 @@
 #include <stdlib.h>
 #include <string>
 
-#include <ignition/utilities/ExtraTestMacros.hh>
+#include <ignition/utils/ExtraTestMacros.hh>
 
 #include "sdf/parser.hh"
 #include "sdf/SDFImpl.hh"
@@ -114,6 +114,14 @@ TEST(check, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
     EXPECT_NE(output.find("Non-unique names"), std::string::npos)
       << output;
   }
+  // Check an SDF world file is allowed to have duplicate plugin names
+  {
+    std::string path = pathBase +"/world_duplicate_plugins.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
 
   // Check an SDF file with sibling elements of the same type (link)
   // that have duplicate names.
@@ -151,6 +159,15 @@ TEST(check, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
       << output;
   }
 
+  // Check an SDF model file is allowed to have duplicate plugin names
+  {
+    std::string path = pathBase +"/model_duplicate_plugins.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+
   // Check an SDF file with sibling elements of the same type (collision)
   // that have duplicate names.
   {
@@ -257,7 +274,7 @@ TEST(check, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
     std::string output =
       custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
     EXPECT_NE(output.find("Joint with name[joint] must "
-                          "specify different link names for parent and child, "
+                          "specify different frame names for parent and child, "
                           "while [link] was specified for both."),
               std::string::npos) << output;
   }
@@ -322,6 +339,17 @@ TEST(check, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
     EXPECT_EQ("Valid.\n", output) << output;
   }
 
+  // Check an SDF file with the infinite values for joint axis limits.
+  // This is a valid file.
+  {
+    std::string path = pathBase +"/joint_axis_infinite_limits.sdf";
+
+    // Check joint_axis_infinite_limits.sdf
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf -k " + path + SdfVersion());
+    EXPECT_EQ("Valid.\n", output) << output;
+  }
+
   // Check an SDF file with the second link specified as the canonical link.
   // This is a valid file.
   {
@@ -920,6 +948,631 @@ TEST(print, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
   }
 }
 
+//////////////////////////////////////////////////
+static bool contains(const std::string &_a, const std::string &_b)
+{
+  return _a.find(_b) != std::string::npos;
+}
+
+/////////////////////////////////////////////////
+TEST(print_rotations_in_degrees, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const std::string path =
+      sdf::testing::TestFile("sdf", "rotations_in_degrees.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true'>1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_rotations_in_radians, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const std::string path =
+      sdf::testing::TestFile("sdf", "rotations_in_radians.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose>1 2 3 0.523756 0.785241 -1.04735</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.0087</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.0087</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_rotations_in_quaternions, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const auto path = sdf::testing::TestFile(
+      "sdf", "rotations_in_quaternions.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose rotation_format='quat_xyzw'>"
+               "1 2 3   0.391948 0.200425 -0.532046 0.723279</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_includes_rotations_in_degrees, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  // Set SDF_PATH so that included models can be found
+  sdf::testing::setenv(
+    "SDF_PATH", sdf::testing::SourceFile("test", "integration", "model"));
+  const std::string path =
+      sdf::testing::TestFile("sdf", "includes_rotations_in_degrees.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true'>1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_includes_rotations_in_radians, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  // Set SDF_PATH so that included models can be found
+  sdf::testing::setenv(
+    "SDF_PATH", sdf::testing::SourceFile("test", "integration", "model"));
+  const std::string path =
+      sdf::testing::TestFile("sdf", "includes_rotations_in_radians.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose>1 2 3 0.523756 0.785241 -1.04735</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.0087</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.0087</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_includes_rotations_in_quaternions,
+     IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  // Set SDF_PATH so that included models can be found
+  sdf::testing::setenv(
+    "SDF_PATH", sdf::testing::SourceFile("test", "integration", "model"));
+  const auto path = sdf::testing::TestFile(
+      "sdf", "includes_rotations_in_quaternions.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose rotation_format='quat_xyzw'>"
+               "1 2 3   0.391948 0.200425 -0.532046 0.723279</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_rotations_in_unnormalized_degrees,
+     IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const std::string path =
+      sdf::testing::TestFile("sdf", "rotations_in_unnormalized_degrees.sdf");
+
+  // Default printing
+  // Unnormalized degree values cannot be returned as is, as its string is
+  // returned by parsing the pose value, whenever a parent Element Attribute,
+  // or PrintConfig is used.
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true'>1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.991 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_rotations_in_unnormalized_radians,
+     IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const std::string path =
+      sdf::testing::TestFile("sdf", "rotations_in_unnormalized_radians.sdf");
+
+  // Default printing
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose>1 2 3 -5.75943 -11.78112 5.23583</pose>");
+
+  // Printing with in_degrees
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.9915 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // Printing with snap_to_degrees 2
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 2 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 44.9915 -60</pose>");
+
+  // Printing with snap_to_degrees 20
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 20 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.9915 -60</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.008
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.008 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.9915 -60.009</pose>");
+
+  // Printing with snap_to_degrees 5, snap_tolerance 0.01
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      "--snap-tolerance 0.01 " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(shuffled_cmd_flags, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const std::string path =
+      sdf::testing::TestFile("sdf", "rotations_in_unnormalized_radians.sdf");
+
+  // -p PATH --degrees
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --degrees " + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.9915 -60.009</pose>");
+
+  // --degrees -p PATH
+  output = custom_exec_str(
+      IgnCommand() + " sdf --degrees -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30.009 44.9915 -60.009</pose>");
+
+  // -p PATH --snap-to-degrees ARG
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path + " --snap-to-degrees 5 " +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // -p --snap-to-degrees ARG PATH
+  output = custom_exec_str(
+      IgnCommand() + " sdf -p --snap-to-degrees 5 " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+
+  // --snap-to-degrees ARG -p PATH
+  output = custom_exec_str(
+      IgnCommand() + " sdf --snap-to-degrees 5 -p " + path + SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 45 -60</pose>");
+}
+
+/////////////////////////////////////////////////
+TEST(print_snap_to_degrees_tolerance_too_high,
+     IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  const std::string path = sdf::testing::TestFile(
+      "sdf",
+      "rotations_in_degrees_high_snap_tolerance.sdf");
+
+  std::string output = custom_exec_str(
+      IgnCommand() + " sdf -p " + path +
+      " --snap-to-degrees 5 " + " --snap-tolerance 4" +
+      SdfVersion());
+  ASSERT_FALSE(output.empty());
+  EXPECT_PRED2(contains, output,
+               "<pose degrees='true' rotation_format='euler_rpy'>"
+               "1 2 3   30 50 60</pose>");
+}
+
 /////////////////////////////////////////////////
 TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(WorldPoseRelativeTo))
 {
@@ -953,10 +1606,10 @@ TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(WorldPoseRelativeTo))
     << "  16 [label=\"F7 (16)\"];\n"
     << "  0 -> 1 [label=1];\n"
     << "  2 -> 3 [label=0];\n"
-    << "  3 -> 4 [label=1];\n"
-    << "  3 -> 5 [label=1];\n"
     << "  8 -> 9 [label=0];\n"
     << "  9 -> 10 [label=1];\n"
+    << "  3 -> 4 [label=1];\n"
+    << "  3 -> 5 [label=1];\n"
     << "  5 -> 6 [label=1];\n"
     << "  5 -> 7 [label=1];\n"
     << "  3 -> 8 [label=1];\n"
@@ -1013,12 +1666,11 @@ TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelPoseRelativeTo))
     << "  29 [label=\"parent_model::M7::__model__ (29)\"];\n"
     << "  30 [label=\"parent_model::M7::L (30)\"];\n"
     << "  1 -> 2 [label=0];\n"
-    << "  2 -> 3 [label=1];\n"
     << "  4 -> 5 [label=0];\n"
-    << "  5 -> 6 [label=1];\n"
-    << "  5 -> 7 [label=1];\n"
     << "  10 -> 11 [label=0];\n"
     << "  11 -> 12 [label=1];\n"
+    << "  5 -> 6 [label=1];\n"
+    << "  5 -> 7 [label=1];\n"
     << "  7 -> 8 [label=1];\n"
     << "  7 -> 9 [label=1];\n"
     << "  5 -> 10 [label=1];\n"
@@ -1034,6 +1686,7 @@ TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelPoseRelativeTo))
     << "  26 -> 27 [label=1];\n"
     << "  28 -> 29 [label=0];\n"
     << "  29 -> 30 [label=1];\n"
+    << "  2 -> 3 [label=1];\n"
     << "  2 -> 4 [label=1];\n"
     << "  4 -> 13 [label=1];\n"
     << "  6 -> 16 [label=1];\n"
@@ -1142,6 +1795,89 @@ TEST(GraphCmd, IGN_UTILS_TEST_DISABLED_ON_WIN32(ModelFrameAttachedTo))
   EXPECT_EQ(sdf::trim(expected.str()), sdf::trim(output));
 }
 
+/////////////////////////////////////////////////
+TEST(inertial_stats, IGN_UTILS_TEST_DISABLED_ON_WIN32(SDF))
+{
+  std::string pathBase = PROJECT_SOURCE_PATH;
+  pathBase += "/test/sdf";
+
+  auto expectedOutput =
+    "Inertial statistics for model: test_model\n"
+    "---\n"
+    "Total mass of the model: 24\n"
+    "---\n"
+    "Centre of mass in model frame: \n"
+    "X: 0\n"
+    "Y: 0\n"
+    "Z: 0\n"
+    "---\n"
+    "Moment of inertia matrix: \n"
+    "304  0    0    \n"
+    "0    304  0    \n"
+    "0    0    604  \n"
+    "---\n";
+
+  // Check a good SDF file by passing the absolute path
+  {
+    std::string path = pathBase +"/inertial_stats.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf --inertial-stats " +
+                      path + SdfVersion());
+    EXPECT_EQ(expectedOutput, output);
+  }
+
+  // Check a good SDF file from the same folder by passing a relative path
+  {
+    std::string path = "inertial_stats.sdf";
+
+    std::string output =
+      custom_exec_str("cd " + pathBase + " && " +
+                      IgnCommand() + " sdf --inertial-stats " +
+                      path + SdfVersion());
+    EXPECT_EQ(expectedOutput, output);
+  }
+
+  expectedOutput =
+          "Error Code 18: Msg: A link named link has invalid inertia.\n\n"
+          "Inertial statistics for model: model\n"
+          "---\n"
+          "Total mass of the model: 0\n"
+          "---\n"
+          "Centre of mass in model frame: \n"
+          "X: 0\n"
+          "Y: 0\n"
+          "Z: 0\n"
+          "---\n"
+          "Moment of inertia matrix: \n"
+          "0  0  0  \n"
+          "0  0  0  \n"
+          "0  0  0  \n"
+          "---\n";
+
+  // Check an invalid SDF file by passing the absolute path
+  {
+    std::string path = pathBase +"/inertial_invalid.sdf";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf --inertial-stats " +
+                      path + SdfVersion());
+    EXPECT_EQ(expectedOutput, output);
+  }
+
+  expectedOutput =
+          "Error: Expected a model file but received a world file.\n";
+  // Check a valid world file.
+  {
+    std::string path = pathBase +"/box_plane_low_friction_test.world";
+
+    std::string output =
+      custom_exec_str(IgnCommand() + " sdf --inertial-stats " +
+                      path + SdfVersion());
+    EXPECT_EQ(expectedOutput, output);
+  }
+}
+
 /////////////////////////////////////////////////
 /// Main
 int main(int argc, char **argv)
