@@ -49,6 +49,43 @@
 namespace sdf
 {
 inline namespace SDF_VERSION_NAMESPACE {
+
+namespace
+{
+//////////////////////////////////////////////////
+/// Holds information about the location of a particular point in an SDFormat
+/// file
+struct SourceLocation
+{
+  /// \brief Xml path where the error was raised.
+  public: std::optional<std::string> xmlPath = std::nullopt;
+
+  /// \brief File path where the error was raised.
+  public: std::optional<std::string> filePath = std::nullopt;
+
+  /// \brief Line number in the file path where the error was raised.
+  public: std::optional<int> lineNumber = std::nullopt;
+
+  /// \brief Sets the source location on an sdf::Error object
+  /// \param[in,out] _error sdf::Error object on which the source location is to
+  /// be set.
+  public: void SetSourceLocationOnError(sdf::Error &_error) const
+  {
+    if (this->xmlPath.has_value())
+    {
+      _error.SetXmlPath(*this->xmlPath);
+    }
+    if (this->filePath.has_value())
+    {
+      _error.SetFilePath(*this->filePath);
+    }
+    if (this->lineNumber.has_value())
+    {
+      _error.SetLineNumber(*this->lineNumber);
+    }
+  }
+};
+}
 //////////////////////////////////////////////////
 /// \brief Internal helper for readFile, which populates the SDF values
 /// from a file
@@ -118,7 +155,9 @@ static bool isSdfFile(const std::string &_fileName)
 
 //////////////////////////////////////////////////
 template <typename TPtr>
-static inline bool _initFile(const std::string &_filename, TPtr _sdf)
+static inline bool _initFile(const std::string &_filename,
+                             const ParserConfig &_config,
+                             TPtr _sdf)
 {
   auto xmlDoc = makeSdfDoc();
   if (tinyxml2::XML_SUCCESS != xmlDoc.LoadFile(_filename.c_str()))
@@ -128,16 +167,219 @@ static inline bool _initFile(const std::string &_filename, TPtr _sdf)
     return false;
   }
 
-  return initDoc(&xmlDoc, _sdf);
+  return initDoc(&xmlDoc, _config, _sdf);
+}
+
+//////////////////////////////////////////////////
+/// Helper function to insert included elements into a parent element.
+/// \param[in] _includeSDF The SDFPtr corresponding to the included element
+/// \param[in] _sourceLoc The location of the include element in the file
+/// \param[in] _merge Whether the included element should be merged into the
+/// parent element. If true, children elements of _includeSDF will be copied to
+/// _parent without introducing a new model scope. N.B, this only works for
+/// included nested models.
+/// \param[in,out] _parent The parent element that contains the <include> tag.
+/// The contents of _includeSDF will be added to this.
+/// \param[out] _errors Captures errors encountered during parsing.
+static void insertIncludedElement(sdf::SDFPtr _includeSDF,
+                                  const SourceLocation &_sourceLoc, bool _merge,
+                                  sdf::ElementPtr _parent,
+                                  const ParserConfig &_config,
+                                  sdf::Errors &_errors)
+{
+  Error invalidFileError(ErrorCode::FILE_READ,
+                         "Included model is invalid. Skipping model.");
+  _sourceLoc.SetSourceLocationOnError(invalidFileError);
+
+  sdf::ElementPtr rootElem = _includeSDF->Root();
+  if (nullptr == rootElem)
+  {
+    _errors.push_back(invalidFileError);
+    return;
+  }
+
+  sdf::ElementPtr firstElem = rootElem->GetFirstElement();
+  if (nullptr == firstElem)
+  {
+    _errors.push_back(invalidFileError);
+    return;
+  }
+
+  if (!_merge)
+  {
+    _parent->InsertElement(firstElem, true);
+    return;
+  }
+  else if (firstElem->GetName() != "model")
+  {
+    Error unsupportedError(
+        ErrorCode::MERGE_INCLUDE_UNSUPPORTED,
+        "Merge-include is only supported for included models");
+    _sourceLoc.SetSourceLocationOnError(unsupportedError);
+    _errors.push_back(unsupportedError);
+    return;
+  }
+  else if (_parent->GetName() != "model")
+  {
+    Error unsupportedError(
+        ErrorCode::MERGE_INCLUDE_UNSUPPORTED,
+        "Merge-include does not support parent element of type " +
+            _parent->GetName());
+    _sourceLoc.SetSourceLocationOnError(unsupportedError);
+    _errors.push_back(unsupportedError);
+    return;
+  }
+
+  // Validate included model's frame semantics
+  // We create a throwaway sdf::Root object in order to validate the
+  // included entity.
+  sdf::Root includedRoot;
+  sdf::Errors includeDOMerrors = includedRoot.Load(_includeSDF, _config);
+  _errors.insert(_errors.end(), includeDOMerrors.begin(),
+                 includeDOMerrors.end());
+
+  const sdf::Model *model = includedRoot.Model();
+  if (nullptr == model)
+  {
+    Error unsupportedError(ErrorCode::MERGE_INCLUDE_UNSUPPORTED,
+                           "Included model is invalid. Skipping model.");
+    _sourceLoc.SetSourceLocationOnError(unsupportedError);
+    _errors.push_back(unsupportedError);
+    return;
+  }
+
+  ElementPtr proxyModelFrame = _parent->AddElement("frame");
+  const std::string proxyModelFrameName =
+      computeMergedModelProxyFrameName(model->Name());
+
+  proxyModelFrame->GetAttribute("name")->Set(proxyModelFrameName);
+
+  // Determine the canonical link so the proxy frame can be attached to it
+  const std::string canonicalLinkName =
+      model->CanonicalLinkAndRelativeName().second;
+
+  proxyModelFrame->GetAttribute("attached_to")->Set(canonicalLinkName);
+
+  auto modelPose = model->RawPose();
+  if (!model->PlacementFrameName().empty())
+  {
+    // M - model frame (__model__)
+    // R - The `relative_to` frame of the placement frame's //pose element.
+    // See resolveModelPoseWithPlacementFrame in FrameSemantics.cc for
+    // notation and documentation
+    ignition::math::Pose3d X_RM = model->RawPose();
+    sdf::Errors resolveErrors = model->SemanticPose().Resolve(X_RM);
+    _errors.insert(_errors.end(), resolveErrors.begin(), resolveErrors.end());
+    modelPose = X_RM;
+  }
+
+  ElementPtr proxyModelFramePose = proxyModelFrame->AddElement("pose");
+  proxyModelFramePose->Set(modelPose);
+
+  // Set the proxyModelFrame's //pose/@relative_to to the frame used in
+  // //include/pose/@relative_to.
+  std::string modelPoseRelativeTo = model->PoseRelativeTo();
+
+  // If empty, use "__model__", since leaving it empty would make it
+  // relative_to the canonical link frame specified in //frame/@attached_to.
+  if (modelPoseRelativeTo.empty())
+  {
+    modelPoseRelativeTo = "__model__";
+  }
+
+  proxyModelFramePose->GetAttribute("relative_to")->Set(modelPoseRelativeTo);
+
+  auto setAttributeToProxyFrame =
+      [&proxyModelFrameName](const std::string &_attr, sdf::ElementPtr _elem,
+                             bool updateIfEmpty)
+  {
+    if (nullptr == _elem)
+      return;
+
+    auto attribute = _elem->GetAttribute(_attr);
+    if (attribute->GetAsString() == "__model__" ||
+        (updateIfEmpty && attribute->GetAsString().empty()))
+    {
+      attribute->Set(proxyModelFrameName);
+    }
+  };
+
+  sdf::ElementPtr nextElem = nullptr;
+  for (auto elem = firstElem->GetFirstElement(); elem; elem = nextElem)
+  {
+    // We need to fetch the next element here before we call elem->SetParent
+    // later in this block.
+    nextElem = elem->GetNextElement();
+
+    if ((elem->GetName() == "link") || (elem->GetName() == "model"))
+    {
+      // Add a pose element even if the element doesn't originally have one
+      setAttributeToProxyFrame("relative_to", elem->GetElement("pose"), true);
+    }
+    else if (elem->GetName() == "frame")
+    {
+      // If //frame/@attached_to is empty, explicitly set it to the name
+      // of the nested model frame.
+      setAttributeToProxyFrame("attached_to", elem, true);
+      setAttributeToProxyFrame("relative_to", elem->GetElementImpl("pose"),
+                               false);
+    }
+    else if (elem->GetName() == "joint")
+    {
+      setAttributeToProxyFrame("relative_to", elem->GetElementImpl("pose"),
+                               false);
+
+      auto parent = elem->FindElement("parent");
+      if (nullptr != parent && parent->Get<std::string>() == "__model__")
+      {
+        parent->Set(proxyModelFrameName);
+      }
+      auto child = elem->FindElement("child");
+      if (nullptr != child && child->Get<std::string>() == "__model__")
+      {
+        child->Set(proxyModelFrameName);
+      }
+
+      // cppcheck-suppress syntaxError
+      // cppcheck-suppress unmatchedSuppression
+      if (auto axis = elem->GetElementImpl("axis"); axis)
+      {
+        setAttributeToProxyFrame("expressed_in", axis->GetElementImpl("xyz"),
+                                 false);
+      }
+
+      if (auto axis2 = elem->GetElementImpl("axis2"); axis2)
+      {
+        setAttributeToProxyFrame("expressed_in", axis2->GetElementImpl("xyz"),
+                                 false);
+      }
+    }
+
+    // Only named and custom elements are copied. Other elements, such as
+    // <static>, <self_collide>, and <enable_wind> are ignored.
+    if ((elem->GetName() == "link") || (elem->GetName() == "model") ||
+        (elem->GetName() == "joint") || (elem->GetName() == "frame") ||
+        (elem->GetName() == "gripper") || (elem->GetName() == "plugin") ||
+        (elem->GetName().find(':') != std::string::npos))
+    {
+      _parent->InsertElement(elem, true);
+    }
+  }
 }
 
 //////////////////////////////////////////////////
 bool init(SDFPtr _sdf)
+{
+  return init(_sdf, ParserConfig::GlobalConfig());
+}
+
+//////////////////////////////////////////////////
+bool init(SDFPtr _sdf, const ParserConfig &_config)
 {
   std::string xmldata = SDF::EmbeddedSpec("root.sdf", false);
   auto xmlDoc = makeSdfDoc();
   xmlDoc.Parse(xmldata.c_str());
-  return initDoc(&xmlDoc, _sdf);
+  return initDoc(&xmlDoc, _config, _sdf);
 }
 
 //////////////////////////////////////////////////
@@ -155,9 +397,10 @@ bool initFile(
   {
     auto xmlDoc = makeSdfDoc();
     xmlDoc.Parse(xmldata.c_str());
-    return initDoc(&xmlDoc, _sdf);
+    return initDoc(&xmlDoc, _config, _sdf);
   }
-  return _initFile(sdf::findFile(_filename, true, false, _config), _sdf);
+  return _initFile(sdf::findFile(_filename, true, false, _config), _config,
+                   _sdf);
 }
 
 //////////////////////////////////////////////////
@@ -175,14 +418,15 @@ bool initFile(
   {
     auto xmlDoc = makeSdfDoc();
     xmlDoc.Parse(xmldata.c_str());
-    return initDoc(&xmlDoc, _sdf);
+    return initDoc(&xmlDoc, _config, _sdf);
   }
-  return _initFile(sdf::findFile(_filename, true, false, _config), _sdf);
+  return _initFile(sdf::findFile(_filename, true, false, _config), _config,
+                   _sdf);
 }
 
 //////////////////////////////////////////////////
 bool initString(
-    const std::string &_xmlString, const ParserConfig &, SDFPtr _sdf)
+    const std::string &_xmlString, const ParserConfig &_config, SDFPtr _sdf)
 {
   auto xmlDoc = makeSdfDoc();
   if (xmlDoc.Parse(_xmlString.c_str()))
@@ -191,7 +435,7 @@ bool initString(
     return false;
   }
 
-  return initDoc(&xmlDoc, _sdf);
+  return initDoc(&xmlDoc, _config, _sdf);
 }
 
 //////////////////////////////////////////////////
@@ -220,7 +464,9 @@ inline tinyxml2::XMLElement *_initDocGetElement(tinyxml2::XMLDocument *_xmlDoc)
 }
 
 //////////////////////////////////////////////////
-bool initDoc(tinyxml2::XMLDocument *_xmlDoc, SDFPtr _sdf)
+bool initDoc(tinyxml2::XMLDocument *_xmlDoc,
+             const ParserConfig &_config,
+             SDFPtr _sdf)
 {
   auto element = _initDocGetElement(_xmlDoc);
   if (!element)
@@ -228,11 +474,13 @@ bool initDoc(tinyxml2::XMLDocument *_xmlDoc, SDFPtr _sdf)
     return false;
   }
 
-  return initXml(element, _sdf->Root());
+  return initXml(element, _config, _sdf->Root());
 }
 
 //////////////////////////////////////////////////
-bool initDoc(tinyxml2::XMLDocument *_xmlDoc, ElementPtr _sdf)
+bool initDoc(tinyxml2::XMLDocument *_xmlDoc,
+             const ParserConfig &_config,
+             ElementPtr _sdf)
 {
   auto element = _initDocGetElement(_xmlDoc);
   if (!element)
@@ -240,11 +488,13 @@ bool initDoc(tinyxml2::XMLDocument *_xmlDoc, ElementPtr _sdf)
     return false;
   }
 
-  return initXml(element, _sdf);
+  return initXml(element, _config, _sdf);
 }
 
 //////////////////////////////////////////////////
-bool initXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf)
+bool initXml(tinyxml2::XMLElement *_xml,
+             const ParserConfig &_config,
+             ElementPtr _sdf)
 {
   const char *refString = _xml->Attribute("ref");
   if (refString)
@@ -362,7 +612,7 @@ bool initXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf)
     else
     {
       ElementPtr element(new Element);
-      initXml(child, element);
+      initXml(child, _config, element);
       _sdf->AddElementDescription(element);
     }
   }
@@ -375,7 +625,7 @@ bool initXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf)
 
     ElementPtr element(new Element);
 
-    initFile(filename, element);
+    initFile(filename, _config, element);
 
     // override description for include elements
     tinyxml2::XMLElement *description = child->FirstChildElement("description");
@@ -402,7 +652,7 @@ SDFPtr readFile(
 {
   // Create and initialize the data structure that will hold the parsed SDF data
   sdf::SDFPtr sdfParsed(new sdf::SDF());
-  sdf::init(sdfParsed);
+  sdf::init(sdfParsed, _config);
 
   // Read an SDF file, and store the result in sdfParsed.
   if (!sdf::readFile(_filename, _config, sdfParsed, _errors))
@@ -1198,14 +1448,46 @@ static bool resolveFileNameFromUri(tinyxml2::XMLElement *_includeXml,
   return true;
 }
 
+//////////////////////////////////////////////////
+// Helper function called from readXml to validate the //include tag by calling
+// readXml on it. This is only here for error checking. We won't use the
+// resulting sdf::ElementPtr because the contents of the //include are accessed
+// directly via tinyxml in the subsequent code.
+/// \param[in] _xml Pointer to the TinyXML element that corresponds to the
+/// <include> element
+/// \param[in,out] _sdf SDF pointer to the parent of the <include> element
+/// \param[in] _config Custom parser configuration
+/// \param[in] _source Source of the XML document
+/// \param[out] _errors Captures errors found during parsing.
+static void validateIncludeElement(tinyxml2::XMLElement *_xml,
+                                   ElementPtr _sdf, const ParserConfig &_config,
+                                   const std::string &_source, Errors &_errors)
+{
+  for (unsigned int descCounter = 0;
+      descCounter != _sdf->GetElementDescriptionCount(); ++descCounter)
+  {
+    ElementPtr elemDesc = _sdf->GetElementDescription(descCounter);
+    if (elemDesc->GetName() == _xml->Value())
+    {
+      ElementPtr element = elemDesc->Clone();
+      if (!readXml(_xml, element, _config, _source, _errors))
+      {
+        Error err(
+            ErrorCode::ELEMENT_INVALID,
+            std::string("Error reading element <") +
+            _xml->Value() + ">",
+            _source,
+            _xml->GetLineNum());
+        _errors.push_back(err);
+      }
+    }
+  }
+}
+
 //////////////////////////////////////////////////
 bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
     const ParserConfig &_config, const std::string &_source, Errors &_errors)
 {
-  std::string errorSourcePath = _source;
-  if (_source == kSdfStringSource || _source == kUrdfStringSource)
-    errorSourcePath = "<" + _source + ">";
-
   // Check if the element pointer is deprecated.
   if (_sdf->GetRequired() == "-1")
   {
@@ -1223,7 +1505,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
     {
       Error err(
           ErrorCode::ELEMENT_MISSING,
-          "SDF Element<" + _sdf->GetName() + "> is missing", errorSourcePath);
+          "SDF Element<" + _sdf->GetName() + "> is missing",
+          _source);
       err.SetXmlPath(_sdf->XmlPath());
       _errors.push_back(err);
       return false;
@@ -1234,12 +1517,6 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
     }
   }
 
-  if (_xml->GetText() != nullptr && _sdf->GetValue())
-  {
-    if (!_sdf->GetValue()->SetFromString(_xml->GetText()))
-      return false;
-  }
-
   // check for nested sdf
   std::string refSDFStr = _sdf->ReferenceSDF();
   if (!refSDFStr.empty())
@@ -1251,7 +1528,7 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
     ElementPtr refSDF;
     refSDF.reset(new Element);
     std::string refFilename = refSDFStr + ".sdf";
-    initFile(refFilename, refSDF);
+    initFile(refFilename, _config, refSDF);
     _sdf->RemoveFromParent();
     _sdf->Copy(refSDF);
 
@@ -1261,9 +1538,22 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
       _sdf->SetLineNumber(lineNumber.value());
   }
 
-  if (!readAttributes(_xml, _sdf, _config, errorSourcePath, _errors))
+  if (!readAttributes(_xml, _sdf, _config, _source, _errors))
     return false;
 
+  if (_xml->GetText() != nullptr && _sdf->GetValue())
+  {
+    if (!_sdf->GetValue()->SetFromString(_xml->GetText()))
+      return false;
+  }
+  else if (_sdf->GetValue())
+  {
+    if (!_sdf->GetValue()->Reparse())
+      return false;
+    if (!_sdf->GetValue()->SetFromString(""))
+      return false;
+  }
+
   if (_sdf->GetCopyChildren())
   {
     copyChildren(_sdf, _xml, false);
@@ -1282,6 +1572,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
     {
       if (std::string("include") == elemXml->Value())
       {
+        validateIncludeElement(elemXml, _sdf, _config, _source, _errors);
+
         tinyxml2::XMLElement *uriElement = elemXml->FirstChildElement("uri");
 
         const std::string includeXmlPath = _sdf->XmlPath() + "/include[" +
@@ -1289,7 +1581,7 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
         const std::string uriXmlPath = includeXmlPath + "/uri";
 
         if (!resolveFileNameFromUri(elemXml, _config, includeXmlPath,
-                errorSourcePath, filename, _errors))
+                _source, filename, _errors))
           continue;
 
         // If the file is not an SDFormat file, it is assumed that it will
@@ -1301,31 +1593,32 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
           // a new sdf pointer is created here by cloning a fresh sdf template
           // pointer instead of calling init every iteration.
           // SDFPtr includeSDF(new SDF);
-          // init(includeSDF);
+          // init(includeSDF, _config);
           static SDFPtr includeSDFTemplate;
           if (!includeSDFTemplate)
           {
             includeSDFTemplate.reset(new SDF);
-            init(includeSDFTemplate);
+            init(includeSDFTemplate, _config);
           }
           SDFPtr includeSDF(new SDF);
           includeSDF->Root(includeSDFTemplate->Root()->Clone());
 
-          if (!readFile(filename, includeSDF))
+          if (!readFile(filename, _config, includeSDF, _errors))
           {
             Error err(
                 ErrorCode::FILE_READ,
                 "Unable to read file[" + filename + "]",
-                errorSourcePath, uriElement->GetLineNum());
+                _source,
+                uriElement->GetLineNum());
             err.SetXmlPath(uriXmlPath);
             _errors.push_back(err);
             return false;
           }
 
-          // For now there is only a warning if there is more than one model,
-          // actor or light element, or two different types of those elements.
-          // For compatibility with old behavior, this chooses the first element
-          // in the preference order: model->actor->light
+          // Emit an error if there is more than one model, actor or light
+          // element, or two different types of those elements. For
+          // compatibility with old behavior, this chooses the first element in
+          // the preference order: model->actor->light
           sdf::ElementPtr topLevelElem;
           for (const auto &elementType : {"model", "actor", "light"})
           {
@@ -1340,13 +1633,11 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                 std::stringstream ss;
                 ss << "Found other top level element <" << elementType
                   << "> in addition to <" << topLevelElem->GetName()
-                  << "> in include file. This is unsupported and in future "
-                  << "versions of libsdformat will become an error";
+                  << "> in include file.";
                 Error err(
                     ErrorCode::ELEMENT_INCORRECT_TYPE, ss.str(), filename);
                 err.SetXmlPath("/sdf/" + std::string(elementType));
-                enforceConfigurablePolicyCondition(
-                    _config.WarningsPolicy(), err, _errors);
+                _errors.push_back(err);
               }
             }
           }
@@ -1357,7 +1648,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                 ErrorCode::ELEMENT_MISSING,
                 "Failed to find top level <model> / <actor> / <light> for "
                 "<include>\n",
-                errorSourcePath, uriElement->GetLineNum());
+                _source,
+                uriElement->GetLineNum());
             err.SetXmlPath(uriXmlPath);
             _errors.push_back(err);
             continue;
@@ -1370,14 +1662,12 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
           if (nullptr != nextTopLevelElem)
           {
             std::stringstream ss;
-            ss << "Found more than one of " << topLevelElem->GetName()
-              << " for <include>. This is unsupported and in future "
-              << "versions of libsdformat will become an error";
+            ss << "Found more than one " << topLevelElem->GetName()
+              << " for <include>.";
             Error err(
                 ErrorCode::ELEMENT_INCORRECT_TYPE, ss.str(), filename);
             err.SetXmlPath("/sdf/" + topLevelElementType);
-            enforceConfigurablePolicyCondition(
-                _config.WarningsPolicy(), err, _errors);
+            _errors.push_back(err);
           }
 
           bool isModel = topLevelElementType == "model";
@@ -1398,6 +1688,25 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
           {
             sdf::ElementPtr poseElem = topLevelElem->GetElement("pose");
 
+            auto setAttribute =
+                [&poseElem, &poseElemXml](const std::string &_attribName)
+            {
+              const char *attrib = poseElemXml->Attribute(_attribName.c_str());
+              auto attribParam = poseElem->GetAttribute(_attribName);
+              if (attrib && attribParam)
+              {
+                attribParam->SetFromString(attrib);
+              }
+              else if (attribParam)
+              {
+                attribParam->Reset();
+              }
+            };
+
+            setAttribute("relative_to");
+            setAttribute("degrees");
+            setAttribute("rotation_format");
+
             if (poseElemXml->GetText())
             {
               poseElem->GetValue()->SetFromString(poseElemXml->GetText());
@@ -1406,16 +1715,6 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
             {
               poseElem->GetValue()->Reset();
             }
-
-            const char *relativeTo = poseElemXml->Attribute("relative_to");
-            if (relativeTo)
-            {
-              poseElem->GetAttribute("relative_to")->SetFromString(relativeTo);
-            }
-            else
-            {
-              poseElem->GetAttribute("relative_to")->Reset();
-            }
           }
 
           if (isModel && elemXml->FirstChildElement("static"))
@@ -1436,7 +1735,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                   ErrorCode::MODEL_PLACEMENT_FRAME_INVALID,
                   "<pose> is required when specifying the placement_frame "
                   "element",
-                  errorSourcePath, elemXml->GetLineNum());
+                  _source,
+                  elemXml->GetLineNum());
               err.SetXmlPath(placementFrameXmlPath);
               _errors.push_back(err);
               return false;
@@ -1451,7 +1751,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                   "'" + placementFrameVal +
                   "' is reserved; it cannot be used as a value of "
                   "element [placement_frame]",
-                  errorSourcePath, placementFrameElem->GetLineNum());
+                  _source,
+                  placementFrameElem->GetLineNum());
               err.SetXmlPath(placementFrameXmlPath);
               _errors.push_back(err);
             }
@@ -1484,7 +1785,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                   Error err(
                       ErrorCode::ELEMENT_INVALID,
                       "Error reading plugin element",
-                      errorSourcePath, childElemXml->GetLineNum());
+                      _source,
+                      childElemXml->GetLineNum());
                   err.SetXmlPath(pluginXmlPath);
                   _errors.push_back(err);
                   return false;
@@ -1506,7 +1808,6 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
           }
 
           auto includeSDFFirstElem = includeSDF->Root()->GetFirstElement();
-          includeSDFFirstElem->SetParent(_sdf);
           auto includeDesc = _sdf->GetElementDescription("include");
           if (includeDesc)
           {
@@ -1516,8 +1817,12 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
             copyChildren(includeInfo, elemXml, false);
             includeSDFFirstElem->SetIncludeElement(includeInfo);
           }
-          _sdf->InsertElement(includeSDFFirstElem);
+          bool toMerge = elemXml->BoolAttribute("merge", false);
+          SourceLocation sourceLoc{includeXmlPath, _source,
+                                   elemXml->GetLineNum()};
 
+          insertIncludedElement(includeSDF, sourceLoc, toMerge, _sdf, _config,
+                                _errors);
           continue;
         }
       }
@@ -1549,7 +1854,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                 ErrorCode::ELEMENT_INVALID,
                 std::string("Error reading element <") +
                 elemXml->Value() + ">",
-                errorSourcePath, elemXml->GetLineNum());
+                _source,
+                elemXml->GetLineNum());
             err.SetXmlPath(elemXmlPath);
             _errors.push_back(err);
             return false;
@@ -1574,7 +1880,9 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
 
         Error err(
             ErrorCode::ELEMENT_INCORRECT_TYPE,
-            ss.str(), errorSourcePath, elemXml->GetLineNum());
+            ss.str(),
+            _source,
+            elemXml->GetLineNum());
         err.SetXmlPath(elemXmlPath);
         enforceConfigurablePolicyCondition(
             _config.UnrecognizedElementsPolicy(), err, _errors);
@@ -1605,7 +1913,8 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
                 ErrorCode::ELEMENT_MISSING,
                 "XML Missing required element[" + elemDesc->GetName() +
                 "], child of element[" + _sdf->GetName() + "]",
-                errorSourcePath, _xml->GetLineNum());
+                _source,
+                _xml->GetLineNum());
             missingElementError.SetXmlPath(elemXmlPath);
             _errors.push_back(missingElementError);
             return false;
@@ -1624,65 +1933,6 @@ bool readXml(tinyxml2::XMLElement *_xml, ElementPtr _sdf,
   return true;
 }
 
-/////////////////////////////////////////////////
-void copyChildren(ElementPtr _sdf,
-                  tinyxml2::XMLElement *_xml,
-                  const bool _onlyUnknown)
-{
-  // Iterate over all the child elements
-  tinyxml2::XMLElement *elemXml = nullptr;
-  for (elemXml = _xml->FirstChildElement(); elemXml;
-       elemXml = elemXml->NextSiblingElement())
-  {
-    std::string elem_name = elemXml->Name();
-
-    if (_sdf->HasElementDescription(elem_name))
-    {
-      if (!_onlyUnknown)
-      {
-        sdf::ElementPtr element = _sdf->AddElement(elem_name);
-
-        // FIXME: copy attributes
-        for (const auto *attribute = elemXml->FirstAttribute();
-             attribute; attribute = attribute->Next())
-        {
-          element->GetAttribute(attribute->Name())->SetFromString(
-            attribute->Value());
-        }
-
-        // copy value
-        const char *value = elemXml->GetText();
-        if (value)
-        {
-          element->GetValue()->SetFromString(value);
-        }
-        copyChildren(element, elemXml, _onlyUnknown);
-      }
-    }
-    else
-    {
-      ElementPtr element(new Element);
-      element->SetParent(_sdf);
-      element->SetName(elem_name);
-      if (elemXml->GetText() != nullptr)
-      {
-        element->AddValue("string", elemXml->GetText(), "1");
-      }
-
-      for (const tinyxml2::XMLAttribute *attribute = elemXml->FirstAttribute();
-           attribute; attribute = attribute->Next())
-      {
-        element->AddAttribute(attribute->Name(), "string", "", 1, "");
-        element->GetAttribute(attribute->Name())->SetFromString(
-          attribute->Value());
-      }
-
-      copyChildren(element, elemXml, _onlyUnknown);
-      _sdf->InsertElement(element);
-    }
-  }
-}
-
 /////////////////////////////////////////////////
 bool convertFile(const std::string &_filename, const std::string &_version,
                  SDFPtr _sdf)
@@ -2358,5 +2608,11 @@ bool shouldValidateElement(sdf::ElementPtr _elem)
 
   return true;
 }
+
+/////////////////////////////////////////////////
+std::string computeMergedModelProxyFrameName(const std::string &_modelName)
+{
+  return "_merged__" + _modelName + "__model__";
+}
 }
 }
