@@ -32,6 +32,7 @@
 #include "sdf/InterfaceModel.hh"
 #include "sdf/Model.hh"
 #include "sdf/Param.hh"
+#include "sdf/PrintConfig.hh"
 #include "sdf/Root.hh"
 #include "sdf/Types.hh"
 #include "sdf/World.hh"
@@ -60,7 +61,8 @@ sdf::InterfaceModelPtr parseModel(toml::Value &_doc,
 
   for (auto &[name, frame] : _doc["frames"].Map())
   {
-    const auto attachedTo = frame["attached_to"].ParamGet<std::string>();
+    const auto attachedTo =
+        frame["attached_to"].ParamGet<std::string>("__model__");
     const auto pose = frame["pose"].ParamGet<ignition::math::Pose3d>();
     model->AddFrame({name, attachedTo, pose});
   }
@@ -79,35 +81,53 @@ sdf::InterfaceModelPtr parseModel(toml::Value &_doc,
   return model;
 }
 
-sdf::InterfaceModelPtr customTomlParser(
-    const sdf::NestedInclude &_include, sdf::Errors &_errors)
+class CustomTomlParser
 {
-  toml::Document doc = toml::parseToml(_include.resolvedFileName, _errors);
-  if (_errors.empty())
+  /// \brief Constructor
+  /// \param[in] _supportsMergeInclude Whether the parser supports merge include
+  /// \param[in] _overridePoseInParser Whether the parser should apply pose
+  /// overrides from //include/pose
+  public: CustomTomlParser(bool _supportsMergeInclude = true,
+                           bool _overridePoseInParser = true)
+      : supportsMergeInclude(_supportsMergeInclude),
+        overridePoseInParser(_overridePoseInParser)
   {
-    const std::string modelName =
-        _include.localModelName.value_or(doc["name"].ParamGet<std::string>());
-
-    if (_include.isStatic.has_value())
-    {
-      // if //include/static is set, override the value in the inluded model
-      sdf::Param param("static", "bool", "false", false);
-      param.Set(*_include.isStatic);
-      doc["static"] = {param};
-    }
-    if (_include.includeRawPose.has_value())
-    {
-      // if //include/static is set, override the value in the inluded model
-      sdf::Param poseParam("pose", "pose", "", false);
-      poseParam.Set(*_include.includeRawPose);
-      doc["pose"] = {poseParam};
-    }
-
-    return parseModel(doc, modelName);
   }
-  return nullptr;
-}
 
+  public: sdf::InterfaceModelPtr operator()(const sdf::NestedInclude &_include,
+                                            sdf::Errors &_errors)
+  {
+    toml::Document doc = toml::parseToml(_include.ResolvedFileName(), _errors);
+    if (_errors.empty())
+    {
+      const std::string modelName =
+        _include.LocalModelName().value_or(doc["name"].ParamGet<std::string>());
+
+      if (_include.IsStatic().has_value())
+      {
+        // if //include/static is set, override the value in the inluded model
+        sdf::Param param("static", "bool", "false", false);
+        param.Set(*_include.IsStatic());
+        doc["static"] = {param};
+      }
+      if (this->overridePoseInParser && _include.IncludeRawPose().has_value())
+      {
+        // if //include/static is set, override the value in the inluded model
+        sdf::Param poseParam("pose", "pose", "", false);
+        poseParam.Set(*_include.IncludeRawPose());
+        doc["pose"] = {poseParam};
+      }
+
+      auto model = parseModel(doc, modelName);
+      model->SetParserSupportsMergeInclude(this->supportsMergeInclude);
+      return model;
+    }
+    return nullptr;
+  }
+
+  public: bool supportsMergeInclude;
+  public: bool overridePoseInParser{true};
+};
 
 bool endsWith(const std::string &_str, const std::string &_suffix)
 {
@@ -131,9 +151,11 @@ class InterfaceAPI : public ::testing::Test
           return sdf::filesystem::append(modelDir, _file);
         });
   }
+  public: void CheckFrameSemantics(const sdf::World *world);
 
   public: std::string modelDir;
   public: sdf::ParserConfig config;
+  public: CustomTomlParser customTomlParser;
 };
 
 /////////////////////////////////////////////////
@@ -162,26 +184,26 @@ TEST_F(InterfaceAPI, NestedIncludeData)
   auto testNonce1Parser =
       [&](const sdf::NestedInclude &_include, sdf::Errors &_errors)
   {
-    if (!endsWith(_include.uri, ".nonce_1"))
+    if (!endsWith(_include.Uri(), ".nonce_1"))
       return nullptr;
 
     const std::string fileName = "file_wont_be_parsed.nonce_1";
-    EXPECT_EQ(fileName, _include.uri);
+    EXPECT_EQ(fileName, _include.Uri());
     EXPECT_EQ(sdf::filesystem::append(this->modelDir, fileName),
-        _include.resolvedFileName);
-    EXPECT_EQ("box", *_include.localModelName);
-    EXPECT_TRUE(_include.isStatic.has_value());
-    EXPECT_TRUE(_include.isStatic.value());
+              _include.ResolvedFileName());
+    EXPECT_EQ("box", *_include.LocalModelName());
+    EXPECT_TRUE(_include.IsStatic().has_value());
+    EXPECT_TRUE(_include.IsStatic().value());
 
-    EXPECT_TRUE(_include.includeRawPose.has_value());
+    EXPECT_TRUE(_include.IncludeRawPose().has_value());
     EXPECT_EQ(ignition::math::Pose3d(1, 0, 0, 0, 0, 0),
-        _include.includeRawPose.value());
+              _include.IncludeRawPose().value());
 
-    EXPECT_TRUE(_include.includePoseRelativeTo.has_value());
-    EXPECT_EQ("F1", _include.includePoseRelativeTo.value());
-    EXPECT_TRUE(_include.includeElement->HasElement("extra"));
+    EXPECT_TRUE(_include.IncludePoseRelativeTo().has_value());
+    EXPECT_EQ("F1", _include.IncludePoseRelativeTo().value());
+    EXPECT_TRUE(_include.IncludeElement()->HasElement("extra"));
 
-    auto extra = _include.includeElement->GetElement("extra");
+    auto extra = _include.IncludeElement()->GetElement("extra");
     EXPECT_TRUE(extra->HasElement("info1"));
     EXPECT_TRUE(extra->HasElement("info2"));
     EXPECT_EQ("value1", extra->Get<std::string>("info1"));
@@ -196,14 +218,14 @@ TEST_F(InterfaceAPI, NestedIncludeData)
   auto testNonce2Parser =
       [&](const sdf::NestedInclude &_include, sdf::Errors &_errors)
   {
-    if (!endsWith(_include.uri, ".nonce_2"))
+    if (!endsWith(_include.Uri(), ".nonce_2"))
       return nullptr;
     const std::string fileName = "file_wont_be_parsed.nonce_2";
-    EXPECT_EQ(fileName, _include.uri);
-    EXPECT_EQ(
-        sdf::filesystem::append(modelDir, fileName), _include.resolvedFileName);
-    EXPECT_FALSE(_include.localModelName.has_value());
-    EXPECT_FALSE(_include.isStatic);
+    EXPECT_EQ(fileName, _include.Uri());
+    EXPECT_EQ(sdf::filesystem::append(modelDir, fileName),
+              _include.ResolvedFileName());
+    EXPECT_FALSE(_include.LocalModelName().has_value());
+    EXPECT_FALSE(_include.IsStatic());
 
     // Add error for test expectation later on.
     _errors.emplace_back(
@@ -268,7 +290,7 @@ TEST_F(InterfaceAPI, CustomParserPrecedence)
   this->config.RegisterCustomModelParser(createCustomParser(3));
   sdf::Root root;
   sdf::Errors errors = root.LoadSdfString(testSdf, this->config);
-  EXPECT_TRUE(errors.empty());
+  EXPECT_TRUE(errors.empty()) << errors;
   // Check that custom parsers are called in reverse order of registration.
   ASSERT_EQ(3u, customParserCallOrder.size());
   // Parser 0 will not be visited because parser 1 will be successful.
@@ -299,7 +321,7 @@ void TomlParserTest(const sdf::InterfaceModelConstPtr &_interfaceModel)
     EXPECT_EQ(expLinks[link.Name()], link.PoseInModelFrame());
   }
   std::map <std::string, std::pair<std::string, Pose3d>> expFrames = {
-      {"frame_1", {"", Pose3d(0, 1, 0.0, 0, 0, 0)}},
+      {"frame_1", {"__model__", Pose3d(0, 1, 0.0, 0, 0, 0)}},
       {"frame_2", {"lower_link", Pose3d(0, 0, 1, 0, 0, 0)}},
   };
 
@@ -343,7 +365,7 @@ TEST_F(InterfaceAPI, TomlParserWorldInclude)
   const std::string testFile = sdf::testing::TestFile(
       "sdf", "world_include_with_interface_api.sdf");
 
-  this->config.RegisterCustomModelParser(customTomlParser);
+  this->config.RegisterCustomModelParser(this->customTomlParser);
   sdf::Root root;
   sdf::Errors errors = root.Load(testFile, this->config);
   EXPECT_TRUE(errors.empty()) << errors;
@@ -362,7 +384,7 @@ TEST_F(InterfaceAPI, TomlParserModelInclude)
   const std::string testFile = sdf::testing::TestFile(
       "sdf", "model_include_with_interface_api.sdf");
 
-  this->config.RegisterCustomModelParser(customTomlParser);
+  this->config.RegisterCustomModelParser(this->customTomlParser);
   sdf::Root root;
   sdf::Errors errors = root.Load(testFile, this->config);
   EXPECT_TRUE(errors.empty()) << errors;
@@ -375,21 +397,9 @@ TEST_F(InterfaceAPI, TomlParserModelInclude)
   TomlParserTest(interfaceModel);
 }
 
-/////////////////////////////////////////////////
-TEST_F(InterfaceAPI, FrameSemantics)
+void InterfaceAPI::CheckFrameSemantics(const sdf::World *world)
 {
   using ignition::math::Pose3d;
-  const std::string testFile = sdf::testing::TestFile(
-      "sdf", "include_with_interface_api_frame_semantics.sdf");
-
-  this->config.RegisterCustomModelParser(customTomlParser);
-  sdf::Root root;
-  sdf::Errors errors = root.Load(testFile, config);
-  EXPECT_TRUE(errors.empty()) << errors;
-
-  const sdf::World *world = root.WorldByIndex(0);
-  ASSERT_NE(nullptr, world);
-  EXPECT_EQ(1u, world->InterfaceModelCount());
 
   auto resolvePoseNoErrors =
       [](const sdf::SemanticPose &_semPose, const std::string &_relativeTo = "")
@@ -543,6 +553,42 @@ TEST_F(InterfaceAPI, FrameSemantics)
   }
 }
 
+/////////////////////////////////////////////////
+TEST_F(InterfaceAPI, FrameSemantics)
+{
+  const std::string testFile = sdf::testing::TestFile(
+      "sdf", "include_with_interface_api_frame_semantics.sdf");
+  this->config.RegisterCustomModelParser(this->customTomlParser);
+  {
+    sdf::Root root;
+    sdf::Errors errors = root.Load(testFile, config);
+    EXPECT_TRUE(errors.empty()) << errors;
+
+    const sdf::World *world = root.WorldByIndex(0);
+    ASSERT_NE(nullptr, world);
+    EXPECT_EQ(1u, world->InterfaceModelCount());
+
+    SCOPED_TRACE("InterfaceAPI.FrameSemantics");
+    this->CheckFrameSemantics(world);
+  }
+  {
+    // Check without //include/pose override applied in parser.
+    sdf::Root root;
+    sdf::ParserConfig newConfig = this->config;
+    CustomTomlParser parserWithoutPoseOverride(true, false);
+    newConfig.RegisterCustomModelParser(parserWithoutPoseOverride);
+    sdf::Errors errors = root.Load(testFile, newConfig);
+    EXPECT_TRUE(errors.empty()) << errors;
+
+    const sdf::World *world = root.WorldByIndex(0);
+    ASSERT_NE(nullptr, world);
+    EXPECT_EQ(1u, world->InterfaceModelCount());
+
+    SCOPED_TRACE("InterfaceAPI.FrameSemantics_NoPoseOverrideInParser");
+    this->CheckFrameSemantics(world);
+  }
+}
+
 /////////////////////////////////////////////////
 TEST_F(InterfaceAPI, Reposturing)
 {
@@ -588,17 +634,19 @@ TEST_F(InterfaceAPI, Reposturing)
   auto repostureTestParser = [&](const sdf::NestedInclude &_include,
                                  sdf::Errors &) -> sdf::InterfaceModelPtr
   {
-    bool fileHasCorrectSuffix = endsWith(_include.resolvedFileName, ".nonce_1");
-    EXPECT_TRUE(fileHasCorrectSuffix) << "File: " << _include.resolvedFileName;
+    bool fileHasCorrectSuffix =
+        endsWith(_include.ResolvedFileName(), ".nonce_1");
+    EXPECT_TRUE(fileHasCorrectSuffix)
+        << "File: " << _include.ResolvedFileName();
     if (!fileHasCorrectSuffix)
       return nullptr;
 
-    const std::string absoluteModelName =
-        sdf::JoinName(_include.absoluteParentName, *_include.localModelName);
+    const std::string absoluteModelName = sdf::JoinName(
+        _include.AbsoluteParentName(), *_include.LocalModelName());
 
-    auto model = std::make_shared<sdf::InterfaceModel>(*_include.localModelName,
-        makeRepostureFunc(absoluteModelName), false, "base_link",
-        _include.includeRawPose.value_or(Pose3d {}));
+    auto model = std::make_shared<sdf::InterfaceModel>(
+        *_include.LocalModelName(), makeRepostureFunc(absoluteModelName), false,
+        "base_link", _include.IncludeRawPose().value_or(Pose3d{}));
     model->AddLink({"base_link", {}});
     models[absoluteModelName] = model;
 
@@ -639,6 +687,7 @@ TEST_F(InterfaceAPI, Reposturing)
 
     return testing::AssertionSuccess();
   };
+
   // There are two included models using a custom parser.
   // In each of the included models, there are two models and two links.
   ASSERT_EQ(8u, posesAfterReposture.size());
@@ -662,8 +711,8 @@ TEST_F(InterfaceAPI, PlacementFrame)
   auto repostureTestParser =
       [&](const sdf::NestedInclude &_include, sdf::Errors &)
   {
-    std::string modelName =
-        sdf::JoinName(_include.absoluteParentName, *_include.localModelName);
+    std::string modelName = sdf::JoinName(_include.AbsoluteParentName(),
+                                          *_include.LocalModelName());
     auto repostureFunc = [modelName = modelName, &modelPosesAfterReposture](
                              const sdf::InterfaceModelPoseGraph &_graph)
     {
@@ -673,9 +722,9 @@ TEST_F(InterfaceAPI, PlacementFrame)
       modelPosesAfterReposture[modelName] = pose;
     };
 
-    auto model = std::make_shared<sdf::InterfaceModel>(*_include.localModelName,
-        repostureFunc, false, "base_link",
-        _include.includeRawPose.value_or(Pose3d {}));
+    auto model = std::make_shared<sdf::InterfaceModel>(
+        *_include.LocalModelName(), repostureFunc, false, "base_link",
+        _include.IncludeRawPose().value_or(Pose3d{}));
     model->AddLink({"base_link", Pose3d(0, 1, 0, 0, 0, 0)});
     model->AddFrame({"frame_1", "__model__", Pose3d(0, 0, 1, 0, 0, 0)});
     return model;
@@ -817,7 +866,7 @@ TEST_F(InterfaceAPI, NameCollision)
 {
   using ignition::math::Pose3d;
 
-  this->config.RegisterCustomModelParser(customTomlParser);
+  this->config.RegisterCustomModelParser(this->customTomlParser);
 
   // ---------------- Name collision in //world/include ----------------
   {
@@ -859,6 +908,370 @@ TEST_F(InterfaceAPI, NameCollision)
   }
 }
 
+class InterfaceAPIMergeInclude : public InterfaceAPI
+{
+};
+
+/////////////////////////////////////////////////
+TEST_F(InterfaceAPIMergeInclude, MergeIncludeNotSupported)
+{
+  const std::string testSdf = R"(
+  <sdf version="1.9">
+    <model name="parent_model">
+      <include merge="true">
+        <uri>double_pendulum.toml</uri>
+      </include>
+    </model>
+  </sdf>)";
+  CustomTomlParser parserWithNoMergeInclude(false);
+  this->config.RegisterCustomModelParser(parserWithNoMergeInclude);
+  sdf::Root root;
+  sdf::Errors errors = root.LoadSdfString(testSdf, this->config);
+  ASSERT_FALSE(errors.empty());
+  EXPECT_EQ(sdf::ErrorCode::MERGE_INCLUDE_UNSUPPORTED, errors[0].Code());
+}
+
+/////////////////////////////////////////////////
+TEST_F(InterfaceAPIMergeInclude, FrameSemantics)
+{
+  const std::string testFile = sdf::testing::TestFile(
+      "sdf", "merge_include_with_interface_api_frame_semantics.sdf");
+  this->config.RegisterCustomModelParser(this->customTomlParser);
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile, config);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_NE(nullptr, world);
+
+  SCOPED_TRACE("InterfaceAPIMergeInclude.FrameSemantics");
+  this->CheckFrameSemantics(world);
+}
+
+/////////////////////////////////////////////////
+TEST_F(InterfaceAPIMergeInclude, Reposturing)
+{
+  using ignition::math::Pose3d;
+  const std::string testFile = sdf::testing::TestFile(
+      "sdf", "merge_include_with_interface_api_reposture.sdf");
+
+  std::unordered_map<std::string, Pose3d> posesAfterReposture;
+  std::unordered_map<std::string, std::vector<std::string>> elementsToReposture;
+
+  // Create a resposture callback function for a given absolute model name. The
+  // name is used to store poses in `posesAfterReposture`.
+  auto makeRepostureFunc = [&](const std::string &_absoluteName)
+  {
+    auto repostureFunc =
+        [modelName = _absoluteName, &elementsToReposture, &posesAfterReposture](
+            const sdf::InterfaceModelPoseGraph &_graph)
+    {
+      auto modelIt = elementsToReposture.find(modelName);
+      ASSERT_TRUE(modelIt != elementsToReposture.end());
+
+      for (const auto &elem : modelIt->second)
+      {
+        ignition::math::Pose3d pose;
+        sdf::Errors errors =
+            _graph.ResolveNestedFramePose(pose, elem);
+        EXPECT_TRUE(errors.empty()) << errors;
+        posesAfterReposture[sdf::JoinName(modelName, elem)] = pose;
+      }
+    };
+    return repostureFunc;
+  };
+
+  auto repostureTestParser = [&](const sdf::NestedInclude &_include,
+                                 sdf::Errors &) -> sdf::InterfaceModelPtr
+  {
+    bool fileHasCorrectSuffix =
+        endsWith(_include.ResolvedFileName(), ".nonce_1");
+    EXPECT_TRUE(fileHasCorrectSuffix)
+        << "File: " << _include.ResolvedFileName();
+    if (!fileHasCorrectSuffix)
+      return nullptr;
+
+    // Use parent name because we know merge=true
+    const std::string &absoluteModelName = _include.AbsoluteParentName();
+    // The following is equivalent to
+    // <model name="M0"> <!-- Merged into parent model
+    //   <pose relative_to="F1">0 0 0  π/2 0 0</pose> <!-- From //include -->
+    //   <!-- World pose of M0: (1 2 3   π/2 0 0)
+    //   <link name="base_link"/> <!-- World pose: (1 2 3   π/2 0 0) -->
+    //   <link name="top_link">
+    //     <pose>0 0 1   0 0 0</pose> <!-- World pose: (1 1 3   π/2 0 0) -->
+    //   </link>
+    //   <joint name="j1" type="fixed">
+    //     <pose>1 0 0   0 0 0</pose> <!-- World pose: (2 1 3   0 0 0) -->
+    //     <parent>base_link</parent>
+    //     <child>top_link</child>
+    //   </joint>
+    //   <frame name="frame1">
+    //     <pose>0 1 0   0 0 0</pose> <!-- World pose: (1 2 4   π/2 0 0) -->
+    //   </frame>
+    //   <frame name="frame2" attached_to="frame1">
+    //     <pose>0 0 1   0 0 0</pose> <!-- World pose: (1 1 4   π/2 0 0) -->
+    //   </frame>
+    //   <model name="nested_model">
+    //     <pose>3 0 0   0 0 0</pose> <!-- World pose: (4 2 3   π/2 0 0) -->
+    //     <link name="nested_link">
+    //       <pose>0 0 0   π/2 0 0</pose> <!-- World pose: (4 2 3   π 0 0) -->
+    //     </link>
+    //   </model>
+    // </model>
+    auto model = std::make_shared<sdf::InterfaceModel>(
+        *_include.LocalModelName(), makeRepostureFunc(absoluteModelName), false,
+        "base_link", Pose3d{});
+    model->AddLink({"base_link", {}});
+    model->AddLink({"top_link", Pose3d(0, 0, 1, 0, 0, 0)});
+    model->AddJoint({"j1", "top_link", Pose3d(1, 0, 0, 0, 0, 0)});
+    model->AddFrame({"frame1", "__model__", Pose3d(0, 1, 0, 0, 0, 0)});
+    model->AddFrame({"frame2", "frame1", Pose3d(0, 0, 1, 0, 0, 0)});
+    elementsToReposture[absoluteModelName].emplace_back("base_link");
+    elementsToReposture[absoluteModelName].emplace_back("top_link");
+    elementsToReposture[absoluteModelName].emplace_back("j1");
+    elementsToReposture[absoluteModelName].emplace_back("frame1");
+    elementsToReposture[absoluteModelName].emplace_back("frame2");
+
+    elementsToReposture[absoluteModelName].emplace_back("nested_model");
+    const std::string absoluteNestedModelName =
+        sdf::JoinName(absoluteModelName, "nested_model");
+    auto nestedModel = std::make_shared<sdf::InterfaceModel>("nested_model",
+        makeRepostureFunc(absoluteNestedModelName), false, "nested_link",
+        Pose3d(3, 0, 0, 0, 0, 0));
+
+    nestedModel->AddLink({"nested_link", Pose3d(0, 0, 0, IGN_PI_2, 0, 0)});
+    elementsToReposture[absoluteNestedModelName].emplace_back("nested_link");
+
+    model->AddNestedModel(nestedModel);
+    model->SetParserSupportsMergeInclude(true);
+    return model;
+  };
+
+  this->config.RegisterCustomModelParser(repostureTestParser);
+  this->config.SetFindCallback(
+      [](const auto &_fileName)
+      {
+        return _fileName;
+      });
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile, this->config);
+  EXPECT_TRUE(errors.empty()) << errors;
+  auto checkPose =
+      [&posesAfterReposture](
+          const std::string &_name, const Pose3d &_expectedPose)
+  {
+    auto it = posesAfterReposture.find(_name);
+    if (it == posesAfterReposture.end())
+      return testing::AssertionFailure() << _name << " not found in map";
+
+    if (_expectedPose != it->second)
+    {
+      return testing::AssertionFailure()
+          << "Expected pose: " << _expectedPose << " actual: " << it->second;
+    }
+
+    return testing::AssertionSuccess();
+  };
+  // There is one included model using a custom parser containing two models and
+  // two links.
+  ASSERT_EQ(7u, posesAfterReposture.size());
+  EXPECT_TRUE(checkPose("parent_model::base_link", {1, 2, 3, IGN_PI_2, 0, 0}));
+  EXPECT_TRUE(checkPose("parent_model::top_link", {1, 1, 3, IGN_PI_2, 0, 0}));
+  EXPECT_TRUE(checkPose("parent_model::j1", {2, 1, 3, IGN_PI_2, 0, 0}));
+  EXPECT_TRUE(checkPose("parent_model::frame1", {1, 2, 4, IGN_PI_2, 0, 0}));
+  EXPECT_TRUE(checkPose("parent_model::frame2", {1, 1, 4, IGN_PI_2, 0, 0}));
+  EXPECT_TRUE(
+      checkPose("parent_model::nested_model", {4, 2, 3, IGN_PI_2, 0, 0}));
+  EXPECT_TRUE(checkPose(
+      "parent_model::nested_model::nested_link", {4, 2, 3, IGN_PI, 0, 0}));
+}
+
+
+/////////////////////////////////////////////////
+TEST_F(InterfaceAPIMergeInclude, PlacementFrame)
+{
+  using ignition::math::Pose3d;
+  std::unordered_map<std::string, Pose3d> modelPosesAfterReposture;
+  auto repostureTestParser =
+      [&](const sdf::NestedInclude &_include, sdf::Errors &)
+  {
+    std::string modelName = sdf::JoinName(_include.AbsoluteParentName(),
+                                          *_include.LocalModelName());
+    auto repostureFunc = [modelName = modelName, &modelPosesAfterReposture](
+                             const sdf::InterfaceModelPoseGraph &_graph)
+    {
+      ignition::math::Pose3d pose;
+      sdf::Errors errors = _graph.ResolveNestedModelFramePoseInWorldFrame(pose);
+      EXPECT_TRUE(errors.empty()) << errors;
+      modelPosesAfterReposture[modelName] = pose;
+    };
+
+    auto model = std::make_shared<sdf::InterfaceModel>(
+        *_include.LocalModelName(), repostureFunc, false, "base_link",
+        _include.IncludeRawPose().value_or(Pose3d{}));
+    model->AddLink({"base_link", Pose3d(0, 1, 0, 0, 0, 0)});
+    model->AddFrame({"frame_1", "__model__", Pose3d(0, 0, 1, 0, 0, 0)});
+    model->SetParserSupportsMergeInclude(true);
+    return model;
+  };
+
+  this->config.RegisterCustomModelParser(repostureTestParser);
+  this->config.SetFindCallback(
+      [](const auto &_fileName)
+      {
+        return _fileName;
+      });
+
+  // ---------------- Placement frame in //sdf/model/include ----------------
+  {
+    const std::string testSdf = R"(
+  <sdf version="1.9">
+    <model name="parent_model">
+      <include merge="true">
+        <uri>non_existent_file.test</uri>
+        <name>test_model</name>
+        <pose>1 0 0 0 0 0</pose>
+        <placement_frame>frame_1</placement_frame>
+      </include>
+      <frame name="test_frame"/>
+    </model>
+  </sdf>)";
+    sdf::Root root;
+    sdf::Errors errors = root.LoadSdfString(testSdf, this->config);
+    EXPECT_TRUE(errors.empty()) << errors;
+
+    const auto *parentModel = root.Model();
+    ASSERT_NE(nullptr, parentModel);
+    const auto *testFrame = parentModel->FrameByName("test_frame");
+    ASSERT_NE(nullptr, testFrame);
+    {
+      // Since there is no InterfaceModel::SemanticPose, we resolve the pose of
+      // test_frame relative to the test_model and take the inverse as the pose
+      // of the test_model relative to the world.
+      Pose3d pose;
+      sdf::Errors resolveErrors = testFrame->SemanticPose().Resolve(
+          pose, "_merged__test_model__model__");
+      EXPECT_TRUE(resolveErrors.empty()) << resolveErrors;
+      EXPECT_EQ(Pose3d(1, 0, -1, 0, 0, 0), pose.Inverse());
+    }
+    {
+      Pose3d pose;
+      sdf::Errors resolveErrors =
+          testFrame->SemanticPose().Resolve(pose, "frame_1");
+      EXPECT_TRUE(resolveErrors.empty()) << resolveErrors;
+      EXPECT_EQ(Pose3d(1, 0, 0, 0, 0, 0), pose.Inverse());
+    }
+  }
+
+  // ---------------- Placement frame in //world//model/include ----------------
+  {
+    const std::string testSdf = R"(
+  <sdf version="1.9">
+    <world name="default">
+      <model name="parent_model">
+        <include merge="true">
+          <uri>non_existent_file.test</uri>
+          <name>test_model</name>
+          <pose>1 0 0 0 0 0</pose>
+          <placement_frame>frame_1</placement_frame>
+        </include>
+        <frame name="test_frame"/>
+      </model>
+    </world>
+  </sdf>)";
+    sdf::Root root;
+    sdf::Errors errors = root.LoadSdfString(testSdf, this->config);
+    EXPECT_TRUE(errors.empty()) << errors;
+    const auto *world = root.WorldByIndex(0);
+    ASSERT_NE(nullptr, world);
+    const auto *parentModel = world->ModelByIndex(0);
+    ASSERT_NE(nullptr, parentModel);
+    const auto *testFrame = parentModel->FrameByName("test_frame");
+    ASSERT_NE(nullptr, testFrame);
+    {
+      // Since there is no InterfaceModel::SemanticPose, we resolve the pose of
+      // test_frame relative to the test_model and take the inverse as the pose
+      // of the test_model relative to the world.
+      Pose3d pose;
+      sdf::Errors resolveErrors = testFrame->SemanticPose().Resolve(
+          pose, "_merged__test_model__model__");
+      EXPECT_TRUE(resolveErrors.empty()) << resolveErrors;
+      EXPECT_EQ(Pose3d(1, 0, -1, 0, 0, 0), pose.Inverse());
+    }
+    {
+      Pose3d pose;
+      sdf::Errors resolveErrors =
+          testFrame->SemanticPose().Resolve(pose, "frame_1");
+      EXPECT_TRUE(resolveErrors.empty()) << resolveErrors;
+      EXPECT_EQ(Pose3d(1, 0, 0, 0, 0, 0), pose.Inverse());
+    }
+  }
+}
+
+/////////////////////////////////////////////////
+// Tests PrintConfig
+TEST_F(InterfaceAPI, TomlParserModelIncludePrintConfig)
+{
+  const std::string testFile = sdf::testing::TestFile(
+      "sdf", "model_include_with_interface_api.sdf");
+
+  this->config.RegisterCustomModelParser(customTomlParser);
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile, this->config);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(nullptr, model);
+  const sdf::ElementPtr includeElem = model->Element()->GetFirstElement();
+  ASSERT_NE(nullptr, includeElem);
+
+  const std::string expectedIncludeStr =
+R"(<include>
+  <uri>double_pendulum.toml</uri>
+  <pose>1 0 0 0 0 0</pose>
+  <extra>
+    <info1>value1</info1>
+    <info2>value2</info2>
+  </extra>
+</include>
+)";
+
+  sdf::PrintConfig printConfig;
+  EXPECT_EQ(includeElem->ToString("", printConfig), expectedIncludeStr);
+  printConfig.SetPreserveIncludes(true);
+  EXPECT_EQ(includeElem->ToString("", printConfig), expectedIncludeStr);
+}
+
+/////////////////////////////////////////////////
+TEST_F(InterfaceAPIMergeInclude, JointModelChild)
+{
+  const std::string testSdf = R"(
+  <sdf version="1.9">
+    <model name="parent_model">
+      <link name="L1"/>
+      <include merge="true">
+        <uri>joint_child_model_frame.toml</uri>
+      </include>
+      <frame name="frame_1" attached_to="joint_model_child"/>
+    </model>
+  </sdf>)";
+
+  this->config.RegisterCustomModelParser(customTomlParser);
+  sdf::Root root;
+  sdf::Errors errors = root.LoadSdfString(testSdf, this->config);
+  EXPECT_TRUE(errors.empty()) << errors;
+
+  const sdf::Model *model = root.Model();
+  ASSERT_NE(nullptr, model);
+  {
+    auto frame = model->FrameByName("frame_1");
+    std::string body;
+    frame->ResolveAttachedToBody(body);
+    EXPECT_EQ("base", body);
+  }
+}
+
 /////////////////////////////////////////////////
 TEST_F(InterfaceAPI, JointParentOrChildInNestedModel)
 {
