@@ -243,7 +243,7 @@ TEST(DOMLink, Sensors)
   const sdf::Link *link = model->LinkByIndex(0);
   ASSERT_NE(nullptr, link);
   EXPECT_EQ("link", link->Name());
-  EXPECT_EQ(22u, link->SensorCount());
+  EXPECT_EQ(26u, link->SensorCount());
 
   // Get the altimeter sensor
   const sdf::Sensor *altimeterSensor = link->SensorByIndex(0);
@@ -274,6 +274,9 @@ TEST(DOMLink, Sensors)
   EXPECT_EQ(ignition::math::Pose3d(0.1, 0.2, 0.3, 0, 0, 0),
             camSensor->RawPose());
   EXPECT_DOUBLE_EQ(0.75, camSensor->HorizontalFov().Radian());
+  EXPECT_EQ("", cameraSensor->Topic());
+  EXPECT_EQ("my_camera/trigger", camSensor->TriggerTopic());
+  EXPECT_TRUE(camSensor->Triggered());
   EXPECT_EQ(640u, camSensor->ImageWidth());
   EXPECT_EQ(480u, camSensor->ImageHeight());
   EXPECT_EQ(sdf::PixelFormatType::RGB_INT8, camSensor->PixelFormat());
@@ -318,6 +321,19 @@ TEST(DOMLink, Sensors)
   EXPECT_EQ(ignition::math::Pose3d(4, 5, 3, 0, 0, 0), pose);
   EXPECT_TRUE(contactSensor->EnableMetrics());
 
+  // Get the custom sensor
+  const sdf::Sensor *customSensor = link->SensorByName("custom_sensor");
+  ASSERT_NE(nullptr, customSensor);
+  EXPECT_EQ("custom_sensor", customSensor->Name());
+  EXPECT_EQ(sdf::SensorType::CUSTOM, customSensor->Type());
+  ASSERT_NE(nullptr, customSensor->Element());
+  auto myNamespaceElem = customSensor->Element()->GetElement(
+      "my_namespace:custom_sensor");
+  ASSERT_NE(nullptr, myNamespaceElem);
+  auto someParamElem = myNamespaceElem->GetElement("some_param");
+  ASSERT_NE(nullptr, someParamElem);
+  EXPECT_EQ(123, someParamElem->Get<int>());
+
   // Get the depth sensor
   const sdf::Sensor *depthSensor = link->SensorByName("depth_sensor");
   ASSERT_NE(nullptr, depthSensor);
@@ -339,6 +355,9 @@ TEST(DOMLink, Sensors)
   const sdf::Camera *rgbdCamSensor = rgbdSensor->CameraSensor();
   ASSERT_NE(nullptr, rgbdCamSensor);
   EXPECT_EQ("my_rgbd_camera", rgbdCamSensor->Name());
+  EXPECT_EQ("", rgbdSensor->Topic());
+  EXPECT_EQ("", rgbdCamSensor->TriggerTopic());
+  EXPECT_TRUE(rgbdCamSensor->Triggered());
 
   // Get the thermal sensor
   const sdf::Sensor *thermalSensor = link->SensorByName("thermal_sensor");
@@ -352,6 +371,35 @@ TEST(DOMLink, Sensors)
   ASSERT_NE(nullptr, thermalCamSensor);
   EXPECT_EQ("my_thermal_camera", thermalCamSensor->Name());
 
+  // Get the segmentation sensor
+  const sdf::Sensor *segmentationSensor =
+    link->SensorByName("segmentation_sensor");
+  ASSERT_NE(nullptr, segmentationSensor);
+  EXPECT_EQ("segmentation_sensor", segmentationSensor->Name());
+  EXPECT_EQ(sdf::SensorType::SEGMENTATION_CAMERA, segmentationSensor->Type());
+  EXPECT_EQ(ignition::math::Pose3d(37, 38, 39, 0, 0, 0),
+            segmentationSensor->RawPose());
+  const sdf::Camera *segmentationCameraSensor =
+    segmentationSensor->CameraSensor();
+  ASSERT_NE(nullptr, segmentationCameraSensor);
+  EXPECT_EQ("my_segmentation_camera", segmentationCameraSensor->Name());
+  EXPECT_TRUE(segmentationCameraSensor->HasSegmentationType());
+  EXPECT_EQ("semantic", segmentationCameraSensor->SegmentationType());
+
+  // Get the boundingbox sensor
+  const sdf::Sensor *boundingboxSensor =
+    link->SensorByName("boundingbox_sensor");
+  ASSERT_NE(nullptr, boundingboxSensor);
+  EXPECT_EQ("boundingbox_sensor", boundingboxSensor->Name());
+  EXPECT_EQ(sdf::SensorType::BOUNDINGBOX_CAMERA, boundingboxSensor->Type());
+  EXPECT_EQ(ignition::math::Pose3d(37, 38, 39, 0, 0, 0),
+            boundingboxSensor->RawPose());
+  const sdf::Camera *boundingboxCamSensor = boundingboxSensor->CameraSensor();
+  ASSERT_NE(nullptr, boundingboxCamSensor);
+  EXPECT_EQ("my_boundingbox_camera", boundingboxCamSensor->Name());
+  EXPECT_TRUE(boundingboxCamSensor->HasBoundingBoxType());
+  EXPECT_EQ("2d", boundingboxCamSensor->BoundingBoxType());
+
   // Get the force_torque sensor
   const sdf::Sensor *forceTorqueSensor =
     link->SensorByName("force_torque_sensor");
@@ -605,6 +653,40 @@ TEST(DOMLink, Sensors)
   EXPECT_DOUBLE_EQ(3.4, airSensor->PressureNoise().Mean());
   EXPECT_DOUBLE_EQ(5.6, airSensor->PressureNoise().StdDev());
   EXPECT_DOUBLE_EQ(123.4, airSensor->ReferenceAltitude());
+
+  // Get the wide angle camera sensor
+  EXPECT_TRUE(link->SensorNameExists("wide_angle_camera_sensor"));
+  const sdf::Sensor *wideAngleCameraSensor =
+      link->SensorByName("wide_angle_camera_sensor");
+  ASSERT_NE(nullptr, wideAngleCameraSensor);
+  EXPECT_EQ("wide_angle_camera_sensor", wideAngleCameraSensor->Name());
+  EXPECT_EQ(sdf::SensorType::WIDE_ANGLE_CAMERA, wideAngleCameraSensor->Type());
+  EXPECT_EQ(ignition::math::Pose3d(20, 30, 40, 0, 0, 0),
+      wideAngleCameraSensor->RawPose());
+  EXPECT_FALSE(wideAngleCameraSensor->EnableMetrics());
+  const sdf::Camera *wideAngleCam = wideAngleCameraSensor->CameraSensor();
+  ASSERT_NE(nullptr, wideAngleCam);
+  EXPECT_EQ("wide_angle_cam", wideAngleCam->Name());
+  EXPECT_EQ(ignition::math::Pose3d(0.2, 0.3, 0.4, 0, 0, 0),
+            wideAngleCam->RawPose());
+  EXPECT_DOUBLE_EQ(3.14, wideAngleCam->HorizontalFov().Radian());
+  EXPECT_EQ("wideanglecamera", wideAngleCameraSensor->Topic());
+  EXPECT_EQ("", wideAngleCam->TriggerTopic());
+  EXPECT_FALSE(wideAngleCam->Triggered());
+  EXPECT_EQ(320u, wideAngleCam->ImageWidth());
+  EXPECT_EQ(240u, wideAngleCam->ImageHeight());
+  EXPECT_EQ(sdf::PixelFormatType::RGB_INT8, wideAngleCam->PixelFormat());
+  EXPECT_DOUBLE_EQ(0.1, wideAngleCam->NearClip());
+  EXPECT_DOUBLE_EQ(100, wideAngleCam->FarClip());
+  EXPECT_EQ("custom", wideAngleCam->LensType());
+  EXPECT_TRUE(wideAngleCam->LensScaleToHfov());
+  EXPECT_DOUBLE_EQ(1.05, wideAngleCam->LensC1());
+  EXPECT_DOUBLE_EQ(4.0, wideAngleCam->LensC2());
+  EXPECT_DOUBLE_EQ(0.0, wideAngleCam->LensC3());
+  EXPECT_DOUBLE_EQ(1.0, wideAngleCam->LensFocalLength());
+  EXPECT_EQ("tan", wideAngleCam->LensFunction());
+  EXPECT_DOUBLE_EQ(3.1415, wideAngleCam->LensCutoffAngle().Radian());
+  EXPECT_EQ(512, wideAngleCam->LensEnvironmentTextureSize());
 }
 
 /////////////////////////////////////////////////
