@@ -31,6 +31,12 @@
 #include "test_config.h"
 #include "test_utils.hh"
 
+/////////////////////////////////////////////////
+std::string findFileCb(const std::string &_input)
+{
+  return sdf::testing::TestFile("integration", "model", _input);
+}
+
 //////////////////////////////////////////////////
 TEST(DOMModel, NotAModel)
 {
@@ -504,63 +510,6 @@ TEST(DOMRoot, LoadNestedCanonicalLink)
   EXPECT_EQ("deep::deeper::deepest::deepest_link", body);
 }
 
-/////////////////////////////////////////////////
-TEST(DOMModel, LoadModelWithDuplicateChildNames)
-{
-  // Redirect sdfwarn output
-  std::stringstream buffer;
-  sdf::testing::RedirectConsoleStream redir(
-      sdf::Console::Instance()->GetMsgStream(), &buffer);
-
-#ifdef _WIN32
-  sdf::Console::Instance()->SetQuiet(false);
-  sdf::testing::ScopeExit revertSetQuiet(
-      []
-      {
-        sdf::Console::Instance()->SetQuiet(true);
-      });
-#endif
-
-  {
-    buffer.str("");
-    const std::string testFile =
-      sdf::testing::TestFile("sdf", "model_link_joint_same_name.sdf");
-
-    // Load the SDF file
-    sdf::Root root;
-    auto errors = root.Load(testFile);
-    EXPECT_TRUE(errors.empty());
-    for (const auto &err : errors)
-    {
-      std::cout << err << std::endl;
-    }
-
-    // Check warning message
-    EXPECT_NE(
-        std::string::npos,
-        buffer.str().find("Non-unique name[attachment] detected 2 times in XML "
-                          "children of model with name[link_joint_same_name]"))
-        << buffer.str();
-  }
-
-  // Check that there's an exception for "plugin" elements
-  {
-    buffer.str("");
-    const std::string testFile =
-      sdf::testing::TestFile("sdf", "model_duplicate_plugins.sdf");
-
-    // Load the SDF file
-    sdf::Root root;
-    auto errors = root.Load(testFile);
-    EXPECT_TRUE(errors.empty());
-    for (const auto &err : errors)
-    {
-      std::cout << err << std::endl;
-    }
-    EXPECT_TRUE(buffer.str().empty()) << buffer.str();
-  }
-}
-
 /////////////////////////////////////////////////
 TEST(DOMRoot, LoadNestedExplicitCanonicalLink)
 {
@@ -653,3 +602,153 @@ TEST(DOMRoot, LoadInvalidNestedModelWithoutLinks)
   // errors[5]
   // errors[6]
 }
+
+/////////////////////////////////////////////////
+TEST(DOMModel, LoadModelWithDuplicateChildNames)
+{
+  // Redirect sdfwarn output
+  std::stringstream buffer;
+  sdf::testing::RedirectConsoleStream redir(
+      sdf::Console::Instance()->GetMsgStream(), &buffer);
+
+#ifdef _WIN32
+  sdf::Console::Instance()->SetQuiet(false);
+  sdf::testing::ScopeExit revertSetQuiet(
+      []
+      {
+        sdf::Console::Instance()->SetQuiet(true);
+      });
+#endif
+
+  {
+    buffer.str("");
+    const std::string testFile =
+      sdf::testing::TestFile("sdf", "model_link_joint_same_name.sdf");
+
+    // Load the SDF file
+    sdf::Root root;
+    auto errors = root.Load(testFile);
+    EXPECT_TRUE(errors.empty()) << errors;
+
+    // Check warning message
+    EXPECT_NE(
+        std::string::npos,
+        buffer.str().find("Non-unique name[attachment] detected 2 times in XML "
+                          "children of model with name[link_joint_same_name]"))
+        << buffer.str();
+  }
+
+  // Check that there's an exception for "plugin" elements
+  {
+    buffer.str("");
+    const std::string testFile =
+      sdf::testing::TestFile("sdf", "model_duplicate_plugins.sdf");
+
+    // Load the SDF file
+    sdf::Root root;
+    auto errors = root.Load(testFile);
+    EXPECT_TRUE(errors.empty()) << errors;
+    EXPECT_TRUE(buffer.str().empty()) << buffer.str();
+  }
+}
+
+//////////////////////////////////////////////////
+TEST(DOMModel, ModelPlugins)
+{
+  const std::string testFile =
+    sdf::testing::TestFile("sdf", "world_complete.sdf");
+
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile);
+  EXPECT_TRUE(errors.empty());
+  ASSERT_NE(nullptr, root.Element());
+  EXPECT_EQ(testFile, root.Element()->FilePath());
+
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_NE(nullptr, world);
+
+  const sdf::Model *model = world->ModelByIndex(0);
+  ASSERT_NE(nullptr, model);
+
+  ASSERT_EQ(2u, model->Plugins().size());
+  EXPECT_EQ("model_plugin1", model->Plugins()[0].Name());
+  EXPECT_EQ("test/file/model1", model->Plugins()[0].Filename());
+  EXPECT_EQ("model_plugin2", model->Plugins()[1].Name());
+  EXPECT_EQ("test/file/model2", model->Plugins()[1].Filename());
+}
+
+//////////////////////////////////////////////////
+TEST(DOMModel, IncludeModelWithPlugin)
+{
+  const std::string testFile =
+    sdf::testing::TestFile("sdf", "include_model_with_plugin.sdf");
+
+  sdf::ParserConfig config;
+  config.SetFindCallback(findFileCb);
+
+  sdf::Root root;
+  sdf::Errors errors = root.Load(testFile, config);
+  EXPECT_TRUE(errors.empty());
+  ASSERT_NE(nullptr, root.Element());
+  EXPECT_EQ(testFile, root.Element()->FilePath());
+
+  const sdf::World *world = root.WorldByIndex(0);
+  ASSERT_NE(nullptr, world);
+
+  const sdf::Model *model = world->ModelByIndex(0);
+  ASSERT_NE(nullptr, model);
+
+  sdf::OutputConfig outConfig;
+  // Test ToElement with useInclude == true
+  {
+    outConfig.SetToElementUseIncludeTag(true);
+    sdf::ElementPtr elem = model->ToElement(outConfig);
+
+    // There should be a uri
+    ASSERT_TRUE(elem->HasElement("uri"));
+    EXPECT_EQ("test_model_with_plugin", elem->Get<std::string>("uri"));
+
+    // There should be a plugin
+    ASSERT_TRUE(elem->HasElement("plugin"));
+    sdf::ElementPtr pluginElem = elem->GetElement("plugin");
+    EXPECT_EQ("test_model_include_plugin",
+        pluginElem->Get<std::string>("name"));
+
+    EXPECT_EQ("include/test/model/plugin",
+        pluginElem->Get<std::string>("filename"));
+
+    // There should be only one plugin
+    ASSERT_EQ(nullptr, pluginElem->GetNextElement("plugin"));
+  }
+
+  // Test ToElement with useInclude == false. This will result in the full
+  // model SDF which would have two plugins, one from the <include> tag and
+  // one from the included model
+  {
+    outConfig.SetToElementUseIncludeTag(false);
+    sdf::ElementPtr elem = model->ToElement(outConfig);
+
+    // There should NOT be a uri
+    ASSERT_FALSE(elem->HasElement("uri"));
+
+    // There should be a plugin
+    ASSERT_TRUE(elem->HasElement("plugin"));
+    sdf::ElementPtr pluginElem = elem->GetElement("plugin");
+    EXPECT_EQ("test_model_plugin",
+        pluginElem->Get<std::string>("name"));
+
+    EXPECT_EQ("test/model/plugin",
+        pluginElem->Get<std::string>("filename"));
+
+    // There should be a second plugin with different information
+    pluginElem = pluginElem->GetNextElement("plugin");
+    ASSERT_NE(nullptr, pluginElem);
+    EXPECT_EQ("test_model_include_plugin",
+        pluginElem->Get<std::string>("name"));
+    EXPECT_EQ("include/test/model/plugin",
+        pluginElem->Get<std::string>("filename"));
+
+    // THere should not be third plugin.
+    ASSERT_EQ(nullptr, pluginElem->GetNextElement("plugin"));
+  }
+}
